<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Cart Factory
 * 
 * @class IG_ES_Cart_Factory
 * 
 * @since 4.6.5
 */
class IG_ES_Cart_Factory extends IG_ES_Factory {

	public static $model = 'IG_ES_Cart';

	/**
	 * Method to get cart object by cart_id
	 * 
	 * @param int $cart_id
	 * 
	 * @return IG_ES_Cart|bool
	 */
	public static function get( $cart_id ) {
		return parent::get( $cart_id );
	}

	/**
	 * Method to get cart object by user_id
	 * 
	 * @param $user_id
	 * @return IG_ES_Cart|bool
	 */
	public static function get_by_user_id( $user_id ) {

		if ( ! $user_id ) {
			return false;
		}

		if ( ES_Cache::is_exists( $user_id, 'cart_user_id' ) ) {
			return static::get( ES_Cache::get( $user_id, 'cart_user_id' ) );
		}

		$cart = new IG_ES_Cart();
		$cart->get_by( 'user_id', $user_id );

		if ( ! $cart->exists ) {
			ES_Cache::set( $user_id, 0, 'cart_user_id' );
			return false;
		}

		return $cart;
	}

	/**
	 * Get cart by guest ID
	 * 
	 * @param $guest_id
	 * @return IG_ES_Cart|bool
	 */
	public static function get_by_guest_id( $guest_id ) {

		if ( ! $guest_id ) {
			return false;
		}

		if ( ES_Cache::is_exists( $guest_id, 'cart_guest_id' ) ) {
			return static::get( ES_Cache::get( $guest_id, 'cart_guest_id' ) );
		}

		$cart = new IG_ES_Cart();
		$cart->get_by( 'guest_id', $guest_id );

		if ( ! $cart->exists ) {
			ES_Cache::set( $guest_id, 0, 'cart_guest_id' );
			return false;
		}

		return $cart;
	}


	/**
	 * Method to get cart object by token
	 * 
	 * @param $token
	 * @return IG_ES_Cart|bool
	 */
	public static function get_by_token( $token ) {

		if ( ! $token ) {
			return false;
		}

		$cart = new IG_ES_Cart();
		$cart->get_by( 'token', $token );

		if ( ! $cart->exists ) {
			return false;
		}

		return $cart;
	}


	/**
	 * Method to update cart cache.
	 * 
	 * @param Cart $cart
	 */
	public static function update_cache( $cart ) {
		parent::update_cache( $cart );

		if ( $cart->get_user_id() ) {
			ES_Cache::set( $cart->get_user_id(), $cart->get_id(), 'cart_user_id' );
		}
	}

	/**
	 * Method to clean cart cache
	 * 
	 * @param Cart $cart
	 */
	public static function clean_cache( $cart ) {
		parent::clean_cache( $cart );

		static::clear_cached_prop( $cart, 'user_id', 'cart_user_id' );
	}

}
