<?php

/**
 * Helper class for IG_ES_Factory class
 * 
 * @class IG_ES_Factories
 */
class IG_ES_Factories {

	/** 
	 * Array of registered factory classes
	 *
	 * @var array
	 **/
	private static $factories;

	/**
	 * Get all registered factories
	 * 
	 * @return array
	 */
	public static function get_factories() {
		if ( ! isset( self::$factories ) ) {
			self::$factories = apply_filters( 'ig_es_factories', array(
				'cart' => 'IG_ES_Cart_Factory',
			));
		}
		return self::$factories;
	}


	/**
	 * Get registered factory class
	 * 
	 * @param $type
	 * @return bool|IG_ES_Factory
	 */
	public static function get_factory( $type ) {

		if ( ! $type ) {
			return false;
		}

		$factories = self::get_factories();

		return isset( $factories[$type] ) ? $factories[$type] : false;
	}


	/**
	 * Update object cache
	 * 
	 * @param ES_DB $object
	 */
	public static function update_object_cache( $object ) {
		$factory = self::get_factory( $object->object_type );
		if ( $factory ) {
			$factory::update_cache( $object );
		}
	}


	/**
	 * Clean object cache
	 * 
	 * @param ES_DB $object
	 */
	public static function clean_object_cache( $object ) {
		$factory = self::get_factory( $object->object_type );
		if ( $factory ) {
			$factory::clean_cache( $object );
		}
	}

}
