<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Factory class for getting guest data
 * 
 * @class IG_ES_Guest_Factory
 */
class IG_ES_Guest_Factory extends IG_ES_Factory {

	public static $model = 'IG_ES_Guest';

	/**
	 * Get guest by guest id
	 * 
	 * @param int $guest_id
	 * @return IG_ES_Guest|bool
	 */
	public static function get( $guest_id ) {
		return parent::get( $guest_id );
	}


	/**
	 * Get guest by email
	 * 
	 * @param $email
	 * @return IG_ES_Guest|bool
	 */
	public static function get_by_email( $email ) {

		if ( ! is_email( $email ) ) {
			return false;
		}

		if ( ES_Cache::is_exists( $email, 'guest_email' ) ) {
			return static::get( ES_Cache::get( $email, 'guest_email' ) );
		}

		$guest = new IG_ES_Guest();
		$guest->get_by( 'email', $email );

		if ( ! $guest->exists ) {
			ES_Cache::set( $email, 0, 'guest_email' );
			return false;
		}

		return $guest;
	}

	/**
	 * Get guest by key
	 * 
	 * @param $key
	 *
	 * @return IG_ES_Guest|bool
	 */
	public static function get_by_key( $key ) {

		if ( ! $key ) {
			return false;
		}

		$guest = new IG_ES_Guest();
		$guest->get_by( 'tracking_key', $key );

		if ( ! $guest->exists ) {
			return false;
		}

		return $guest;
	}

	/**
	 * Update cache
	 * 
	 * @param IG_ES_Guest $guest
	 */
	public static function update_cache( $guest ) {
		parent::update_cache( $guest );

		ES_Cache::set( $guest->get_email(), $guest->get_id(), 'guest_email' );
	}


	/**
	 * Clean cache
	 * 
	 * @param IG_ES_Guest $guest
	 */
	public static function clean_cache( $guest ) {
		parent::clean_cache( $guest );

		static::clear_cached_prop( $guest, 'email', 'guest_email' );
	}


	/**
	 * Save guest data
	 * 
	 * @param string $email
	 * @param string|bool $tracking_key
	 * @return IG_ES_Guest
	 */
	public static function create( $email, $tracking_key = false ) {

		global $woocommerce;

		if ( ! $tracking_key ) {

			// Can't look up the customer in this situation.
			if ( ! isset( $woocommerce->session ) ) {
				return false;
			}

			$tracking_key = $woocommerce->session->get_customer_id();
		}

		$guest = new IG_ES_Guest();
		$guest->set_email( ES_Clean::email( $email) );
		$guest->set_key( $tracking_key );
		$guest->set_date_created( new DateTime() );
		$guest->save();

		return $guest;
	}

}
