<?php
/**
 * Query cart based on auguements.
 *
 * @since       4.6.5
 * @version     1.0
 * @package     Email Subscribers
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to query cart based on arguements.
 *
 * @class Workflow_Query
 *
 * @since 4.6.5
 */
class IG_ES_WC_Cart_Query {

	/**
	 * Query arguements
	 *
	 * @var array
	 */
	public $args;

	/**
	 * Return result type
	 *
	 * @var string
	 */
	public $return = 'objects';

	/**
	 * Return result type
	 *
	 * @var string
	 */
	public $limit = 0;

	/**
	 * Construct
	 */
	public function __construct() {
		$this->args = array(
			'order'    => 'ASC',
			'order_by' => 'priority',
		);
	}

	/**
	 * Set status arguement
	 * 
	 * @param string|array $status active, abandoned
	 * @param $compare bool|string - defaults to '=' or 'IN' if array
	 * @return $this
	 */
	public function where_status( $status ) {
		$this->args['status'] = $status;
	}

	/**
	 * Set date modified arguement
	 * 
	 * @param string|DateTime $date
	 * @param $compare bool|string - defaults to '=' or 'IN' if array
	 * @return $this
	 */
	public function where_date_modified( $date, $compare = false ) {
		$this->args['last_modified'] = $date;
	}

	/**
	 * Set return object
	 *
	 * @param objects|ids $return Result format ids or objects.
	 */
	public function set_return( $return ) {
		$this->return = $return;
	}

	/**
	 * Set return object
	 *
	 * @param objects|ids $return Result format ids or objects.
	 */
	public function set_limit( $limit ) {
		$this->limit = $limit;
	}

	/**
	 * Get cart based on query arguements
	 *
	 * @return ES_Workflow[] $cart Workflow object
	 */
	public function get_results() {

		$this->args['fields'] = array();
		if ( 'ids' === $this->return ) {
			$this->args['fields'][] = 'id';
		}

		if ( $this->limit > 0 ) {
			$this->args['per_page'] = $this->limit;
		}

		$results = ES()->carts_db->get_carts( $this->args );

		if ( ! $results ) {
			return array();
		}

		$carts = array();

		foreach ( $results as $post ) {

			if ( 'ids' === $this->return ) {
				$carts[] = $post;
			} else {
				$workflow    = new IG_ES_Cart( $post );
				$carts[] = $workflow;
			}
		}

		return $carts;
	}
}
