<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Campaign_Admin_Pro' ) ) {
	/**
	 * The admin-specific functionality of the plugin.
	 *
	 * Admin Settings
	 *
	 * @package    Email_Subscribers
	 * @subpackage Email_Subscribers/admin
	 */
	class ES_Campaign_Admin_Pro {
	
		// class instance
		public static $instance;
	
		/**
		 * Campaign ID
		 */
		private $campaign_data = array();
	
		// class constructor
		public function __construct() {
			$this->init();
		}

		public static function get_instance() {
			if ( ! isset( self::$instance ) ) {
				self::$instance = new self();
			}
	
			return self::$instance;
		}
	
		public function init() {
			$this->register_hooks();
		}
	
		public function register_hooks() {
			add_action( 'ig_es_campaign_preview_tab_options', array( $this, 'add_campaign_preview_tab_options' ) );
			add_action( 'ig_es_campaign_preview_options_content', array( $this, 'show_campaign_preview_options_content' ) );
			add_action( 'ig_es_after_campaign_tracking_options_settings', array( $this, 'show_pro_tracking_options' ) );
			add_action( 'ig_es_' . IG_CAMPAIGN_TYPE_NEWSLETTER . '_scheduling_options_settings', array( &$this, 'show_broadcast_scheduling_options' ) );

			//add scheduler data
			add_filter( 'ig_es_' . IG_CAMPAIGN_TYPE_NEWSLETTER . '_data', array( &$this, 'add_broadcast_scheduler_data' ), 10, 2 );
		}

		/**
		 * Method to show left pan fields in campaign content section.
		 *
		 * @param array $campaign_data Broadcast data
		 *
		 * @since 4.4.7
		 *
		 */
		public function add_campaign_preview_tab_options( $campaign_data = array() ) {
			?>
			<div id="send-preview-email-tab" class="campaign-preview-option cursor-pointer text-sm font-normal text-gray-600" title="<?php echo esc_attr__( 'Send a test email', 'email-subscribers' ); ?>">
				<svg xmlns="http://www.w3.org/2000/svg" class="h-5 w-5" fill="none" viewBox="0 0 24 24" stroke="currentColor">
					<path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M3 8l7.89 5.26a2 2 0 002.22 0L21 8M5 19h14a2 2 0 002-2V7a2 2 0 00-2-2H5a2 2 0 00-2 2v10a2 2 0 002 2z" />
				</svg>
			</div>
			<?php
		}

		/**
		 * Method to show left pan fields in campaign content section.
		 *
		 * @param array $campaign_data Broadcast data
		 *
		 * @since 4.4.7
		 *
		 */
		public function show_campaign_preview_options_content( $campaign_data = array() ) {
			$type = isset( $campaign_data['type'] ) ? $campaign_data['type'] : 'campaign';
			?>
			<div id="campaign-email-preview-container" class="hidden">
				<p class="mx-4 mb-2">
					<?php echo esc_html__( 'Email address to receive preview email for testing', 'email-subscribers' ); ?>
				</p>
				<div class="p-3 bg-gray-100">
					<input id="ig_es_preview_email_address" placeholder="<?php echo esc_attr( 'Enter email address'); ?>" class="campaign-preview-option inline-block text-sm leading-5 border-gray-400 rounded-md shadow-sm form-input mr-2" name="ig_es_preview_email_address" autocomplete="email" style="min-width: 300px;">
					<button id="send-<?php echo esc_attr($type); ?>-preview-email-btn" type="button" class="ig-es-inline-loader rounded-md border text-indigo-600 border-indigo-500 text-sm leading-5 font-medium transition ease-in-out duration-150 select-none inline-flex justify-center hover:text-indigo-500 hover:border-indigo-600 hover:shadow-md focus:outline-none focus:shadow-outline-indigo focus:shadow-lg mt-1 px-4 py-2">
						<span><?php echo esc_html__( 'Send email', 'email-subscribers' ); ?></span>
						<svg class="es-btn-loader animate-spin h-4 w-4 text-indigo"
										xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24">
							<circle class="opacity-25" cx="12" cy="12" r="10" stroke="currentColor"
									stroke-width="4"></circle>
							<path class="opacity-75" fill="currentColor"
									d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path>
						</svg>
					</button>
					<div class="es-send-success es-icon mt-1" style="display:none;">
						<?php esc_html_e( 'Email sent successfully ', 'email-subscribers' ); ?>
					</div>
					<div class="es-send-error es-icon mt-1" style="display:none;">
						<?php esc_html_e( 'Something went wrong. Please try again later', 'email-subscribers' ); ?>
					</div>
				</div>
			</div>
			<?php
		}

		/**
		 * Method to show available tracking option in pro version
		 *
		 * @param array $campaign_data
		 *
		 * @return void
		 */
		public function show_pro_tracking_options( $campaign_data = array() ) {
			$enable_link_tracking = ! empty( $campaign_data['meta']['enable_link_tracking'] ) ? $campaign_data['meta']['enable_link_tracking'] : get_option( 'ig_es_track_link_click', 'yes' );

			?>

			<div class="flex mt-3 w-full">
				<div class="w-11/12 text-sm font-normal text-gray-600"><?php echo esc_html__( 'Link tracking', 'email-subscribers' ); ?>
				</div>

				<div>
					<label for="enable_link_tracking" class="inline-flex items-center cursor-pointer">
							<span class="relative">
								<input id="enable_link_tracking" name="campaign_data[meta][enable_link_tracking]"
									   type="checkbox" class="absolute es-check-toggle opacity-0 w-0 h-0"
									   value="yes" <?php checked( $enable_link_tracking, 'yes' ); ?>
								/>
								<span class="es-mail-toggle-line block w-8 h-5 bg-gray-300 rounded-full shadow-inner"></span>
								<span class="es-mail-toggle-dot absolute transition-all duration-300 ease-in-out block w-3 h-3 mt-1 ml-1 bg-white rounded-full shadow inset-y-0 left-0 focus-within:shadow-outline"></span>
							</span>
					</label>
				</div>
			</div>

			<?php
		}

		

		/**
		 * Method to show scheduling options
		 *
		 * @param array $campaign_data
		 *
		 * @return void
		 */
		public function show_broadcast_scheduling_options( $campaign_data = array() ) {
			//scheduler helper
			$timezone_format     = _x( 'Y-m-d h:i A', 'timezone date format' );
			$local_date          = '';
			$local_time          = '';
			$scheduled_date_time = ! empty( $campaign_data['meta']['date'] ) ? $campaign_data['meta']['date'] : '';
			if ( $scheduled_date_time ) {
				$schedule_date_timestamp = strtotime( $scheduled_date_time );
				$gmt_offset_option       = get_option( 'gmt_offset' );
				$gmt_offset              = ( ! empty( $gmt_offset_option ) ) ? $gmt_offset_option : 0;
				$locat_date_timestamp    = $schedule_date_timestamp + ( $gmt_offset * HOUR_IN_SECONDS );
				$local_date_time         = gmdate( 'Y-m-d H:i A', $locat_date_timestamp );
				$local_date_time_data    = explode( ' ', $local_date_time );
				$local_date              = $local_date_time_data[0];
				$local_time              = $local_date_time_data[1] . ' ' . $local_date_time_data[2];
			}
			$scheduling_option = ! empty( $campaign_data['meta']['scheduling_option'] ) ? $campaign_data['meta']['scheduling_option'] : 'schedule_now';
			?>
			<div class="block w-full px-4 pt-2 pb-8">
				<span class="border-t border-gray-200 pt-2 block text-sm font-medium leading-5 text-gray-700"><?php echo esc_html__( 'Send options', 'email-subscribers' ); ?></span>
				<div class="py-2">
					<input type="radio" name="campaign_data[scheduling_option]" class="form-radio" id="send_now"
						value="schedule_now" <?php checked( $scheduling_option, 'schedule_now' ); ?>>
					<label for="send_now"
						class="text-sm font-medium text-gray-500"><?php echo esc_html__( 'Send Now', 'email-subscribers' ); ?></label>
				</div>
				<div class="py-2">
					<input type="radio" name="campaign_data[scheduling_option]" class="form-radio" id="schedule_later"
						   value="schedule_later" <?php checked( $scheduling_option, 'schedule_later' ); ?>>
					<label for="schedule_later"
						   class="text-sm font-medium text-gray-500"><?php echo esc_html__( 'Schedule for later', 'email-subscribers' ); ?>
					</label>
					<br>
					<?php
					if ( ! empty( $scheduling_option ) && 'schedule_now' === $scheduling_option ) {
						$schedule_wrapper_class = 'hidden';
					} else {
						$schedule_wrapper_class = '';
					}
					?>
					<div id="schedule_later"
						 class="display_schedule px-6 <?php echo esc_attr( $schedule_wrapper_class ); ?>">
						<div class="flex pt-4">
							<div class="flex w-full w-11/12">
								<label class="text-sm font-normal leading-5 text-gray-600 pt-1"><?php echo esc_html__( 'Date', 'email-subscribers' ); ?></label>
								<input class="cursor-pointer font-normal text-sm py-1 ml-2 form-input" type="text"
									   id="es_schedule_date" name="campaign_data[es_schedule_date]"
									   value="<?php echo esc_attr( $local_date ); ?>">
							</div>
							<div>
								<svg fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round"
									 stroke-width="2" viewBox="0 0 24 24" class="text-gray-600 w-5 h-5 my-1 ml-2">
									<path d="M8 7V3m8 4V3m-9 8h10M5 21h14a2 2 0 002-2V7a2 2 0 00-2-2H5a2 2 0 00-2 2v12a2 2 0 002 2z"></path>
								</svg>
							</div>
						</div>
						<div class="flex pt-3">
							<div class="flex w-11/12">
								<label class="text-sm font-normal leading-5 text-gray-600 pt-1"><?php echo esc_html__( 'Time', 'email-subscribers' ); ?></label>
								<input class="es_schedule_time cursor-pointer font-normal text-sm px-3 py-1 ml-2"
									   type="text" id="es_schedule_time" name="campaign_data[es_schedule_time]"
									   value="<?php echo esc_attr( $local_time ); ?>">
							</div>
							<div>
								<svg fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round"
									 stroke-width="2" viewBox="0 0 24 24"
									 class="text-gray-600 w-5 h-5 my-1 ml-2 float-right">
									<path d="M12 8v4l3 3m6-3a9 9 0 11-18 0 9 9 0 0118 0z"></path>
								</svg>
							</div>
						</div>
						<div class=" pb-3">
							<div class="block px-2 py-2 mt-3 bg-gray-300 rounded-md ">
								<h3 class="text-gray-600 text-sm font-normal"><?php echo esc_html__( 'Local Time: ', 'email-subscribers' ); ?>
									&nbsp;&nbsp;
									<?php echo esc_html__( date_i18n( $timezone_format ) ); ?>
								</h3>
							</div>
						</div>
					</div>
				</div>
			</div>
			<?php
		}

		public function add_broadcast_scheduler_data( $data ) {

			$date              = ! empty( $data['es_schedule_date'] ) ? $data['es_schedule_date'] : '0000-00-00';
			$time              = ! empty( $data['es_schedule_time'] ) ? $data['es_schedule_time'] : '00:00:00';
			$scheduling_option = ! empty( $data['scheduling_option'] ) ? $data['scheduling_option'] : 'schedule_now';

			$schedule_str = '';

			if ( 'schedule_now' === $scheduling_option ) {
				// Get time without GMT offset, as we are adding later on.
				$schedule_str = time();
			} else {
				if ( '0000-00-00 00:00:00' !== $date . ' ' . $time ) {
					$schedule_str = strtotime( $date . ' ' . $time );
				}
			}

			if ( ! empty( $schedule_str ) ) {
				$gmt_offset_option = get_option( 'gmt_offset' );
				$gmt_offset        = ( ! empty( $gmt_offset_option ) ) ? $gmt_offset_option : 0;
				$schedule_date     = gmdate( 'Y-m-d H:i:s', $schedule_str - ( $gmt_offset * HOUR_IN_SECONDS ) );

				$data['start_at'] = $schedule_date;
				$meta             = ! empty( $data['meta'] ) ? maybe_unserialize( $data['meta'] ) : array();
				$meta['type']     = 'one_time';
				$meta['date']     = $schedule_date;
				$data['meta']     = maybe_serialize( $meta );
			}

			return $data;
		}
	
	}
}

ES_Campaign_Admin_Pro::get_instance();
