<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Pro_Bounce_Handler' ) ) {

	/**
	 * Class ES_Pro_Bounce_Handler
	 *
	 * @since 4.8.2
	 */
	class ES_Pro_Bounce_Handler extends ES_Services {
		/**
		 * Instance of the class
		 *
		 * @var null | ES_Pro_Bounce_Handler
		 */
		private static $_instance = null;

		/**
		 * Service command
		 *
		 * @var string
		 */
		public $cmd = '/email/bounce/:mailbox/:limit';

		protected $bounced_contacts_per_page = 500;

		/**
		 * ES_Pro_Bounce_Handler constructor.
		 */
		public function __construct() {
			parent::__construct();

			add_action( 'ig_es_plugin_deactivate', array( $this, 'clear_scheduled_bounce_handler' ) );
			if ( 'yes' === get_option( 'ig_es_enable_bounce_handling_feature', 'yes' ) ) {
				add_action( 'ig_es_plugin_activate', array( $this, 'schedule_bounce_handler' ) );
				//Change E-Mail headers
				add_filter( 'ig_es_mailer_email_headers', array( $this, 'email_headers' ), 10, 2 );
				add_filter( 'ig_es_smtp_mailer_email_headers', array( $this, 'smtp_email_headers' ), 10 );
				add_filter( 'ig_es_php_mailer_email_headers', array( $this, 'smtp_email_headers' ), 10 );
				//UI updater
				add_filter( 'ig_es_get_statuses_key_name_map', array( $this, 'get_statuses_key_name' ), 10, 2 );
				add_filter( 'ig_es_contact_column_subscriber', array( $this, 'contact_column_subscriber' ), 10, 4 );
				add_filter( 'ig_es_campaign_rules', array( $this, 'campaign_rules' ), 10 );
				add_action( 'ig_es_campaigns_extra_filters', array( $this, 'campaigns_extra_filters' ), 10, 7 );
				//Scheduler handler
				add_action( 'ig_es_bounce_handler_cron_action', array( $this, 'handle_cron_bounce_handler' ) );
				add_action( 'ig_es_bounce_handler_single_action', array( $this, 'handle_cron_bounce_handler' ) );
				//Showing WebHook URL
				add_action( 'ig_es_before_get_pepipost_doc_block', array( $this, 'display_pepipost_webhook_url' ) );
				add_action( 'ig_es_before_get_sendgrid_doc_block', array( $this, 'display_sendgrid_webhook_url' ) );
				add_action( 'ig_es_before_get_mailgun_doc_block', array( $this, 'display_mailgun_webhook_url' ) );
				add_action( 'ig_es_before_get_sparkpost_doc_block', array( $this, 'display_sparkpost_webhook_url' ) );
				add_action( 'ig_es_before_get_postmark_doc_block', array( $this, 'display_postmark_webhook_url' ) );
				add_action( 'ig_es_before_get_amazon_ses_doc_block', array( $this, 'display_amazon_ses_webhook_url' ) );
				add_action( 'ig_es_before_get_sendinblue_doc_block', array( $this, 'display_sendinblue_webhook_url' ) );
				add_action( 'ig_es_before_get_mailjet_doc_block', array( $this, 'display_mailjet_webhook_url' ) );
			}
		}

		/**
		 * Show extra filters in campaigns conditions modal window
		 *
		 * @param $input_name
		 * @param $value_arr
		 * @param $value
		 * @param $i
		 * @param $j
		 * @param $select_list_class
		 * @param $select_list_attr
		 */
		public function campaigns_extra_filters( $input_name, $value_arr, $value, $i, $j, $select_list_class, $select_list_attr ) {
			?>
			<div class="ig-es-conditions-value-field"
				 data-fields=",bounce_status,">
				<select name="<?php echo esc_attr( $input_name ); ?>[<?php echo esc_attr( $i ); ?>][<?php echo esc_attr( $j ); ?>][value][]"
						class="condition-value <?php echo esc_attr( $select_list_class ); ?>" <?php echo esc_attr( $select_list_attr ); ?>>
					<option value=""><?php echo esc_html__( 'Any status', 'email-subscribers' ); ?></option>
					<option value="1" <?php echo( in_array( '1', $value_arr ) ? 'selected="' . esc_attr( 'selected' ) . '"' : '' ); ?>><?php echo esc_html__( 'Soft bounced', 'email-subscribers' ); ?></option>
					<option value="2" <?php echo( in_array( '2', $value_arr ) ? 'selected="' . esc_attr( 'selected' ) . '"' : '' ); ?>><?php echo esc_html__( 'Hard bounced', 'email-subscribers' ); ?></option>
				</select>
			</div>
			<?php
		}

		/**
		 * Add some more rules in contacts filtering in campaigns
		 *
		 * @param array $rules
		 *
		 * @return array
		 */
		public function campaign_rules( $rules = array() ) {
			if ( is_array( $rules ) && array_key_exists( 'Subscriber', $rules ) ) {
				$rules['Subscriber']['bounce_status'] = array(
					'name' => esc_html__( 'Bounce status', 'email-subscribers' )
				);
			}

			return $rules;
		}

		/**
		 * Show filter in audience page
		 *
		 * @param $statuses
		 * @param $page String
		 *
		 * @return array|mixed
		 */
		public function get_statuses_key_name( $statuses, $page = '' ) {
			if ( is_array( $statuses ) && 'audience_listing_page' == $page ) {
				$statuses['soft_bounced'] = __( 'Soft Bounced', 'email-subscribers' );
				$statuses['hard_bounced'] = __( 'Hard Bounced', 'email-subscribers' );
			}

			return $statuses;
		}

		/**
		 * Get the WebHook URL
		 *
		 * @param $esp
		 *
		 * @return string
		 */
		public function get_webhook_url( $esp ) {
			$api_url = rtrim( $this->api_url, '/' );

			return "$api_url/email/bounce/$esp/{$this->get_return_path(true)}";
		}

		/**
		 * Display Mailjet webhook details
		 * @since 5.3.2
		 */
		public function display_mailjet_webhook_url() {
			$this->display_webhook_url_content( 'mailjet', 'mailjet', esc_html__( 'Mailjet Bounce notification URL', 'email-subscribers' ) );
		}

		/**
		 * Display Sendinblue webhook details
		 * @since 5.3.2
		 */
		public function display_sendinblue_webhook_url() {
			$this->display_webhook_url_content( 'sendinblue', 'sendinblue', esc_html__( 'Sendinblue Bounce notification URL', 'email-subscribers' ) );
		}

		/**
		 * Display Amazon SES webhook details
		 */
		public function display_amazon_ses_webhook_url() {
			$this->display_webhook_url_content( 'Amazon_SES', 'amazon_ses', esc_html__( 'Amazon SES Bounce notification URL', 'email-subscribers' ) );
		}

		/**
		 * Display PostMark webhook details
		 */
		public function display_postmark_webhook_url() {
			$this->display_webhook_url_content( 'postmark', 'postmark', esc_html__( 'PostMark Bounce WebHook URL', 'email-subscribers' ) );
		}

		/**
		 * Display SparkPost webhook details
		 */
		public function display_sparkpost_webhook_url() {
			$this->display_webhook_url_content( 'sparkpost', 'sparkpost', esc_html__( 'SparkPost Bounce WebHook URL', 'email-subscribers' ) );
		}

		/**
		 * Display MailGun webhook details
		 */
		public function display_mailgun_webhook_url() {
			$this->display_webhook_url_content( 'mailgun', 'mailgun', esc_html__( 'MailGun Bounce WebHook URL', 'email-subscribers' ) );
		}

		/**
		 * Display SendGrid webhook details
		 */
		public function display_sendgrid_webhook_url() {
			$this->display_webhook_url_content( 'sendgrid', 'sendgrid', esc_html__( 'SendGrid Bounce WebHook URL', 'email-subscribers' ) );
		}

		/**
		 * Display PepiPost webhook details
		 */
		public function display_pepipost_webhook_url() {
			$this->display_webhook_url_content( 'pepipost', 'pepipost', esc_html__( 'PepiPost Bounce WebHook URL', 'email-subscribers' ) );
		}

		/**
		 * Display webhook details
		 *
		 * @param $esp_attribute
		 * @param $esp
		 * @param $title
		 */
		public function display_webhook_url_content( $esp_attribute, $esp, $title ) {
			?>
			<div class="mb-4 es_sub_headline ig_es_<?php echo esc_attr__( $esp_attribute ); ?>_div_wrapper <?php echo esc_attr__( $esp_attribute ); ?>">
				<div class="pt-4" style=""><strong><?php echo esc_html__( $title ); ?></strong></div>
				<div class="form-input h-10 mt-2 mb-1 text-sm border-gray-400 w-3/5 bg-gray-100">
					<div class="flex">
						<div class="truncate">
							<?php
							echo esc_url( $this->get_webhook_url( $esp ) );
							?>
						</div>
						<div class="right-0 top-0 cursor-pointer" data-copyurl="
						<?php
						echo esc_url( $this->get_webhook_url( $esp ) );
						?>
						" data-copy="true">
							<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"
								 class="fill-current text-blue-500">
								<path d="M22 2v22h-20v-22h3c1.23 0 2.181-1.084 3-2h8c.82.916 1.771 2 3 2h3zm-11 1c0 .552.448 1 1 1 .553 0 1-.448 1-1s-.447-1-1-1c-.552 0-1 .448-1 1zm9 1h-4l-2 2h-3.897l-2.103-2h-4v18h16v-18zm-13 9.729l.855-.791c1 .484 1.635.852 2.76 1.654 2.113-2.399 3.511-3.616 6.106-5.231l.279.64c-2.141 1.869-3.709 3.949-5.967 7.999-1.393-1.64-2.322-2.686-4.033-4.271z"/>
							</svg>
						</div>
					</div>
				</div>
			</div>
			<?php
		}

		/**
		 * Alter contact column subscriber
		 *
		 * @param $title
		 * @param $item
		 *
		 * @return mixed|string
		 */
		public function contact_column_subscriber( $title, $item ) {
			$bounced_status = intval( $item['bounce_status'] );
			if ( 1 === $bounced_status ) {
				$title .= ' <span class="bounce-badge low">' . __( 'Soft bounced', 'email-subscribers' ) . '</span>';
			} else if ( 2 === $bounced_status ) {
				$title .= ' <span class="bounce-badge high">' . __( 'Hard bounced', 'email-subscribers' ) . '</span>';
			}

			return $title;
		}

		/**
		 * Generate the unique mailbox username
		 *
		 * @return mixed|void
		 */
		public static function get_bounce_mailbox_username() {
			$site_url  = get_site_url();
			$site_url  = preg_replace( '(^https?://)', '', $site_url );
			$prefix    = ES_Common::generate_random_string( 4 );
			$suffix    = ES_Common::generate_random_string( 4 );
			$unique_id = uniqid();
			$username  = "bounce_{$site_url}_{$prefix}_{$unique_id}_$suffix";
			$username  = str_replace( array( ':', '/', '-', '\\', '.' ), '_', $username );

			return apply_filters( 'ig_es_get_bounce_mailbox_username', $username );
		}

		/**
		 * Get the return path
		 *
		 * @return string
		 */
		public function get_return_path( $username_only = false ) {
			$mailbox_user = get_option( 'ig_es_bounce_mailbox_user', '' );
			if ( empty( $mailbox_user ) ) {
				$mailbox_user = self::get_bounce_mailbox_username();
				update_option( 'ig_es_bounce_mailbox_user', $mailbox_user );
			}
			if ( $username_only ) {
				$return_path = $mailbox_user;
			} else {
				$return_path = $mailbox_user . '@box.icegram.com';
			}

			return apply_filters( 'ig_es_bounce_handling_get_return_path', $return_path, $this );
		}

		/**
		 * Make a request to ETP server and mark contacts as bounced or not
		 */
		public function handle_cron_bounce_handler() {
			$mailbox   = $this->get_return_path();
			$this->cmd = str_replace( array( ':mailbox', ':limit' ), array(
				$mailbox,
				$this->bounced_contacts_per_page
			), $this->cmd );
			$response  = $this->send_request( array( '' ), 'GET' );
			if ( ! is_wp_error( $response ) ) {
				if ( isset( $response['data'] ) && is_array( $response['data'] ) ) {
					$bounced_contacts = $response['data'];
					if ( count( $bounced_contacts ) < $this->bounced_contacts_per_page ) {
						$this->update_contacts( $bounced_contacts );
					} else {
						$this->update_contacts( $bounced_contacts );
						$next_task_time = time() + MINUTE_IN_SECONDS; // Schedule next task after 1 minute from current time.
						IG_ES_Background_Process_Helper::add_action_scheduler_task( 'ig_es_bounce_handler_single_action', array(), false, false, $next_task_time );
					}
				}
			}
		}

		/**
		 * Mark contacts as bounced contacts
		 *
		 * @param $bounced_contacts
		 *
		 * @return bool
		 */
		public function update_contacts( $bounced_contacts ) {
			global $wpdb;
			if ( count( $bounced_contacts ) > 0 ) {
				$default_bounced_email_data = array(
					'bounced_email' => '',
					'bounce_type'   => 'soft',
					'campaign_id'   => 0,
					'message_id'    => 0,
				);
				foreach ( $bounced_contacts as $bounced_email ) {
					$details     = wp_parse_args( $bounced_email, $default_bounced_email_data );
					$contact     = sanitize_email( $details['bounced_email'] );
					$bounce_type = trim( $details['bounce_type'] );
					if ( ! empty( $contact ) && in_array( $bounce_type, array( 'soft', 'hard' ) ) ) {
						$wpdb->update( "{$wpdb->prefix}ig_contacts", array(
							'bounce_status' => 'soft' === $bounce_type ? 1 : 2,
							'updated_at'    => current_time( 'mysql', true )
						), array( 'email' => $contact ) );
					}
				}

				return true;
			} else {
				return false;
			}
		}

		/**
		 * Alter the Email header before sending mail
		 *
		 * @param $headers
		 * @param $data
		 *
		 * @return mixed
		 */
		public function email_headers( $headers, $data ) {
			$return_path = $this->get_return_path();
			if ( ! empty( $return_path ) ) {
				if ( count( $headers ) > 0 ) {
					foreach ( $headers as $index => $header ) {
						//Remove any return path found in the headers
						if ( preg_match( '/return-path/i', $header ) > 0 ) {
							unset( $headers[ $index ] );
						}
					}
				}
				$headers[] = "Return-Path: <{$return_path}>";
				$headers[] = 'X-Ig-Es-Campaign-ID: ' . ig_es_get_data( $data, 'campaign_id', 0 );
				$headers[] = 'X-Ig-Es-Message-ID: ' . ig_es_get_data( $data, 'message_id', 0 );
			}

			return $headers;
		}


		/**
		 * Alter the SMTP mailer Email header before sending mail
		 *
		 * @param $phpmailer
		 */
		public function smtp_email_headers( $phpmailer ) {
			if ( method_exists( $phpmailer, 'addCustomHeader' ) ) {
				$return_path = $this->get_return_path();
				if ( ! empty( $return_path ) ) {
					$phpmailer->addCustomHeader( 'Return-Path', '<' . $return_path . '>' );
					$phpmailer->addCustomHeader( 'X-Ig-Es-Campaign-ID', 0 );
					$phpmailer->addCustomHeader( 'X-Ig-Es-Message-ID', 0 );
				}
			}
		}

		/**
		 *  Scheduled bounce handler on plugin activation
		 *
		 * @return bool
		 */
		public function schedule_bounce_handler() {
			//Schedule an action if it's not already scheduled
			if ( ! wp_next_scheduled( 'ig_es_bounce_handler_cron_action' ) ) {
				$response = wp_schedule_event( time(), 'daily', 'ig_es_bounce_handler_cron_action', array(), true );
				if ( $response instanceof WP_Error ) {
					return false;
				}
			}

			return true;
		}

		/**
		 * Clear the scheduled bounce handler on plugin deactivation
		 */
		public function clear_scheduled_bounce_handler() {
			wp_clear_scheduled_hook( 'ig_es_bounce_handler_cron_action' );
		}

		/**
		 * Init the Bounce handler as singleton
		 *
		 * @return ES_Pro_Bounce_Handler|null
		 */
		public static function init() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}

			return self::$_instance;
		}

	}
}

ES_Pro_Bounce_Handler::init();
