<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

class ES_Custom_Fields_Table extends ES_List_Table {

	/**
	 * ES_DB_Custom_Fields object
	 *
	 * @since 4.8.4
	 * @var $db
	 */
	protected $db;

	/**
	 * Numbers of list options per page
	 *
	 * @since 4.8.4
	 * @var string
	 */
	public static $option_per_page = 'es_custom_fields_per_page';

	/**
	 * ES_Custom_Fields_Table constructor.
	 *
	 * @since 4.8.4
	 */
	public function __construct() {

		parent::__construct(
			array(
				'singular' => __( 'Custom Field', 'email-subscribers' ), // singular name of the listed records
				'plural'   => __( 'Custom Fields', 'email-subscribers' ), // plural name of the listed records
				'ajax'     => false, // does this table support ajax?,
				'screen'   => 'es_custom_fields',
			)
		);

		$this->db = new ES_DB_Custom_Fields();
	}

	/**
	 * Add Screen Option
	 *
	 * @since 4.8.4
	 */
	public static function screen_options() {

		$option = 'per_page';
		$args   = array(
			'label'   => __( 'Number of custom fields per page', 'email-subscribers' ),
			'default' => 20,
			'option'  => self::$option_per_page,
		);

		add_screen_option( $option, $args );

	}

	public function load_custom_fields() {	
		$action = ig_es_get_request_data( 'action' );
		?>
		<div class="wrap pt-4 font-sans">
			<?php
			if ( 'edit' === $action ) {
				$cf_id = ig_es_get_request_data( 'id' );

				$this->edit_custom_field( absint( $cf_id ) );
			} elseif ( 'new' === $action) {
				$this->es_create_new_custom_field();
			} else {
				?>
				<div class="max-w-full -mt-3 font-sans">
					<header class="wp-heading-inline">
						<div class="flex">
							<div>
								<nav class="text-gray-400 my-0" aria-label="Breadcrumb">
									<ol class="list-none p-0 inline-flex">
										<li class="flex items-center text-sm tracking-wide">
										<a class="hover:underline " href="admin.php?page=es_forms"><?php esc_html_e( 'Forms ', 'email-subscribers' ); ?></a>
										<svg class="fill-current w-2.5 h-2.5 mx-2 mt-mx" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M285.476 272.971L91.132 467.314c-9.373 9.373-24.569 9.373-33.941 0l-22.667-22.667c-9.357-9.357-9.375-24.522-.04-33.901L188.505 256 34.484 101.255c-9.335-9.379-9.317-24.544.04-33.901l22.667-22.667c9.373-9.373 24.569-9.373 33.941 0L285.475 239.03c9.373 9.372 9.373 24.568.001 33.941z"></path></svg>
										</li>
									</ol>
								</nav>
								<h2 class="-mt-1.5 text-2xl font-medium text-gray-700 sm:leading-7 sm:truncate"> <?php esc_html_e( 'Custom Fields', 'email-subscribers' ); ?>
								</h2>
							</div>
							<div class="mt-4"> <a href="admin.php?page=es_custom_fields&action=new" class="ig-es-title-button ml-2"><?php esc_html_e('Add New', 'email-subscribers'); ?></a>
							</div>
						</div>
					</header>
					<div><hr class="wp-header-end"></div>
					<div id="poststuff" class="es-items-lists es-lists-table">
						<div id="post-body" class="metabox-holder column-1">
							<div id="post-body-content">
								<div class="meta-box-sortables ui-sortable">
									<form method="get">
										<input type="hidden" name="page" value="es_custom_fields" />
										<?php
										// Display search field and other available filter fields.
										$this->prepare_items();
										?>
									</form>
									<form method="post">
										<?php
										// Display bulk action fields, pagination and list items.
										$this->display();
										?>
									</form>
								</div>
							</div>
						</div>
						<br class="clear">
					</div>
				</div>
			<?php
			}
	}

	public function es_create_new_custom_field() {

		global $wpbd;
		$submitted = ig_es_get_request_data( 'submitted' );

		if ( 'submitted' === $submitted ) {

			$nonce     		= ig_es_get_request_data( '_wpnonce' );
			$cf_data   		= ig_es_get_request_data( 'custom_field', array(), false );
			$cf_type   		= ! empty( $cf_data['type'] ) ? sanitize_text_field( $cf_data['type'] ) : '';
			$options_arr    = ! empty( $cf_data['values'] ) ? array_filter( explode( PHP_EOL, $cf_data['values'] ) , 'strlen' ) : array();
			$options   		= ! empty( $options_arr) ? array_map('trim', $options_arr ) : array();
			$cf_values 		= ! empty( $options && in_array( $cf_data['type'], array('dropdown', 'radio') ) ) ? maybe_serialize( array('values' => $options ) ) : '';

			$cf_slug   = '';
			$cf_label  = '';

			$validate_data = array(
				'nonce' 	=> $nonce,
				'slug' 		=> $cf_slug,
				'label' 	=> $cf_label,
				'type' 		=> $cf_type,
				'meta'		=> $options,
			);
			
			if ( ! empty( $cf_data['label'] ) ) {
				$cf_label    	 	 		 = sanitize_text_field( $cf_data['label'] );
				$slug      	     			 = str_replace( ' ', '', strtolower( $cf_label ) );
				$validate_data['label'] 	 = $cf_label;
				$cf_slug   	     			 = $this->get_slug_name_with_prefix( $validate_data );
				$existing_similar_slug_count = $wpbd->get_var( $wpbd->prepare( "SELECT count(*) FROM {$wpbd->prefix}ig_custom_fields WHERE slug LIKE %s ", $wpbd->esc_like( $cf_slug ) . '%'  )); 

				//Checking if a custom field slug exists. If yes, append slug with incremented number. 
				if ( $existing_similar_slug_count > 0 ) {
					$cf_slug = $cf_slug . $existing_similar_slug_count;
				}

				$validate_data['slug'] = $cf_slug;
				
			} 
			
			$response = $this->validate_data( $validate_data );

			if ( 'error' === $response['status'] ) {
				$message = $response['message'];
				ES_Common::show_message( $message, 'error' );
				$this->prepare_custom_field_form( null, $validate_data );

				return;
			}

			$data = array(
				'slug' 	 => $cf_slug,
				'label'  => $cf_label,
				'type' 	 => $cf_type,
				'meta'	 => $cf_values,
			);

			$cf_added			= $this->save_custom_field( null, $data );
			$cf_column_added 	= ES()->contacts_db->add_custom_field_col_in_contacts_table( $cf_slug, $cf_type );

			if ( $cf_added && $cf_column_added ) {
				$message = __( 'Custom field added successfully!', 'email-subscribers' );
				ES_Common::show_message( $message, 'success' );
			}
		}

		$this->prepare_custom_field_form();
	}

	/**
	 * Edit custom field
	 *
	 * @param $id
	 *
	 * @since 4.8.4
	 */
	public function edit_custom_field( $id ) {
		$submitted = ig_es_get_request_data( 'submitted' );
		
		if ( 'submitted' === $submitted ) {

			$nonce     		= ig_es_get_request_data( '_wpnonce' );
			$cf_data   		= ig_es_get_request_data( 'custom_field', array(), false );
			$cf_label  		= ! empty( $cf_data['label'] ) ? sanitize_text_field( $cf_data['label'] ) : '';
			$cf_type    	= ! empty( $cf_data['type'] ) ? sanitize_text_field( $cf_data['type'] ) : '';
			$options_arr    = ! empty( $cf_data['values'] ) ? array_filter( explode( PHP_EOL, $cf_data['values'] ) , 'strlen' ) : array();
			$options   		= ! empty( $options_arr) ? array_map('trim', $options_arr ) : array();
			$cf_values 		= ! empty( $options ) ? maybe_serialize( array('values' => $options ) ) : '';

			$validate_data = array(
				'nonce'  => $nonce,
				'label'  => $cf_label,
				'type' 	 => $cf_type,
				'meta'	 => $options,
			);

			$response = $this->validate_data( $validate_data );

			if ( 'error' === $response['status'] ) {
				$message = $response['message'];
				ES_Common::show_message( $message, 'error' );
				$this->prepare_custom_field_form( $id, $validate_data );

				return;
			}

			$data = array(
				'label'  => $cf_label,
				'type' 	 => $cf_type,
				'meta'	 => $cf_values,
			);


			$save = $this->save_custom_field( $id, $data );
			if ( $save ) {
				$message = __( 'Custom field updated successfully!', 'email-subscribers' );
				ES_Common::show_message( $message, 'success' );
			} 
		} else {
			$data = $this->db->get( $id );
		}

		$this->prepare_custom_field_form( $id, $data );

		?>

		<?php
	}

	/**
	 * Validate data
	 *
	 * @param $data
	 *
	 * @return array
	 */
	public function validate_data( $data ) {

		$nonce        = $data['nonce'];
		$cf_label     = $data['label'];
		$cf_type   	  = $data['type'];
		$cf_options   = $data['meta'];
		$action    = ig_es_get_request_data( 'action' );

		$status  = 'error';
		$error   = false;
		$message = '';

		if ( wp_verify_nonce( $nonce, 'es_custom_field' ) ) {
			
			if ( empty( $cf_type ) && 'edit' !== $action ) {
				$message = __( 'Please select custom field type', 'email-subscribers' );
				$error   = true;
			}

			if ( empty( $cf_label ) ) {
				$message = __( 'Please add custom field label', 'email-subscribers' );
				$error   = true;
			} 

			if ( ! empty( $cf_type ) && empty( $cf_options ) && in_array( $cf_type, array( 'radio', 'dropdown' ) ) ) {
				$message = __( 'Please enter options for selected custom field type', 'email-subscribers' );
				$error   = true;
			}
		} else {
			$message = __( 'You do not have permission to edit custom field', 'email-subscribers' );
			$error   = true;
		}

		if ( ! $error ) {
			$status = 'success';
		}

		$response = array(
			'status'  => $status,
			'message' => $message,
		);

		return $response;

	}

	public function prepare_custom_field_form( $id = 0, $data = array() ) {

		$is_new = empty( $id ) ? 1 : 0;
		$submitted = ig_es_get_request_data( 'submitted' );
		$action = 'new';
		$disable_field = '';

		if ( ! $is_new ) {
			$action = 'edit';
			$disable_field = 'es-custom-field-disabled';
		}

		$cf_slug = '';
		if ( isset( $data['slug'] ) ) {
			if ( 'submitted' === $submitted ) {
				$cf_slug = $data['slug'];
			} else {
				$cf_slug = ES_Common::get_slug_without_prefix( $data['slug'] );
			}
		}

		$cf_label 	= isset( $data['label'] ) ? $data['label'] : '';
		$cf_type  	= isset( $data['type'] ) ? $data['type'] : '';
		$cf_options = ! empty( $data['meta'] ) ? maybe_unserialize( $data['meta'] ) : '';
		$options 	= isset( $cf_options['values'] ) ? implode( PHP_EOL, $cf_options['values'] ) : '';
		$nonce 		= wp_create_nonce( 'es_custom_field' );

		?>

		<div class="max-w-full -mt-3 font-sans">
			<header class="wp-heading-inline">
				<div class="md:flex md:items-center md:justify-between justify-center">
					<div class="flex-1 min-w-0">
						<nav class="text-gray-400 my-0" aria-label="Breadcrumb">
							<ol class="list-none p-0 inline-flex">
								<li class="flex items-center text-sm tracking-wide">
								<a class="hover:underline " href="admin.php?page=es_forms"><?php esc_html_e( 'Forms ', 'email-subscribers' ); ?></a>
								<svg class="fill-current w-2.5 h-2.5 mx-2 mt-mx" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M285.476 272.971L91.132 467.314c-9.373 9.373-24.569 9.373-33.941 0l-22.667-22.667c-9.357-9.357-9.375-24.522-.04-33.901L188.505 256 34.484 101.255c-9.335-9.379-9.317-24.544.04-33.901l22.667-22.667c9.373-9.373 24.569-9.373 33.941 0L285.475 239.03c9.373 9.372 9.373 24.568.001 33.941z"></path></svg>

								<a class="hover:underline" href="admin.php?page=es_custom_fields&action=manage-custom-fields"><?php esc_html_e( ' Custom Fields ', 'email-subscribers'); ?></a> 
								 <svg class="fill-current w-2.5 h-2.5 mx-2 mt-mx" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M285.476 272.971L91.132 467.314c-9.373 9.373-24.569 9.373-33.941 0l-22.667-22.667c-9.357-9.357-9.375-24.522-.04-33.901L188.505 256 34.484 101.255c-9.335-9.379-9.317-24.544.04-33.901l22.667-22.667c9.373-9.373 24.569-9.373 33.941 0L285.475 239.03c9.373 9.372 9.373 24.568.001 33.941z"></path></svg>
								 </li>
							</ol>
						</nav>
						<h2 class="-mt-1 text-2xl font-medium text-gray-700 sm:leading-7 sm:truncate">
								<?php
								if ( $is_new ) {
									esc_html_e( 'Add new field', 'email-subscribers' );
								} else {
									esc_html_e( 'Edit custom field', 'email-subscribers' );
								}

								?>
						</h2>
					</div>
				</div>
			</header>
				<div><hr class="wp-header-end"></div>
				<div class="rounded max-w-full ">
					<div id="poststuff">
						<div id="post-body" class="metabox-holder column-1 mt-0.5">
							<div id="post-body-content">
								<div class="bg-white shadow-md rounded-lg mt-5">
									<form class="ml-5 mr-4 text-left pt-8 mt-2 item-center " method="post" action="admin.php?page=es_custom_fields&action=<?php echo esc_attr( $action ); ?>&id=<?php echo esc_attr( $id ); ?>&_wpnonce=<?php echo esc_attr( $nonce ); ?>">

										<div class="flex flex-row border-b border-gray-100">
											<div class="flex w-1/5">
												<div class="ml-4 pt-6 px-3	">
													<label for="name" class="block text-sm leading-5 font-medium text-gray-600"><?php esc_html_e( 'Label', 'email-subscribers' ); ?></label>
												</div>
											</div>
											<div class="flex">
												<div class="ml-16 mb-4 h-10 mr-4 mt-4">
													<div class="h-10 relative">
														<input class="form-input block border-gray-400 w-full pl-3 pr-12 focus:bg-gray-100 sm:text-sm sm:leading-5" placeholder="<?php echo esc_html__( 'Enter custom field label', 'email-subscribers' ); ?>" id="es_cf_label" name="custom_field[label]" value="<?php echo esc_attr( $cf_label ); ?>"/>
													</div>
												</div>
											</div>
										</div>
										
										<div class="flex flex-row">
											<div class="flex w-1/5">
												<div class="ml-4 pt-6 px-3	">
													<label for="name" class="block text-sm leading-5 font-medium text-gray-600"><?php esc_html_e( 'Type', 'email-subscribers' ); ?></label>
												</div>
											</div>
											<div class="flex">
												<div class="ml-16 mb-4 h-10 mr-4 mt-4 es_custom_field_type">
													<div class="h-10 relative">
														<select class="relative form-select shadow-sm border border-gray-400 <?php echo esc_html( $disable_field ); ?>" name="custom_field[type]" id="es_cf_type">
															<?php
																$allowedtags = ig_es_allowed_html_tags_in_esc();
																$custom_field_types = ES_Common::prepare_fieldtype_dropdown_options( $cf_type );
																echo wp_kses( $custom_field_types, $allowedtags );
															?>
														</select>
													</div>
												</div>
											</div>
										</div>

										<div class="flex flex-row border-b border-gray-100 es_custom_field_options" style="display: none">
											<div class="flex w-1/5">
											</div>
											<div class="flex">
												<div class="ml-16 mb-4 mr-4 es_custom_field_type">
													<div class="relative">
														<textarea class="form-textarea text-sm" rows="4" cols="50" name="custom_field[values]" id="es_custom_field_options"><?php echo esc_html( $options ); ?></textarea>
														<p class="italic text-xs text-gray-400 mt-2 leading-snug pb-4"><?php esc_html_e( 'Enter the different options (one per line) that you want to show in the form.', 'email-subscribers' ); ?></p>
													</div>
												</div>
											</div>
										</div>
										
										<?php
										
										$submit_button_text = $is_new ? __( 'Save Custom Field', 'email-subscribers' ) : __( 'Save Changes', 'email-subscribers' );
										?>
										<!-- <input type="hidden" class="es-customfield-key" id="es_cf_slug" name="custom_field[slug]" value="<?php echo esc_attr( $cf_slug ); ?>" /> -->
										<input type="hidden" name="submitted" value="submitted"/>
										<input type="submit" name="submit" id="submit" class="cursor-pointer align-middle ig-es-primary-button px-4 py-2 my-4 ml-6 mr-2" value="<?php echo esc_attr( $submit_button_text ); ?>"/>
										<a href="admin.php?page=es_custom_fields&action=manage-custom-fields" class="cursor-pointer align-middle rounded-md border border-indigo-600 hover:shadow-md focus:outline-none focus:shadow-outline-indigo text-sm leading-5 font-medium transition ease-in-out duration-150 px-4 my-2 py-2 mx-2 "><?php esc_html_e( 'Cancel', 'email-subscribers' ); ?></a>
									</form>
								</div>
							</div>
						</div>

						<br class="clear">
					</div>

				</div>

				<?php

	}

	/**
	 *  Associative array of columns
	 *
	 * @return array
	 */
	public function get_columns() {

		$allowedtags  = ig_es_allowed_html_tags_in_esc();
		
		$columns = array(
			//'cb'           => '<input type="checkbox" />',
			'label'   	   => __( 'Label', 'email-subscribers' ),
			'type' 		   => __( 'Type', 'email-subscribers' ),
		);

		return apply_filters( 'ig_es_lists_columns', $columns );
	}

	/**
	 * Save custom field
	 *
	 * @param $id
	 * @param $data
	 *
	 * @return bool|int|void
	 *
	 * @since 4.8.4
	 */
	public function save_custom_field( $id, $data ) {

		if ( ! empty( $id ) ) {
			$return = $this->db->update_custom_field( $id, $data );
		} else {
			$return = $this->db->add_custom_field( $data );
		}
		return $return;
	}

	/**
	 * Save custom field
	 *
	 * @param $type
	 *
	 * @return string
	 *
	 * @since 4.8.4
	 */
	public function get_slug_name_with_prefix( $data ) {
	
		if ( ! empty( $data ) ) {
			if ( isset( $data['type'] ) && isset( $data['label'] ) ) {
				$col_type = $this->get_col_type_from_field_type( $data['type'] );
				$label = strtolower( str_replace(' ', '', $data['label'] ) );
				return 'cf_' . $col_type . '_' . $label;
			}	
		}
		
		return '';
	}

	/**
	 * Get column type
	 *
	 * @param $type
	 *
	 * @return string
	 *
	 * @since 4.8.4
	 */
	public function get_col_type_from_field_type( $type = '' ) {
	
		switch ( $type ) {
			case 'date':
				return 'date';
			case 'number':
				return 'int';
			case 'radio':
			case 'dropdown':
				return 'varchar';
			default:
				return 'longtext';
		}
		
	}

	/**
	 * Render a column when no column specific method exist.
	 *
	 * @param array  $item
	 * @param string $column_name
	 *
	 * @return mixed
	 */
	public function column_default( $item, $column_name ) {

		switch ( $column_name ) {
			case 'type':
				return ucfirst( $item[ $column_name ] );
			default:
				return $item[ $column_name ];
		}
	}

	/**
	 * Method for Label column
	 *
	 * @param array $item an array of DB data
	 *
	 * @return string
	 */
	public function column_label( $item ) {

		$nonce = wp_create_nonce( 'es_custom_field' );
		$title   = '<strong>' . $item['label'] . '</strong>';
		$actions = array();
		
		$page    = ig_es_get_request_data( 'page' );
		$actions = array(
			'edit'   => '<a href="?page=' . esc_attr( $page ) . '&action=edit&id=' . absint( $item['id'] ) . '&_wpnonce=' . $nonce . '" class="text-indigo-600">' . esc_html__( 'Edit', 'email-subscribers' ) . '</a>',

			'delete' => '<a href="?page=' . esc_attr( $page ) . '&action=delete&id=' . absint( $item['id'] ) . '&_wpnonce=' . $nonce . '" onclick="return checkDelete()">' . esc_html__( 'Delete', 'email-subscribers' ) . '</a>',
		);
		

		return $title . $this->row_actions( $actions );
	}

	/**
	 * Handles data query and filter, sorting, and pagination.
	 */
	public function prepare_items() {

		$this->_column_headers = $this->get_column_info();

		/** Process bulk action */
		$this->process_bulk_action();

		$per_page     = $this->get_items_per_page( self::$option_per_page, 25 );
		$current_page = $this->get_pagenum();
		$total_items  = $this->get_custom_fields( 0, 0, true );

		$this->set_pagination_args(
			array(
				'total_items' => $total_items, // WE have to calculate the total number of items.
				'per_page'    => $per_page, // WE have to determine how many items to show on a page.
			)
		);

		$this->items = $this->get_custom_fields( $per_page, $current_page );
	}

	public function process_bulk_action() {

		// Detect when a bulk action is being triggered...
		if ( 'edit' === $this->current_action() ) {

			// In our file that handles the request, verify the nonce.
			$nonce = esc_attr( ig_es_get_request_data( '_wpnonce' ) );

			if ( ! wp_verify_nonce( $nonce, 'es_custom_field' ) ) {
				$message = __( 'You do not have permission to edit custom field', 'email-subscribers' );
				ES_Common::show_message( $message, 'error' );
			} else {
				$this->edit_custom_field( absint( ig_es_get_request_data( 'id' ) ) );
				$message = __( 'Custom field updated successfully!', 'email-subscribers' );
				ES_Common::show_message( $message, 'success' );
			}
		}

		if ( 'delete' === $this->current_action() ) {
			$message = '';
			// In our file that handles the request, verify the nonce.
			$nonce = esc_attr( ig_es_get_request_data( '_wpnonce' ) );

			if ( ! wp_verify_nonce( $nonce, 'es_custom_field' ) ) {
				$message = __( 'You do not have permission to delete custom field', 'email-subscribers' );
				ES_Common::show_message( $message, 'error' );
			
			} else {
				$cf_id = absint( ig_es_get_request_data( 'id' ) );
				
				$col_deleted   = ES()->contacts_db->delete_col_by_custom_field_id( array( $cf_id ) );
				$field_deleted = $this->db->delete_custom_fields( array( $cf_id ) );

				if ( $col_deleted && $field_deleted ) {
					$message = __( 'Custom field deleted successfully!', 'email-subscribers' );
				} else {
					$message = __( 'Oops...There was an issue while deleting the custom fields', 'email-subscribers' );
				}
				ES_Common::show_message( $message, 'success' );
				
			}
		}

		// $action  = ig_es_get_request_data( 'action' );
		// $action2 = ig_es_get_request_data( 'action2' );
		// // If the delete bulk action is triggered
		// if ( ( 'bulk_delete' === $action ) || ( 'bulk_delete' === $action2 ) ) {

		// 	$custom_fields = ig_es_get_request_data( 'custom_fields' );

		// 	if ( ! empty( $custom_fields ) > 0 ) {

		// 		$this->db->delete_custom_fields( $custom_fields );
		// 		$message = __( 'Custom field(s) deleted successfully!', 'email-subscribers' );
		// 		ES_Common::show_message( $message, 'success' );
		// 	} else {
		// 		$message = __( 'Please select custom field', 'email-subscribers' );
		// 		ES_Common::show_message( $message, 'error' );

		// 		return;
		// 	}
		// }
	}

	/**
	 * Retrieve custom fields data from the database
	 *
	 * @param int $per_page
	 * @param int $page_number
	 *
	 * @return mixed
	 */
	public function get_custom_fields( $per_page = 5, $page_number = 1, $do_count_only = false ) {
		global $wpbd;

		$order_by = sanitize_sql_orderby( ig_es_get_request_data( 'orderby' ) );
		$order    = ig_es_get_request_data( 'order' );

		if ( $do_count_only ) {
			$sql = 'SELECT count(*) as total FROM ' . IG_CUSTOM_FIELDS_TABLE;
		} else {
			$sql = 'SELECT * FROM ' . IG_CUSTOM_FIELDS_TABLE;
		}

		$args  = array(); 
		$query = array();

		$add_where_clause = false;

		if ( ! $do_count_only ) {

			// Prepare Order by clause
			$order                 = ! empty( $order ) ? strtolower( $order ) : 'desc';
			$expected_order_values = array( 'asc', 'desc' );
			if ( ! in_array( $order, $expected_order_values ) ) {
				$order = 'desc';
			}

			$default_order_by = esc_sql( 'label' );
			$expected_order_by_values = array( 'label', 'Type' );

			if ( ! in_array( $order_by, $expected_order_by_values ) ) {
				$order_by_clause = " ORDER BY {$default_order_by} DESC";
			} else {
				$order_by        = esc_sql( $order_by );
				$order_by_clause = " ORDER BY {$order_by} {$order}, {$default_order_by} DESC";
			}

			$sql .= $order_by_clause;
			$sql .= " LIMIT $per_page";
			$sql .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;

			$result = $wpbd->get_results( $sql, 'ARRAY_A' );

		} else {
			$result = $wpbd->get_var( $sql );
		}

		return $result;
	}

	/** Text displayed when no list data is available */
	public function no_items() {
		esc_html_e( 'No custom fields available.', 'email-subscribers' );
	}

}
	
