<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Pro_Custom_Fields' ) ) {

	/**
	 * Class ES_Pro_Custom_Fields
	 *
	 * @since 4.8.4
	 */
	class ES_Pro_Custom_Fields {

		/**
		 * ES_Pro_Custom_Fields constructor.
		 */
		public function __construct() {


			add_filter( 'ig_es_add_custom_field_data', array( &$this, 'es_add_subscriber_custom_field_data' ), 10, 2 );

			add_action( 'ig_es_after_form_buttons', array( &$this, 'add_custom_fields_button' ) );
			
			add_action( 'ig_es_additional_form_fields', array( &$this, 'es_additional_form_fields' ), 10, 2 );
			add_action( 'es_add_custom_fields_data_in_form_body', array( &$this, 'add_custom_fields_data_in_form_body' ), 10, 2 );
			add_filter( 'ig_es_after_form_fields', array( &$this, 'es_embed_custom_fields_in_form' ), 10, 2 );
			add_filter( 'es_import_show_more_fields_for_mapping', array( &$this, 'show_more_fields_in_import_for_mapping' ), 10, 1 );
			add_filter( 'es_prepare_additional_contacts_data_for_import', array( &$this, 'prepare_additional_contacts_data_for_import' ), 10, 2 );
			add_action( 'es_show_additional_contacts_data', array( &$this, 'show_additional_contacts_data' ), 10, 2 );
			add_filter( 'es_prepare_cf_data_for_contact_array', array( &$this, 'prepare_cf_data_for_contact_array' ), 10, 2 );
			add_filter( 'es_set_additional_contact_data', array( &$this, 'set_additional_contact_data' ), 10, 2 );
		}

		/**
		 * Sanitize and add submitted custom field data of subscriber.
		 *
		 * @param array $data 
		 * @param array $form_data 
		 * 
		 * @since 4.8.4
		 */
		public function es_add_subscriber_custom_field_data( $data, $form_data = array() ) {

			if ( isset( $form_data['es_custom_field'] ) ) {
				foreach ( $form_data['es_custom_field'] as $cf_slug => $cf_value ) {
					$data[$cf_slug] = ! empty( $cf_value ) ? sanitize_text_field( $cf_value ) : '';
				}
			}
			return $data;
		}
		
		
		/**
		 * Add Manage Custom fields button on Forms screen
		 * 
		 * @since 4.8.4
		 */
		public function add_custom_fields_button() {
			?>
			<a href="admin.php?page=es_custom_fields&action=manage-custom-fields" class="ig-es-imp-button ig-es-title-button ml-2 leading-5 align-middle">
				<?php esc_html_e( 'Manage Custom Fields', 'email-subscribers' ); ?>
			</a>
		<?php
		}


		/**
		 * Show custom fields in forms
		 *
		 * @param array $form_data 
		 * 
		 * @since 4.8.4
		 */
		public function es_additional_form_fields( $form_data = array(), $data = array() ) {
			
			$custom_fields_data = ES()->custom_fields_db->get_custom_fields();

			if ( isset( $data['custom_fields' ] ) ) {
				$form_data['custom_fields'] = $data['custom_fields'];
			} else {
				//If it is an existing old form we are adding custom_fields data from custom field table to the existing form_data
				$form_data['custom_fields'] = $custom_fields_data;				
			}

			if ( count( $custom_fields_data ) > 0 ) {
				foreach ( $custom_fields_data as $field ) {
					
					//If a new custom field is created, we are merging default data of that custom field to the existing form array
					$search_index = array_search( $field['slug'], array_column( $form_data['custom_fields'], 'slug' ) );
					if ( false !== $search_index) {
						$field = $form_data['custom_fields'][ $search_index ];
					}

					$slug 				= isset( $field['slug'] ) ? $field['slug'] : '';
					$field_label 		= isset( $field['label'] ) ? $field['label'] : '';
					$field_placeholder 	= isset( $field['placeholder'] ) ? $field['placeholder'] : '';
					$field_show 		= ( isset( $field['params']['show'] ) && true === $field['params']['show'] ) ? 'yes' : 'no';
					$field_required 	= ( isset( $field['params']['required'] ) && true === $field['params']['required'] ) ? 'yes' : 'no';

					?>
					
					<tr class="form-field es-form-custom-field">
						<td class="pr-6 pb-6"><b class="text-gray-500 text-sm font-normal pb-2"><?php echo esc_html( $field_label ); ?></b></td>
						<td class="pr-6 pb-6">
							<input type="checkbox" class="form-checkbox es-form-show-custom-field" name="form_data[custom_field][<?php echo esc_html( $slug ); ?>][show]" value="yes" 
							<?php 
							if ( 'yes' === $field_show ) {
								echo 'checked="checked"';
							} 
							?>
								>
						</td>
						<td class="pr-6 pb-6">
							<input type="checkbox" class="form-checkbox es-form-custom-field-required" name="form_data[custom_field][<?php echo esc_html( $slug ); ?>][required]" value="yes" 
							<?php 
							if ( 'yes' === $field_required ) {
								echo 'checked="checked"';
							} 
							?>
							>
							
						</td>
						<td class="pr-6 pb-8">	
							<input class="form-input block border-gray-400 w-5/6 pr-12 h-8 shadow-sm focus:bg-gray-100 sm:text-sm sm:leading-5" name="form_data[custom_field][<?php echo esc_html( $slug ); ?>][label]" value="<?php echo esc_attr( $field_label ); ?>" disabled>
						</td>
						<?php $allowed_placeholder = array( 'text', 'textarea', 'number' ); ?>
						<td class="pr-6 pb-8">
							<input class="form-input block border-gray-400 w-5/6 pr-12 h-8 shadow-sm  focus:bg-gray-100 sm:text-sm sm:leading-5" name="form_data[custom_field][<?php echo esc_html( $slug ); ?>][placeholder]" value="<?php echo esc_attr( $field_placeholder ); ?>" 
							<?php
							if ( ! empty( $field['type'] ) && ! in_array( $field['type'], $allowed_placeholder ) ) {
								echo ' disabled';
							} 
							?>
							>
						</td>
					</tr>
				<?php
				}
			}	
		}

		/**
		 * Add custom field data in form tables body column
		 *
		 * @param array $form_data 
		 * 
		 * @since 4.8.4
		 */
		public function add_custom_fields_data_in_form_body( $body = array(), $data = array() ) {
			
			$custom_fields 		= ES()->custom_fields_db->get_custom_fields();
			$custom_field_data  = array();
			$merged_fields 		= array();
			$field_position  	= count( $body ) + 1;
	
			if ( count( $custom_fields ) > 0 ) {
				foreach ( $custom_fields as $field ) {
					$slug 		 = isset( $field['slug'] ) ? $field['slug'] : '';
					$label 		 = isset( $field['label'] ) ? $field['label'] : '';
					$placeholder = isset( $data['custom_field'][$slug]['placeholder']) ? $data['custom_field'][$slug]['placeholder'] : '';

					$custom_field_data = array(
						array(
							'is_custom_field' => 1,
							'type'  		  => isset( $field['type'] ) ? $field['type'] : '',
							'label'  		  => $label,
							'name'   		  => $label,
							'id'     		  => $slug,
							'slug'     		  => $slug,
							'placeholder'     => $placeholder,
							'params' 		  => array(
								'label'    => $label,
								'show'     => ( isset( $data['custom_field'][$slug]['show'] ) && ! empty( $data['custom_field'][$slug]['show'] ) ) ? true : false,
								'required' => ( isset( $data['custom_field'][$slug]['required'] ) && ! empty( $data['custom_field'][$slug]['required'] ) ) ? true : false,
							),
							'position' => $field_position,
						),
					);
					++$field_position;
					$merged_fields = array_merge( $merged_fields, $custom_field_data );
				}
			}	

			$form_data = array_merge( $body, $merged_fields);
			return $form_data;
		}

		/**
		 * Embed custom fields in front-end form
		 *
		 * @param array $data 
		 * 
		 * @since 4.8.4
		 */
		public function es_embed_custom_fields_in_form( $form_orig_html = '', $data = array() ) {
			
			$custom_fields_data = ES()->custom_fields_db->get_custom_fields();
			$submitted_cf_data  = ig_es_get_post_data( 'custom_field' );
			
			$form_html = '';

			if ( isset( $data['custom_fields' ] ) ) {
				foreach ( $data['custom_fields'] as $field ) {
					$slug 			 = ! empty( $field['slug'] ) ? $field['slug'] : '';
					$label 			 = ! empty( $field['label'] ) ? $field['label'] : '';
					$placeholder 	 = ! empty( $field['placeholder'] ) ? $field['placeholder'] : '';
					$required 		 = ! empty( $field['params']['required'] ) ? 'required' : '';
					$value 			 = ! empty( $field['params']['required'] ) ? $field['params']['required'] : '';
					$submitted_value = isset( $submitted_cf_data[ $slug ] ) ? $submitted_cf_data[ $slug ] : '';

					//Check if custom field exists
					$cf_slug_list = array_column( $custom_fields_data, 'slug', 'slug' );
					if ( isset( $cf_slug_list[ $field['slug'] ] ) ) {
						if ( ! empty( $field['params']['show'] ) && $field['params']['show'] ) {

							$meta  		= ES()->custom_fields_db->get_custom_field_meta_by_slug( $slug );
							$class_name = 'ig_es_' . $slug;
							$form_html .= '<div class="es-field-wrap"><label>' . $label . '<br/>';

							switch ( $field['type'] ) {
								case 'text':
									$form_html .= '<input type="text" name="es_custom_field[' . $slug . ']" class="es_form_cf ' . $class_name . '"  placeholder="' . $placeholder . '"';

									$form_html .= 'required' === $required ? ' required = "' . $required . '"' : '';
									$form_html .= ' value="' . $submitted_value . '"/></label></div>';
									break;
								case 'textarea':
									$form_html .= '<textarea class="form-textarea text-sm" cols="50" name="es_custom_field[' . $slug . ']" class="es_form_cf ' . $class_name . '" placeholder="' . $placeholder . '"';
									
									$form_html .= 'required' === $required ? ' required = "' . $required . '"' : '';
									$form_html .= '>' . $submitted_value . '</textarea></label></div>';
									break;
								case 'dropdown':
									$form_html .= '<select name="es_custom_field[' . $slug . ']" class="es_form_cf ' . $class_name . '"';

									$form_html .= 'required' === $required ? ' required = "' . $required . '">' : '>';
									$form_html .= '<option value="">' . esc_html__('Select option', 'icegram-marketing-automation') . '</option>';
									if ( isset( $meta['values'] ) ) {
										foreach ($meta['values'] as $key) {
											$form_html .= '<option value="' . $key . '">' . $key . '</option>';
										}
									}	
									$form_html .= '</select></label></div>';
									break;
								case 'radio':
								$form_html .= '<table class="ig-es-form-radio-selection"><tbody>';
									if ( isset( $meta['values'] ) ) {
										foreach ($meta['values'] as $key) {
											$form_html .= '<tr><td><input type="radio" name="es_custom_field[' . $slug . ']" class="es_form_cf ' . $class_name . '" placeholder="' . $placeholder . '"';
											$form_html .= 'required' === $required ? ' required = "' . $required . '"' : '';
											$form_html .= ' value="' . $key . '">';
											$form_html .=  $key . '</td></tr>';
										}
									}
									$form_html .= '</tbody></table></label></div>';
									break;
								case 'number':
									$form_html .= '<input type="number" name="es_custom_field[' . $slug . ']" class="es_form_cf ' . $class_name . '" placeholder="' . $placeholder . '"' ;

									$form_html .= 'required' === $required ? ' required = "' . $required . '"' : '';
									$form_html .= ' value="' . $submitted_value . '"/>' . $submitted_value . '</label></div>';
									break;
								case 'date':
									$form_html .= '<input type="date" name="es_custom_field[' . $slug . ']" class="es_form_cf ' . $class_name . '" placeholder="' . $placeholder . '"' ;

									$form_html .= 'required' === $required ? ' required = "' . $required . '"' : '';
									$form_html .= ' value="' . $submitted_value . '"/>' . $submitted_value . '</label></div>';
									break;
								default:
									break;
							}
						}
					}
					
				}
			}	

			$form = $form_orig_html . '' . $form_html;
			return $form;			
		}

		/**
		 * Show column name for Mapping in Import
		 *
		 * @param array $fields
		 * 
		 * @since 4.8.4
		 */
		public function show_more_fields_in_import_for_mapping( $fields = array() ) {

			$custom_field_data = ES()->custom_fields_db->get_custom_fields();
			$custom_field_cols = array();
			if ( count( $custom_field_data ) > 0 ) {
				foreach ($custom_field_data as $key => $data) {
					$fields[$data['slug']] =  $data['label'];
				}
			}
			
			return $fields;
		}

		/**
		 * Sanitize and add submitted custom field data of subscriber.
		 *
		 * @param array $contacts_data 
		 * @param array $insert 
		 * 
		 * @since 4.8.4
		 */
		public function prepare_additional_contacts_data_for_import( $additional_data = array(), $insert = array() ) {

			$insert_cf_data = array();
			foreach ( $insert as $key => $value ) {
				if ( strpos( $key, 'cf_') !== false ) {
		
					$cf_slug_explode = explode( '_', $key );
					$insert_cf_data[$key] = '';

					if ( 'date' === $cf_slug_explode[1] ) {
						if ( ! is_numeric( $insert[$key] ) ) {
							$d 				   	  = sanitize_text_field( $insert[$key] );
							$insert_cf_data[$key] = gmdate( 'Y-m-d', strtotime( $d ) );
						} 
					} elseif ( 'int' === $cf_slug_explode[1] ) {
						if ( is_numeric( $insert[$key] ) ) {
							$insert_cf_data[$key] = $insert[$key];
						} 
					} else {
						$insert_cf_data[$key] = sanitize_text_field( $insert[$key] );
					}
				}
			}
			return $insert_cf_data;
		}

		/**
		 * Show custom field data of contacts
		 *
		 * @param array $contacts_data 
		 * @param array $insert 
		 * 
		 * @since 4.8.4
		 */
		public function show_additional_contacts_data( $contact_data = array() ) {

			$custom_fields = ES()->custom_fields_db->get_custom_fields();
			$allowedtags   = ig_es_allowed_html_tags_in_esc();
			$form_html 	   = '';

			if ( count( $custom_fields ) > 0 ) {
				foreach ( $custom_fields as $field ) {

					$label 			 = ! empty( $field['label'] ) ? $field['label'] : '';
					$slug            = ! empty( $field['slug'] ) ? $field['slug'] : '';
					$submitted_value = ! empty( $contact_data['custom_fields'][$field['slug']] ) ? $contact_data['custom_fields'][$field['slug']] : '';
					
					$form_html .= '<div class="flex flex-row border-b border-gray-100">
						<div class="flex w-1/5">
							<div class="ml-4 pt-6">';

					$form_html .= '<label><span class="block ml-4 pt-1 pr-4 text-sm font-medium text-gray-600 pb-2">' . $label . '</span></label>';

					$form_html .= '</div></div><div class="flex"><div class="ml-16 my-4 mr-4"><div class="relative">';

					
					$meta  = ES()->custom_fields_db->get_custom_field_meta_by_slug( $slug );
				
					switch ( $field['type'] ) {
						case 'text':
							$form_html .= '<input type="text" name="contact_data[custom_fields][' . $slug . ']" class="es-cf-text form-input ig_es_contact_' . $slug . '" value="' . $submitted_value . '">';
							break;
						case 'textarea':
							$form_html .= '<textarea class="form-textarea text-sm" cols="50" name="contact_data[custom_fields][' . $slug . ']" class="form-textarea ig_es_contact_' . $slug . '"';
							
							$form_html .= '/>' . $submitted_value . '</textarea></label>';
							break;
						case 'dropdown':
							$form_html .= '<select name="contact_data[custom_fields][' . $slug . ']" class="es-cf-select form-select ig_es_contact_' . $slug . '">';
							
							$form_html .= '<option value="">' . esc_html__('Select option', 'email-subscribers') . '</option>';
							
							if ( isset( $meta['values'] ) ) {
								foreach ($meta['values'] as $key) {
									$form_html .= '<option value="' . $key . '"';

									if ( ! empty ( $submitted_value ) && $submitted_value === $key ) {
										$form_html .= ' selected = "selected"';
									} 
									$form_html .= '>' . $key . '</option>';
								}
							}

							$form_html .= '</select>';
							break;
						case 'radio':
							if ( isset( $meta['values'] ) ) {
								foreach ($meta['values'] as $key) {
									$form_html .= '<input type="radio" name="contact_data[custom_fields][' . $slug . ']" class="es-cf-radio form-radio ig_es_contact_' . $slug . '" value="' . $key . '" '; 

									if ( ! empty ( $submitted_value ) && $submitted_value === $key ) {
										$form_html .= 'checked="checked"';
									} 
									$form_html .= '>' . $key . '<br>';
								}
							}
							break;
						case 'number':
							$form_html .= '<input type="number" name="contact_data[custom_fields][' . $slug . ']" class="es-cf-number form-input ig_es_contact_' . $slug . '"  value="' . $submitted_value . '" />' ;
							break;
						case 'date':
							$form_html .= '<input type="date" name="contact_data[custom_fields][' . $slug . ']" class="es-cf-date form-input ig_es_contact_' . $slug . '" value="' . $submitted_value . '" />' ;
							break;
						default:
							break;
					}

					$form_html .= '</div></div></div></div>';
				}
			}
			echo wp_kses( $form_html, $allowedtags );
			
		}

		/** Set custom field data to be saved
		 *
		 * @param array $contact
		 * @param array $submitted_data 
		 * 
		 * @since 4.8.4
		 */
		public function prepare_cf_data_for_contact_array( $contact = array(), $submitted = false ) {

			$contact_cf_data = array();
			$contact_data 	 = $contact;

			if ( isset( $contact['custom_fields'] ) && $submitted ) {
				//if the contact details are submitted we will get cf_data in an array with index 'custom_fields'
				$contact_data = $contact['custom_fields'];
			} 

			foreach ($contact_data as $key => $value) {	
				if ( strpos( $key, 'cf_') !== false ) {
					$contact_cf_data['custom_fields'][$key] = $value;
				}
			}

			return $contact_cf_data;
		}

		/** Set custom field data to be saved
		 *
		 * @param array $contact
		 * @param array $submitted_data 
		 * 
		 * @since 4.8.4
		 */
		public function set_additional_contact_data( $contact = array(), $submitted_data = array() ) {

			if ( isset( $submitted_data['custom_fields'] ) ) {
				foreach ( $submitted_data['custom_fields'] as $cf_name => $value ) {
					$contact[$cf_name] = sanitize_text_field( $value );	
				}
			}

			return $contact;
		}
	}
}

new ES_Pro_Custom_Fields();
