<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Pro_Reports_Data' ) ) {
	/**
	 * Get Reports Data
	 * Class ES_Reports_Pro_Data
	 *
	 * @since 4.4.0
	 */
	class ES_Pro_Reports_Data {

		public function __construct() {
			add_action( 'ig_es_view_report_data', array( __CLASS__, 'view_report_data' ) );
			add_filter( 'ig_es_get_country_code', array( __CLASS__, 'get_country_code' ), 10, 2 );
			add_action( 'ig_es_view_report_description_html', array( __CLASS__, 'view_report_description_and_kpi' ), 10, 2 );
			add_action( 'ig_es_display_report_activity_data', array( __CLASS__, 'es_display_report_activity_data' ), 10, 3 );
			add_filter( 'additional_es_campaign_report_columns', array( &$this, 'add_additional_reports_column' ), 99, 1 );
			add_filter( 'additional_es_report_activity_data', array( &$this, 'add_additional_es_report_activity_data' ), 99, 3 );
			add_action( 'ig_es_campaign_reports_filter_options', array( __CLASS__, 'add_campaign_reports_filter_options' ), 99 , 2 );
		}

		/**
		 * Get country code
		 *
		 * @param string $country_code
		 * @param string $ip
		 *
		 * @return mixed|string
		 *
		 * @since 4.5.0
		 */
		public static function get_country_code( $country_code = '', $ip = '' ) {

			if ( empty( $ip ) ) {
				return $country_code;
			}

			$device_location_data = ES_Geolocation::geolocate_ip( $ip );

			$country_code = ! empty( $device_location_data['country_code'] ) ? $device_location_data['country_code'] : '';

			return $country_code;

		}

		/**
		 * Get advance individual reports data
		 *
		 * @version 4.5.0
		 */
		public static function view_report_data( $id ) {
			global $wpdb;

			$notification             = ES_DB_Mailing_Queue::get_notification_by_hash( $id );
			$report_id                = $notification['id'];
			$notification_campaign_id = $notification['campaign_id'];
			$total_email_sent         = ES()->actions_db->get_count_based_on_id_type( $notification_campaign_id, $report_id, IG_MESSAGE_SENT );
			$email_viewed_count       = ES()->actions_db->get_count_based_on_id_type( $notification_campaign_id, $report_id, IG_MESSAGE_OPEN );
			//--->
			$email_unsubscribed_count = ES()->actions_db->get_count_based_on_id_type( $notification_campaign_id, $report_id, IG_CONTACT_UNSUBSCRIBE );
			$avg_unsubscribed_rate    =	!empty($total_email_sent) ? number_format_i18n(( ( $email_unsubscribed_count/$total_email_sent ) * 100 ), 2) : 0;
			//--->
			$avg_open_rate            = ! empty( $total_email_sent) ? number_format_i18n( ( ( $email_viewed_count * 100 ) / $total_email_sent ), 2 ) : 0;
			$email_click_count        = ! empty( $notification_campaign_id ) ? ES()->actions_db->get_count_based_on_id_type( $notification_campaign_id, $report_id, IG_LINK_CLICK ) : 0;
			$avg_click_rate           = ! empty( $total_email_sent ) ? number_format_i18n( ( ( $email_click_count * 100 ) / $total_email_sent ), 2 ) : 0;

			if ( empty( $notification['campaign_id'] ) ) {
				$notification_type = __( 'Post Notification', 'email-subscribers' );
			} else {
				$notification_type = ES()->campaigns_db->get_campaign_type_by_id( $notification['campaign_id'] );
				$notification_type = strtolower( $notification_type );
				$notification_type = ( 'newsletter' === $notification_type ) ? __( 'Broadcast', 'email-subscribers' ) : $notification_type;
			}

				$report_kpi_statistics = array(
					'total_email_sent'			=> number_format_i18n( $total_email_sent ),
					'email_viewed_count'		=> number_format_i18n( $email_viewed_count ),
					'email_unsubscribed_count'	=> number_format_i18n( $email_unsubscribed_count ),
					'email_click_count'			=> number_format_i18n( $email_click_count ),
					'avg_open_rate'				=> $avg_open_rate,
					'avg_click_rate'			=> $avg_click_rate,
					'avg_unsubscribed_rate'		=> $avg_unsubscribed_rate,
				);

				$notification['type']    		 = ucwords( str_replace( '_', ' ', $notification_type ) );
				$notification_subject    		 = $notification['subject'];
				$notification_campaign   	 	 = ES()->campaigns_db->get_campaign_by_id( $notification_campaign_id, - 1 );
				$notification['from_email'] 	 = ! empty( $notification_campaign['from_email'] ) ? $notification_campaign['from_email'] : '';
				$notification_lists_ids 	 	 = ! empty( $notification_campaign['list_ids'] ) ? explode( ',', $notification_campaign['list_ids'] ) : array();
				$notification['list_name'] 		 = ES_Common::prepare_list_name_by_ids( $notification_lists_ids );
				$total_contacts          		 = $notification['count'];
				$notification_status     		 = $notification['status'];
				$campaign_meta					 = ! empty( $notification_campaign['meta'] ) ? unserialize( $notification_campaign['meta']) : '';
				$notification['list_conditions'] = ! empty( $campaign_meta['list_conditions'] ) ? $campaign_meta['list_conditions'] : '';

				$where                = $wpdb->prepare( 'message_id = %d ORDER BY updated_at DESC', $report_id );
				$notification_actions = ES()->actions_db->get_by_conditions( $where );

				$links_where        = $wpdb->prepare( 'message_id = %d', $report_id );
				$notification_links = ES()->links_db->get_by_conditions( $links_where );

				$activity_data = array();
				$time_offset   = get_option( 'gmt_offset' ) * HOUR_IN_SECONDS;
				$date_format   = get_option( 'date_format' );

				if ( ! empty( $notification_actions ) ) {
					foreach ( $notification_actions as $notification_action ) {
						$action_type = (int) $notification_action['type'];
						if ( in_array( $action_type, array( IG_MESSAGE_OPEN, IG_LINK_CLICK ), true ) ) {
							$created_timestamp = $notification_action['created_at'];
							//$created_date      = date_i18n( $date_format, $created_timestamp + $time_offset );
							$created_date = gmdate( 'Y-m-d', $created_timestamp + $time_offset );
							if ( ! isset( $activity_data[ $created_date ] ) ) {
								$activity_data[ $created_date ] = array(
									'opened'  => 0,
									'clicked' => 0,
								);
							}
							if ( IG_MESSAGE_OPEN === $action_type ) {
								$activity_data[ $created_date ]['opened'] ++;
							} elseif ( IG_LINK_CLICK === $action_type ) {
								$activity_data[ $created_date ]['clicked'] ++;
							}
						}
					}
				}
				if ( ! empty( $activity_data ) ) {
					ksort( $activity_data );
				}

				// To display report header information and KPI values
				do_action( 'ig_es_view_report_description_html', $notification, $report_kpi_statistics );
				
				// To display open and click activity, country, mail and device opens, link activity and last 10 open activity
				do_action( 'ig_es_display_report_activity_data', $notification_actions, $notification_links, $activity_data );
		}

		/**
		 * Display Report header information and KPI values
		 *
		 * @version 4.6.9
		 */
		public static function view_report_description_and_kpi( $notification, $report_kpi_statistics ) {
			?>
			<div class="wrap max-w-7xl">
				<div class="wp-heading-inline flex items-center justify-between">
					<div class="flex-shrink-0 break-words">
						<h2 class="text-2xl font-medium leading-7 tracking-wide text-gray-900 pt-1">
							<?php echo esc_html__( 'Report', 'email-subscribers' ); ?>
						</h2>
					</div>
				</div>
				<div class="mt-3 pb-2 w-full bg-white rounded-md shadow flex">
					<div class="w-3/4">
						<div class="flex pl-6 pt-4">
							<div class="w-auto inline-block text-xl text-gray-600 font-medium leading-7 truncate">
								<?php echo esc_html( $notification['subject'] ); ?>
							</div>
							<div class="inline-block ml-2 font-semibold leading-5 tracking-wide text-xs">
								<?php 
								switch ( $notification['status'] ) {
									case 'Sent':  
										?>
										<svg class="inline-block mt-1.5 ml-1 h-5 w-5 text-green-400" fill="currentColor" viewBox="0 0 20 20">
											<title><?php echo esc_attr__( 'Sent', 'email-subscribers' ); ?></title>
											<path fill-rule="evenodd" d="M10 18a8 8 0 100-16 8 8 0 000 16zm3.707-9.293a1 1 0 00-1.414-1.414L9 10.586 7.707 9.293a1 1 0 00-1.414 1.414l2 2a1 1 0 001.414 0l4-4z" clip-rule="evenodd"/>
										</svg>
									<?php 
										break;
									case 'In Queue': 
										?>
									<svg class="inline-block mt-1.5 ml-1 h-5 w-5 text-orange-400" fill="currentColor" viewBox="0 0 20 20">
										<title><?php echo esc_attr__( 'In Queue', 'email-subscribers' ); ?></title>
										<path fill-rule="evenodd" d="M10 18a8 8 0 100-16 8 8 0 000 16zm1-12a1 1 0 10-2 0v4a1 1 0 00.293.707l2.828 2.829a1 1 0 101.415-1.415L11 9.586V6z" clip-rule="evenodd"/>
									</svg>
									<?php 
										break;
									case 'Sending': 
										?>
									<svg class="inline-block mt-1.5 ml-1 h-5 w-5 text-yellow-400" fill="currentColor" viewBox="0 0 20 20">
										<title><?php echo esc_attr__( 'Sending', 'email-subscribers' ); ?></title>
										<path fill-rule="evenodd" d="M10 18a8 8 0 100-16 8 8 0 000 16zm3.707-8.707l-3-3a1 1 0 00-1.414 1.414L10.586 9H7a1 1 0 100 2h3.586l-1.293 1.293a1 1 0 101.414 1.414l3-3a1 1 0 000-1.414z" clip-rule="evenodd"/>
									</svg>
								<?php 
										break;
									case '1': 
										?>
									<span class="inline-flex px-2 text-green-800 bg-green-100 rounded-full"><?php echo esc_html__('Active', 'email-subscribers'); ?></span>
								<?php 
										break;
									case '': 
										?>
									   <span class="inline-flex px-2 text-red-800 bg-red-100 rounded-full"><?php echo esc_html__('Inactive', 'email-subscribers'); ?></span>
							<?php } ?>
							</div>
						</div>
						<div class="w-full text-gray-600 italic font-medium pt-4 text-sm leading-5 overflow-hidden">
							<p class="pl-6 truncate"><?php echo esc_html__( 'Type: ', 'email-subscribers' ); ?>
								<span class="pl-1 font-normal not-italic text-gray-900"><?php echo esc_html( $notification['type'] ); ?></span>
							</p>
							<p class="pl-6 pt-2 truncate"><?php echo esc_html__( 'From: ', 'email-subscribers' ); ?>
								<span class="pl-1 font-normal not-italic text-gray-900"><?php echo esc_html( $notification['from_email'] ); ?></span>
							</p>
							<div class="pl-6 pt-2 inline-block truncate relative w-full">
							<span class="recipient-text"><?php echo esc_html__( 'Recipient(s): ', 'email-subscribers' ); ?></span>
								<div class="pl-1 font-normal not-italic text-gray-900 inline-block truncate w-11/12" style="padding-left: 80px;">
									<?php 
									if ( ! empty( $notification['list_name'] ) ) {
										echo esc_html( $notification['list_name'] ); 
									} else {
										if ( ! empty( $notification['list_conditions'] ) ) {
											do_action( 'ig_es_campaign_show_conditions', $notification['list_conditions'] ); 
										}
									}
									?>
								</div>
							</div>
							<?php if ( ! in_array( $notification['type'], array( 'Sequence Message', 'Workflow Email' ), true ) ) { ?>
							<p class="pl-6 pt-2 text-gray-600 "><?php echo esc_html__( 'Date: ', 'email-subscribers' ); ?>
								<span class="pl-1 font-normal not-italic text-gray-900"><?php echo wp_kses_post( ig_es_format_date_time( $notification['start_at'] ) ); ?></span>
							</p>
						<?php } ?>
						</div>
					</div>
					<div class="w-1/2">
						<div class="flex-1 min-w-0">
							<p class="pt-4 pl-8 text-lg font-medium leading-6 text-gray-400">
								<?php echo esc_html__( 'Statistics', 'email-subscribers' ); ?>
							</p>
							<div class="sm:grid sm:grid-cols-2 ml-6 mr-8">

								<div class="p-2">
									<span class = "text-2xl font-bold leading-none text-indigo-600">
										<?php	echo esc_html( $report_kpi_statistics['email_viewed_count']); ?>
									</span>

									<span class = "text-xl font-bold leading-none text-indigo-600">
										<?php	echo esc_html( ' (' . $report_kpi_statistics['avg_open_rate'] . '%)'); ?>
									</span>

									<p class="mt-1 font-medium leading-6 text-gray-500">
										<?php echo esc_html__( 'Opened', 'email-subscribers' ); ?>
									</p>
								</div>

								<div class="p-2">
									<span class = "text-2xl font-bold leading-none text-indigo-600">
										<?php	echo esc_html( $report_kpi_statistics['email_click_count']); ?>
									</span>

									<span class = "text-xl font-bold leading-none text-indigo-600">
										<?php	echo esc_html( ' (' . $report_kpi_statistics['avg_click_rate'] . '%)'); ?>
									</span>
									
									<p class="mt-1 font-medium leading-6 text-gray-500">
										<?php echo esc_html__( 'Clicked', 'email-subscribers' ); ?>
									</p>
								</div>

								<div class="p-2">
									<span class="text-2xl font-bold leading-none text-indigo-600">
										<?php echo esc_html( $report_kpi_statistics['total_email_sent'] ); ?>
									</span>
									<p class="mt-1 font-medium leading-6 text-gray-500">
										<?php echo esc_html__( 'Sent', 'email-subscribers' ); ?>
									</p>
								</div>

								<div class="p-2">
									<span class = "text-2xl font-bold leading-none text-indigo-600">
										<?php	echo esc_html( $report_kpi_statistics['email_unsubscribed_count']); ?>
									</span>
									<span class = "text-xl font-bold leading-none text-indigo-600">
										<?php	echo esc_html( ' (' . $report_kpi_statistics['avg_unsubscribed_rate'] . '%)'); ?>
									</span>

									<p class="mt-1 font-medium leading-6 text-gray-500">
										<?php echo esc_html__( 'Unsubscribed', 'email-subscribers' ); ?>
									</p>
								</div>
							
							</div>
						</div>
					</div>
				</div>
		<?php
		}

		/**
		* Display open and click activity, country, mail and device opens, link activity and last 10 open activity
		* 
		* @since 4.5.2
		*
		*/
		public static function es_display_report_activity_data( $notification_actions, $notification_links, $activity_data ) {
			global $wpdb; 
			?>
			<div class="mt-6 mb-4">
					<p class="pt-3 text-lg font-medium leading-7 tracking-wide text-gray-600"><?php echo esc_html__( 'Open and Click activity', 'email-subscribers' ); ?>
					</p>
					<div class="bg-white mt-2 mb-4 w-full rounded-md shadow" id="click_link_activity">
					</div>
			</div>

			<div class="mt-6 mb-4 flex">
				<div class="pt-3 mr-4 w-1/2 self-start">
					<p class="text-lg font-medium leading-7 tracking-wide text-gray-600"><?php echo esc_html__( 'Country info', 'email-subscribers' ); ?></p>
					<table class="mt-2 w-full table-fixed bg-white rounded-md shadow overflow-hidden">
						<thead>
						<tr class="es_reports_table_header">
							<th width="10%"></th>
							<th class="w-1/3 px-4 py-3 font-medium ">
								<?php echo esc_html__( 'Country', 'email-subscribers' ); ?>
							</th>
							<th class="w-1/3 px-6 py-3 font-medium">
								<?php echo esc_html__( 'Opens', 'email-subscribers' ); ?>
							</th>
						</tr>
						</thead>
						<tbody>
						<?php
						$country_opens     = array(); 
						$browser_opens     = array(); 
						$os_opens          = array();
						$mail_client_opens = array();
						$row_limit   	   = 5;
						$device_opens      = array(
							'desktop' => 0,
							'tablet'  => 0,
							'mobile'  => 0,
						);

						if ( ! empty( $notification_actions ) ) {
							$country_opens['others'] = 0;
							$mail_client_opens['others'] = 0;
							foreach ( $notification_actions as $notification_action ) {
								if ( ! empty( $notification_action['type'] ) && IG_MESSAGE_OPEN === (int) $notification_action['type'] ) {
									$country_code = $notification_action['country'];
									if ( ! empty( $country_code ) ) {
										if ( ! isset( $country_opens[ $country_code ] ) ) {
											$country_opens[ $country_code ] = 0;
										}
										$country_opens[ $country_code ] += 1;
									} else {
										$country_opens['others'] += 1;
									}
									$device_type = $notification_action['device'];
									if ( ! empty( $device_type ) ) {
										$device_opens[ $device_type ] += 1;
									}
									$browser_type = $notification_action['browser'];
									if ( ! empty( $browser_type ) ) {
										if ( ! isset( $browser_opens[ $browser_type ] ) ) {
											$browser_opens[ $browser_type ] = 0;
										}
										$browser_opens[ $browser_type ] += 1;
									}
									$os_type = $notification_action['os'];
									if ( ! empty( $os_type ) ) {
										if ( ! isset( $os_opens[ $os_type ] ) ) {
											$os_opens[ $os_type ] = 0;
										}
										$os_opens[ $os_type ] += 1;
									}
									$mail_client_type = $notification_action['email_client'];
									if ( ! empty( $mail_client_type ) ) {
										if ( ! isset( $mail_client_opens[ $mail_client_type ] ) ) {
											$mail_client_opens[ $mail_client_type ] = 0;
										}
										$mail_client_opens[ $mail_client_type ] += 1;
									} else {
										$mail_client_opens['others'] += 1;
									}

								}
							}
							//Sorting values in descending order for the graph
							arsort( $browser_opens );
							arsort( $os_opens );
							//Sorting values in descending order for the Mail client open count table 
							arsort( $mail_client_opens );

						}

						if ( ! empty( $country_opens ) && ! empty ( array_filter ( $country_opens ) ) ) {

							$country_opens_data = self::reports_data_count( $country_opens, $row_limit );
							foreach ( $country_opens_data as $code => $opens_count ) {
								if ( 'others' === $code ) {
									$country_name = __('Others', 'email-subscribers');
								} else {
									$country_name = ES_Geolocation::get_countries_iso_code_name_map( $code );
								}
								if ( ! empty( $opens_count ) ) {
									?>
									<tr class="border-b border-gray-200 text-sm leading-5">
										<td class="mx-4 my-3 px-6 py-1 country_flag column-country_flag">
											<p id="es-flag-icon" class="ml-0 xl:ml-1.5 inline-block leading-5 flag-icon flag-icon-<?php echo esc_attr( strtolower($code) ); ?>"></p>
										</td>
										<td class="pl-4 py-3 text-gray-500">
											<?php
											echo esc_html( $country_name );
											?>
										</td>
										<td class="pl-6 py-3 text-gray-600">
											<?php
											echo esc_html( number_format( $opens_count ) );
											?>
										</td>
									</tr>
									<?php
								}
							}
						} else {
							?>
							<tr>
								<td colspan="3" class="px-6 py-3 text-sm leading-5 text-gray-500">
									<?php
									echo esc_html__( 'No country data found.', 'email-subscribers' );
									?>
								</td>
							</tr>
							<?php
						}
						?>
						</tbody>
					</table>
				</div>
				
				<div class="ml-4 w-1/2 self-start">
					<p class="pt-3 text-lg font-medium leading-7 tracking-wide text-gray-600"><?php echo esc_html__( 'Mail Client info', 'email-subscribers' ); ?></p>
					<table class="mt-2 w-full table-fixed bg-white rounded-md shadow overflow-hidden">
						<thead>
						<tr class="es_reports_table_header">
							<th class="w-2/3 px-6 py-3 font-medium ">
								<?php echo esc_html__( 'Mail Client', 'email-subscribers' ); ?>
							</th>
							<th class="w-1/3 px-6 py-3 font-medium">
								<?php echo esc_html__( 'Opens', 'email-subscribers' ); ?>
							</th>
						</tr>
						</thead>
						<tbody>
						<?php
						
						if ( ! empty( $mail_client_opens ) && ! empty ( array_filter ( $mail_client_opens ) ) ) {
							$mail_client_opens_data = self::reports_data_count( $mail_client_opens, $row_limit );
					
							if ( ! empty( $mail_client_opens_data ) ) {
								foreach ( $mail_client_opens_data as $mail_client_name => $opens_count ) {
									if ( 'others' === $mail_client_name) {
										$mail_client_name = __('Others', 'email-subscribers');
									}
									if ( ! empty( $opens_count ) ) {
										?>
									<tr class="border-b border-gray-200 text-sm leading-5">
										<td class="px-6 py-3 text-gray-500">
											<?php
											echo esc_html( $mail_client_name );
											?>
										</td>
										<td class="px-6 py-3 text-gray-600">
											<?php
											echo esc_html( number_format( $opens_count ) );
											?>
										</td>
									</tr>
									<?php
									}
								}
							}
						} else {
							?>
							<tr>
								<td class="px-6 py-3 text-sm leading-5 text-gray-500">
									<?php
									echo esc_html__( 'No mail client data found.', 'email-subscribers' );
									?>
								</td>
							</tr>
							<?php
						}
						?>
						</tbody>
					</table>
				</div>
			</div> 
			<?php
			$graph_open_data = array(
				array(
					'title'      => __( 'Device info', 'email-subscribers' ),
					'info'       => $device_opens,
					'graph_name' => 'device_open_graph',
					'msg'        => __( 'No device data found', 'email-subscribers' ),
				),

				array(
					'title'      => __( 'Browser info', 'email-subscribers' ),
					'info'       => $browser_opens,
					'graph_name' => 'browser_open_graph',
					'msg'        => __( 'No browser data found', 'email-subscribers' ),
				),

				array(
					'title'      => __( 'OS info', 'email-subscribers' ),
					'info'       => $os_opens,
					'graph_name' => 'os_info_graph',
					'msg'        => __( 'No OS data found', 'email-subscribers' ),
				),
			);
			?>
			<div class="mt-6 mb-4 grid w-full gap-8 grid-cols-3">
				<?php foreach ( $graph_open_data as $data ) { ?>
					<div class="w-full">
						<p class="pt-3 text-lg font-medium leading-7 tracking-wide text-gray-600"><?php echo esc_html( $data['title'] ); ?></p>
						<?php
							// Array filter returns non-zero array values. If there are no opened emails then it will return an empty array 
						if ( ! empty( $data['info'] ) && ! empty( array_filter( $data['info'] ) ) ) { 
							?>
									<div class="relative bg-white mt-2 rounded-md shadow" id="<?php echo esc_attr( $data['graph_name'] ); ?>"></div>
						<?php } else { ?>
							<div class="mt-2 bg-white text-sm text-gray-500 rounded-md shadow py-3 px-6 tracking-wide">
								<?php echo esc_html( $data['msg'] ); ?>
							</div>
						<?php } ?>
					</div>
				<?php } ?>
			</div>

			<?php
			if ( ! empty( $notification_links ) ) {
				?>
				<p class="mt-6 pt-3 text-lg font-medium leading-7 tracking-wide text-gray-600">
					<?php echo esc_html__( 'Link activity', 'email-subscribers' ); ?>
				</p>
				<div class="mt-2 mb-4">
					<div class="flex bg-white shadow rounded-md break-words self-start">
						<table id="es_reports_link_activity" class="w-full table-fixed">
							<thead>
							<tr class="es_reports_table_header">
								<th class="w-3/5 px-6 py-3  font-medium "><?php echo esc_html__( 'Link (URL)', 'email-subscribers' ); ?>
								</th>
								<th class="w-1/5 px-6 py-3 font-medium"><?php echo esc_html__( 'Unique Clicks', 'email-subscribers' ); ?>
								</th>
								<th class="w-1/5 px-6 py-3 font-medium"><?php echo esc_html__( 'Total Clicks', 'email-subscribers' ); ?>
								</th>
							</tr>
							</thead>
							<tbody>
							<?php
							foreach ( $notification_links as $notification_link ) {
								$link_id         = $notification_link['id'];
								$link_where      = $wpdb->prepare( 'link_id = %d', $link_id );
								$total_count     = 0;
								$unique_count    = 0;
								
								$link_count_data = ES()->actions_db->get_by_conditions( $link_where );
								
								$site_url = html_entity_decode( $notification_link['link'] );
								$query_string = '';

								if ( strpos( $site_url, '?' ) ) {
									$url_split = explode( '?', $site_url );
									$site_url  = $url_split[0];
									
									$url_params   = array_unique( explode( '&' , $url_split[1]) );
									$query_string = '?' . implode( '&', $url_params );
								}
								$link_url = $site_url . $query_string;

								$notification_link_list[$link_id] = array( 
									'link' 			=> $link_url,
									'total_count' 	=> 0,
									'unique_count' 	=> 0,
								);

								if ( ! empty( $link_count_data ) ) {
									foreach ( $link_count_data as $count_data ) {
										$total_count += $count_data['count'];
										$notification_link_list[$link_id]['total_count'] += $count_data['count'];
										
										if ( ! empty( $count_data['count'] ) ) {
											$unique_count ++;
											$notification_link_list[$link_id]['unique_count'] = $unique_count;
										}
									}
								}
							}
							usort( $notification_link_list, array( 'ES_Pro_Reports_Data', 'es_sort_link_activity'));
							
							foreach ($notification_link_list as $link_id) {
								?>
								<tr class="border-b border-gray-200 text-sm leading-5">
									<td class="px-6 py-3 text-gray-900">
										<?php
										echo esc_url( $link_id['link'] );
										?>
									</td>
									<td class="px-6 py-3 text-gray-600">
										<?php
										echo esc_html( number_format( $link_id['unique_count'] ) );
										?>
									</td>
									<td class="px-6 py-3 text-gray-600">
										<?php
										echo esc_html( number_format( $link_id['total_count'] ) );
										?>
									</td>
								</tr>
								<?php
							}
							?>
							</tbody>
						</table>
					</div>
					
					<div class="es_reports_table_header rounded-b text-right px-6 py-3 ">
						<span id="es_link_activity_less" class="inline-block font-medium mr-4 cursor-pointer hover:underline">
							<svg class="w-4 h-4 inline -mt-1" fill="none" stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M11 19l-7-7 7-7m8 14l-7-7 7-7"></path></svg>
							<?php echo esc_html__( 'Show Less', 'email-subscribers'); ?>
							
						</span>
						<span id="es_link_activity_more" class="inline-block font-medium cursor-pointer hover:underline">
							<?php echo esc_html__( 'Show More', 'email-subscribers'); ?>
							<svg class="w-4 h-4 inline -mt-1" fill="none" stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M13 5l7 7-7 7M5 5l7 7-7 7"></path></svg>
						</span>
					</div>
				</div>
				<?php
			}
			?>

			<script type="text/javascript">

				(function ($) {

					$(document).ready(function () {

						<?php
						$activity_dates   = array();
						$activity_opened  = array();
						$activity_clicked = array();
						if ( ! empty( $activity_data ) ) {
							$activity_dates   = array_keys( $activity_data );
							$activity_opened  = array_column( $activity_data, 'opened' );
							$activity_clicked = array_column( $activity_data, 'clicked' );
						}

						// If we don't have any activity dates to be shown, then show today's date with zero opened/clicked counts to avoid console errors.
						if ( empty( $activity_dates ) ) {
							$today_date       = current_time( 'Y-m-d' );
							$activity_dates[] = $today_date;
							$activity_opened  = 0;
							$activity_clicked = 0;
						} elseif ( 1 === count( $activity_dates ) ) {
							// If we have only one date to show then show previous date with zero opened/clicked counts to avoid console errors.
							// Reference: https://github.com/frappe/charts/issues/220#issuecomment-645944174
							// TODO: Remove this hanlding if the issue get resolved in the frappe chart library.
							$first_date    = $activity_dates[0];
							$previous_date = gmdate( 'Y-m-d', strtotime( '-1 day', strtotime( $first_date ) ) );
							array_unshift( $activity_dates, $previous_date );
							array_unshift( $activity_opened, 0 );
							array_unshift( $activity_clicked, 0 );
						}
						?>

						//for click and link activity
						let open_link_data = {
							labels: <?php echo wp_json_encode( $activity_dates ); ?>,
							datasets: [
								{
									name: '<?php echo esc_html__( 'Opens', 'email-subscribers' ); ?>',
									values: <?php echo wp_json_encode( $activity_opened ); ?>,
									chartType: 'line'
								},
								{
									name: '<?php echo esc_html__( 'Clicks', 'email-subscribers' ); ?>',
									values: <?php echo wp_json_encode( $activity_clicked ); ?>,
									chartType: 'line'
								}
							]
						}


						const open_link_chart = new frappe.Chart("#click_link_activity", {
							title: "",
							data: open_link_data,
							type: 'axis-mixed',
							colors: ['#743ee2',],
							lineOptions: {
								hideDots: 1,
								heatline: 1
							},
							height: 200,
							axisOptions: {
								xIsSeries: true,
							}
						});


						<?php
						//Graph for Device Opens
						$device_opened = array();
						$device_label  = array();
						if ( ! empty( $device_opens ) && ! empty( array_filter( $device_opens ) ) ) {
							$device_label  = array_map( 'ucfirst' , array_keys( $device_opens ) );
							$device_opened = array_values( $device_opens );
							?>

						let device_data = {
							labels: <?php echo json_encode( $device_label ); ?>,
							datasets: [
								{
									name: "device", 
									type: "pie",
									values: <?php echo json_encode( $device_opened ); ?>,
								}
							]
						}

						const device_chart = new frappe.Chart("#device_open_graph", {
							title: "",
							data: device_data,
							type: 'pie',
							colors: ['#743ee2', '#5DADE2', '#F6608B'],
							height: 280,
							maxSlices: 3,
						});
							<?php 
						}

						//Graph for Browser Opens
						$browser_label  = array();
						$browser_opened = array();
						if ( ! empty( $browser_opens ) ) {
							$browser_label  = array_keys( $browser_opens );
							$browser_opened = array_values( $browser_opens );
							?>

						let browser_data = {
							labels: <?php echo json_encode( $browser_label ); ?>,
							datasets: [
								{
									name: "device", 
									type: "pie",
									values: <?php echo json_encode( $browser_opened ); ?>,
								}
							]
						}

						const browser_chart = new frappe.Chart("#browser_open_graph", {
							data: browser_data,
							type: 'pie',
							colors: ['#CA226B', '#F6358A', '#E55451', '#3EA99F', '#77BFC7', '#54C571'],
							height: 280,
							maxSlices: 6,
						});
							<?php 
						}

						//Graph for OS Opens
						$os_label  = array();
						$os_opened = array();
						if ( ! empty( $os_opens )  ) {
							$os_label  = array_keys( $os_opens );
							$os_opened = array_values( $os_opens );
							?>

						let os_data = {
							labels: <?php echo json_encode( $os_label ); ?>,
							datasets: [
								{
									name: "device", 
									type: "pie",
									values: <?php echo json_encode( $os_opened ); ?>,
								}
							]
						}

						const os_chart = new frappe.Chart("#os_info_graph", {
							data: os_data,
							type: 'pie',
							colors: ['#3EA055', '#5DADE2', '#B041FF', '#46C7C7', '#B2E8A2', '#E7A1B0'],
							height: 280,
							maxSlices: 6,
						});
						<?php } ?>
					});
				})(jQuery);


			</script>
			<?php 
		}

		/**
		* Sort link activity by total_clicks in desc
		* 
		* @since 4.7.3
		*
		*/
		public static function es_sort_link_activity( $a, $b ) {
			if ( $a['total_count'] == $b['total_count'] ) {
				return 0;
			}
			return ( $a['total_count'] < $b['total_count'] ) ? 1 : -1;
		}


		/**
		* Count and limit reports data like Country opens, mail client.
		* 
		* @since 4.6.12
		*
		*/
		public static function reports_data_count( $data_opens, $row_limit = 5 ) {
				
			$others = false;
			if ( isset( $data_opens['others'] ) ) {
				$others_count = $data_opens['others'];
				unset( $data_opens['others'] );
				$others = true;
			}

				arsort( $data_opens );

			if ( $others ) {
				$data_opens['others'] = $others_count;
			}

				$row_count      = 0;
				$stats_opens_data = array();
			foreach ( $data_opens as $index => $opens_count ) {
				if ( $row_count < $row_limit ) {
					$stats_opens_data[ $index ] = $opens_count;
				} else {
					if ( ! isset( $stats_opens_data['others'] ) ) {
						$stats_opens_data['others'] = 0;
					}
					$stats_opens_data['others'] += $opens_count;
				}
				$row_count ++;
			}

				return $stats_opens_data;
		}

		/**
		* Display filter options for activity table
		* 
		* @since 4.6.12
		*
		*/
		public static function add_campaign_reports_filter_options( $text = '', $input_id = '' ) {
			?>
			<p class="search-box" id="es-activity-report-filter">
				<label class="screen-reader-text" for="<?php echo esc_attr( $input_id ); ?>"><?php echo esc_attr( $text ); ?>:</label>
				<input type="search" id="<?php echo esc_attr( $input_id ); ?>" name="s" value="<?php _admin_search_query(); ?>" placeholder="Search Email"/>
				<?php submit_button( __( 'Filter Report', 'email-subscribers' ), 'button', false, false, array( 'id' => 'campaign-report-search-submit' ) ); ?>
			</p>
			<p class="search-box search-group-box box-ma10">
				<?php
				$filter_by_status = ig_es_get_request_data( 'filter_activity_report_by_status' );
				?>
				<select name="filter_activity_report_by_status" id="ig_es_filter_activity_report_by_status">
					<?php
					$allowedtags = ig_es_allowed_html_tags_in_esc();
					add_filter( 'safe_style_css', 'ig_es_allowed_css_style' );
					$statuses = array(
						'opened' 	  => __( 'Opened', 'email-subscribers' ),
						'not_opened'  => __( 'Not Opened', 'email-subscribers' ),
					);
					$campaign_report_status = ES_Common::prepare_campaign_report_statuses_dropdown_options( $statuses, $filter_by_status, __( 'All Status', 'email-subscribers' ) );
					echo wp_kses( $campaign_report_status, $allowedtags );
					?>
				</select>
			</p>
			<p class="search-box search-group-box box-ma10">
				<?php 
				$filter_by_country = ig_es_get_request_data( 'filter_activity_report_by_country' );
				$report_activity_country = '<select name="filter_activity_report_by_country" id="ig_es_filter_activity_report_by_country">';
					
				$countries_data = ES_Geolocation::get_countries();
				$report_activity_country .= '<option value="">All</option>';
																					
				foreach ( $countries_data as $country_code => $country_name ) {
						$report_activity_country .= '<option value="' . esc_attr( $country_code ) . '" ';

					if ( ! empty( $filter_by_country ) && $filter_by_country == $country_code ) {
						$report_activity_country .= 'selected = selected';
					}

						$report_activity_country .= '>' . esc_html( $country_name ) . '</option>';
						
				}
		
				$report_activity_country .= '</select>';
				echo wp_kses( $report_activity_country, $allowedtags );
				?>
			</p>
			<?php
		}

		/**
		 * Method to add additional columns in Reports Activity table 
		 * 
		 * @since 4.6.12
		 */
		public function add_additional_reports_column( $es_reports_col ) {

			$report['country_flag']			= '<svg class="h-4 w-4 mt-3 text-gray-600" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M3 21v-4m0 0V5a2 2 0 012-2h6.5l1 1H21l-3 6 3 6h-8.5l-1-1H5a2 2 0 00-2 2zm9-13.5V9" /><title>Country flag</title></svg>'; 


			$es_reports_col = ig_es_array_insert_after( $es_reports_col, 'sr_no' , $report );

			$report_col['device']		 	= __( 'Device', 'email-subscribers' );
			$report_col['email_client'] 	= __( 'Mail Client', 'email-subscribers' );
			$report_col['os'] 				= __( 'OS', 'email-subscribers' );

			$es_reports_col = ig_es_array_insert_after( $es_reports_col, 'viewed_date' , $report_col );

			return $es_reports_col;
		}

		/**
		 * Method to add additional data in Reports Activity table 
		 * 
		 * @since 4.6.12
		 */
		public function add_additional_es_report_activity_data( $view_activity_data, $contact_id, $notification_action ) {

				$country_name			 = ES_Geolocation::get_countries_iso_code_name_map( $notification_action['country'] );
				$country_flag 			 = '<p style="margin-left: -0.35rem"  title="' . $country_name . '" id="es-flag-icon" class="inline-block flag-icon flag-icon-' . strtolower( $notification_action['country'] ) . '"></p>';
				$view_activity_data[$contact_id]['country_flag'] = ! empty( $notification_action['country'] ) ? $country_flag : '';

			if ( ! empty( $notification_action['device'] ) ) {
				switch ( $notification_action['device'] ) {
					case 'desktop':
						$open_device = '<svg class="w-4 h-4" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
								<title>' . esc_attr__( 'Desktop', 'email-subscribers' ) . '</title>
								<path fill="currentColor"
									  d="M19,2H5A3,3,0,0,0,2,5V15a3,3,0,0,0,3,3H7.64l-.58,1a2,2,0,0,0,0,2,2,2,0,0,0,1.75,1h6.46A2,2,0,0,0,17,21a2,2,0,0,0,0-2l-.59-1H19a3,3,0,0,0,3-3V5A3,3,0,0,0,19,2ZM8.77,20,10,18H14l1.2,2ZM20,15a1,1,0,0,1-1,1H5a1,1,0,0,1-1-1V14H20Zm0-3H4V5A1,1,0,0,1,5,4H19a1,1,0,0,1,1,1Z"></path>
							</svg>';
						break;

					case 'tablet':
						$open_device = '<svg class="w-4 h-4" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" id="Layer_1" x="0px" y="0px" width="512px" height="512px" viewBox="0 0 512 512" xml:space="preserve">
								<title>' . esc_attr__( 'Tablet', 'email-subscribers' ) . '</title>
								<path fill-rule="evenodd" clip-rule="evenodd" fill="currentColor"
									  d="M416,0H96C78.313,0,64,14.328,64,32v448c0,17.688,14.313,32,32,32  h320c17.688,0,32-14.313,32-32V32C448,14.328,433.688,0,416,0z M256,496c-13.25,0-24-10.75-24-24s10.75-24,24-24s24,10.75,24,24  S269.25,496,256,496z M400,432H112V48h288V432z"></path>
							</svg>';
						break;

					case 'mobile':
						$open_device = '<svg class="w-4 h-4" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
								<title>' . esc_attr__( 'Mobile', 'email-subscribers' ) . '</title>
								<path fill="currentColor" d="M16,2H8A3,3,0,0,0,5,5V19a3,3,0,0,0,3,3h8a3,3,0,0,0,3-3V5A3,3,0,0,0,16,2Zm1,17a1,1,0,0,1-1,1H8a1,1,0,0,1-1-1V18H17Zm0-3H7V5A1,1,0,0,1,8,4h8a1,1,0,0,1,1,1Z"></path>
							</svg>';
						break;
						
					default:
						$open_device = '<svg class="w-4 h-4" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
								<path fill="currentColor"
									  d="M19,2H5A3,3,0,0,0,2,5V15a3,3,0,0,0,3,3H7.64l-.58,1a2,2,0,0,0,0,2,2,2,0,0,0,1.75,1h6.46A2,2,0,0,0,17,21a2,2,0,0,0,0-2l-.59-1H19a3,3,0,0,0,3-3V5A3,3,0,0,0,19,2ZM8.77,20,10,18H14l1.2,2ZM20,15a1,1,0,0,1-1,1H5a1,1,0,0,1-1-1V14H20Zm0-3H4V5A1,1,0,0,1,5,4H19a1,1,0,0,1,1,1Z"></path>
							</svg>';
						break;
				}
			}

				$view_activity_data[$contact_id]['device'] 		= ! empty( $notification_action['device'] ) ? $open_device : '';
				$view_activity_data[$contact_id]['email_client'] = ! empty( $notification_action['email_client'] ) ? $notification_action['email_client'] : '';
				$view_activity_data[$contact_id]['os'] 			= ! empty( $notification_action['os'] ) ? $notification_action['os'] : '';
			return $view_activity_data;
		}

	}
}
