<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Pro_Send_Mail_In_Customer_Timezone' ) ) {

	/**
	 * Class ES_Pro_Send_Mail_In_Customer_Timezone
	 *
	 * @since 5.0.1
	 */
	class ES_Pro_Send_Mail_In_Customer_Timezone {
		/**
		 * Instance of the class
		 *
		 * @var null | ES_Pro_Send_Mail_In_Customer_Timezone
		 */
		private static $_instance = null;

		/**
		 * ES_Pro_Send_Mail_In_Customer_Timezone constructor.
		 */
		public function __construct() {
			$ig_es_enable_sending_mails_in_customer_timezone = get_option( 'ig_es_enable_sending_mails_in_customer_timezone', 'no' );
			if ( 'yes' === $ig_es_enable_sending_mails_in_customer_timezone ) {
				add_filter( 'ig_es_get_emails_to_be_sent_by_hash_condition', array(
					$this,
					'get_emails_to_be_sent_by_hash_condition'
				) );
			}
			add_filter( 'ig_es_registered_email_sending_settings', array(
				$this,
				'registered_email_sending_settings'
			) );
		}

		public function registered_email_sending_settings( $settings ) {
			$pos         = 2;
			$new_setting = array(
				'ig_es_enable_sending_mails_in_customer_timezone' => array(
					'type'         => 'checkbox',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => 'no',
					'id'           => 'ig_es_enable_sending_mails_in_customer_timezone',
					'name'         => __( 'Enable sending mails in customer timezone?', 'email-subscribers' ),
					'info'         => __( 'Enable this option if you want Email Subscribers to send emails in the customer\'s timezone.', 'email-subscribers' ),
				),
			);

			return array_merge( array_slice( $settings, 0, $pos ), $new_setting, array_slice( $settings, $pos ) );
		}

		public function get_emails_to_be_sent_by_hash_condition( $where ) {
			$send_at_time = current_time( 'mysql', true );
			$where        .= " AND (`send_at` < '$send_at_time' OR `send_at` IS NULL)";

			return $where;
		}

		/**
		 * Init the Bounce handler as singleton
		 *
		 * @return ES_Pro_Send_Mail_In_Customer_Timezone|null
		 */
		public static function init() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}

			return self::$_instance;
		}

	}
}

ES_Pro_Send_Mail_In_Customer_Timezone::init();
