<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once  ABSPATH . 'wp-admin/includes/class-wp-list-table.php' ;
}

class ES_Pro_Sequence_Reports extends WP_List_Table {

	public static $instance;

	public function __construct() {

		parent::__construct( array(
			'singular' => __( 'Report', 'email-subscribers' ), //singular name of the listed records
			'plural'   => __( 'Reports', 'email-subscribers' ), //plural name of the listed records
			'ajax'     => false, //does this table support ajax?,
			'screen'   => 'es_reports'
		) );

	}

	/**
	 * Display list of sequence messages 
	 *
	 * @since 4.5.2
	 */
	public function es_sequence_reports_callback() {

		$campaign_id = ig_es_get_request_data( 'campaign_id' );
		
		$action = ig_es_get_request_data( 'action' );
		if ( 'view' === $action ) {
			$this->view_report( $campaign_id );
		} else {
			?>

			<div class="wrap">
				<h1 class="wp-heading-inline"><span class="text-2xl font-medium leading-7 text-gray-900 sm:leading-9 sm:truncate"><?php echo esc_html__( 'Reports', 'email-subscribers' ); ?></span></h1>
				<div id="poststuff" class="es-items-lists">
					<div id="post-body" class="metabox-holder column-1">
						<div id="post-body-content">
							<div class="meta-box-sortables ui-sortable">
								<form method="get">
									<input type="hidden" name="page" value="es_reports" />
									<?php
									// Display search field and other available filter fields.
									$this->prepare_items();
									?>
								</form>
								<form method="post">
									<?php
									// Display bulk action fields, pagination and list items.
									$this->display();
									?>
								</form>
							</div>
						</div>
					</div>
					<br class="clear">
				</div>
			</div>

			<?php
		}
	}

	public function screen_option() {

		$option = 'per_page';
		$args   = array(
			'label'   => __( 'Reports', 'email-subscribers' ),
			'default' => 10,
			'option'  => 'reports_per_page'
		);

		add_screen_option( $option, $args );

	}

	/**
	 * Get individual sequence message report
	 *
	 * @since 4.5.2
	 *
	 */
	public function view_report( $campaign_id ) {
		global $wpdb;

		$view_report 		= new ES_Campaign_Report();
		$notification       = ES()->campaigns_db->get( $campaign_id );
		$total_email_sent   = ES()->actions_db->get_count_based_on_id_type( $notification['id'], 0, IG_MESSAGE_SENT );
		$email_viewed_count = ES()->actions_db->get_count_based_on_id_type( $notification['id'], 0, IG_MESSAGE_OPEN );
		$email_click_count  = ES()->actions_db->get_count_based_on_id_type( $notification['id'], 0, IG_LINK_CLICK );
		$avg_open_rate      = ! empty( $total_email_sent) ? number_format_i18n( ( ( $email_viewed_count * 100 ) / $total_email_sent ), 2 ) : 0 ;
		$avg_click_rate     = ! empty( $total_email_sent) ? number_format_i18n( ( ( $email_click_count * 100 ) / $total_email_sent ), 2 ) : 0;
		$email_unsubscribed_count = ES()->actions_db->get_count_based_on_id_type( $campaign_id, 0, IG_CONTACT_UNSUBSCRIBE );
		$avg_unsubscribed_rate    =	!empty($total_email_sent) ? number_format_i18n(( ( $email_unsubscribed_count/$total_email_sent ) * 100 ), 2) : 0;

		$report_kpi_statistics = array(
			'total_email_sent'		    => number_format_i18n( $total_email_sent ),
			'email_viewed_count'	    => number_format_i18n( $email_viewed_count ),
			'email_click_count'	        => number_format_i18n( $email_click_count ),
			'email_unsubscribed_count'	=> number_format_i18n( $email_unsubscribed_count ),
			'avg_open_rate'			    => $avg_open_rate,
			'avg_click_rate'		    => $avg_click_rate,
			'avg_unsubscribed_rate'		=> $avg_unsubscribed_rate
		);

		$notification['type']        	 = ucwords( str_replace( '_', ' ', $notification['type'] ) );
		$notification['list_name'] 	 	 = ES()->lists_db->get_list_name_by_id( $notification['list_ids'] );
		$campaign_meta					 = ! empty( $notification['meta'] ) ? unserialize( $notification['meta']) : '';
		$notification['list_conditions'] = ! empty( $campaign_meta['list_conditions'] ) ? $campaign_meta['list_conditions'] : '';
		 
		//Sequence messages have all message ids as 0 for a particular campaign id
		$message_id = 0;
		$where      = $wpdb->prepare( 'campaign_id = %d AND message_id = %d ORDER BY updated_at DESC', $campaign_id, $message_id );
		
		$notification_actions = ES()->actions_db->get_by_conditions( $where );

		$links_where        = $wpdb->prepare( 'message_id = %d AND campaign_id = %d', $message_id, $campaign_id );
		$notification_links = ES()->links_db->get_by_conditions( $links_where );

		$activity_data      = array();
		$view_activity_data = array();
		
		$time_offset = get_option( 'gmt_offset' ) * HOUR_IN_SECONDS;
		$date_format = get_option( 'date_format' );
		
		$contact_ids_arr = array_column( $notification_actions, 'contact_id' );
		if ( ! empty( $contact_ids_arr ) ) {
			$contacts_data = ES()->contacts_db->get_details_by_ids( $contact_ids_arr );

			if ( ! empty( $notification_actions ) ) {				
				foreach ( $notification_actions as $notification_action ) {
					$action_type = (int) $notification_action['type'];
					if ( in_array( $action_type, array( IG_MESSAGE_OPEN, IG_LINK_CLICK ), true ) ) {
						$created_timestamp = $notification_action['created_at'];
						
						$created_date = gmdate( 'Y-m-d', $created_timestamp + $time_offset );
						if ( ! isset( $activity_data[ $created_date ] ) ) {
							$activity_data[ $created_date ] = array(
								'opened'  => 0,
								'clicked' => 0,
							);
						}
						if ( IG_MESSAGE_OPEN === $action_type ) {
							$activity_data[ $created_date ]['opened'] ++;
						} elseif ( IG_LINK_CLICK === $action_type ) {
							$activity_data[ $created_date ]['clicked'] ++;
						}
					}
					$contact_id = $notification_action['contact_id'];

					//Creating data for view activity table
					if ( ! isset( $view_activity_data[$contact_id] ) ) {
						$view_activity_data[$contact_id] = array(
							'email'		=> ! empty( $contacts_data[$contact_id]['email'] ) ? $contacts_data[$contact_id]['email'] : '',
							'opened'	=> 0,
							'opened_at' => 0,
							'status' 	=> 0,
							'sent_at'   => 0,
						);
					}
					if ( IG_MESSAGE_OPEN === $action_type ) {
						$view_activity_data[$contact_id]['opened'] = 'Viewed';
						$view_activity_data[$contact_id]['opened_at']   = ! empty( $notification_action['created_at'] ) ? ES_Common::convert_timestamp_to_date( $notification_action['created_at'] ) : '' ;
					} elseif ( IG_MESSAGE_SENT === $action_type ) {
						   $view_activity_data[$contact_id]['status'] = 'Sent';
						   $view_activity_data[$contact_id]['sent_at']     =  ! empty( $notification_action['created_at'] ) ? ES_Common::convert_timestamp_to_date( $notification_action['created_at'] ) : '' ;
					}
				}
			}
		
			if ( ! empty( $activity_data ) ) {
				ksort( $activity_data );
			}
		}

		// To display report header information and KPI values
		do_action( 'ig_es_view_report_description_html', $notification, $report_kpi_statistics );
		
		// To display open and click activity, country, mail and device opens, link activity and last 10 open activity
		do_action( 'ig_es_display_report_activity_data', $notification_actions, $notification_links, $activity_data );

		// To display view activity table
		$view_report->es_campaign_report_callback();
		?>
		</div>
	<?php
	}


	/** Text displayed when no list data is available */
	public function no_items() {
		echo esc_html__( 'No Reports available.', 'email-subscribers' );
	}

	/**
	 * Render a column when no column specific method exist.
	 *
	 * @param array $item
	 * @param string $column_name
	 *
	 * @return mixed
	 */
	public function column_default( $item, $column_name ) {
		global $wpdb;

		switch ( $column_name ) {
			case 'start_at':
				return ig_es_format_date_time( $item[ $column_name ] );
			case 'type':
				if ( ! empty( $item['type'] ) ) {
					$type = ucwords( str_replace( '_', ' ', $item['type'] ) );
				}
				return $type;
			case 'subject':
				// case 'type':
				//      return ucwords($item[ $column_name ]);
			case 'count':
				return $item[ $column_name ];

			case 'viewed':
				$total_view_count   = ES()->actions_db->get_count_based_on_id_type( $item['id'], 0, IG_MESSAGE_OPEN );
				return $total_view_count;

			default:
				return print_r( $item, true ); //Show the whole array for troubleshooting purposes
		}
	}

	public function column_status( $item ) {
		return ( $item['campaign_status'] ) ? __( 'Active', 'email-subscribers' ) : __( 'Inactive', 'email-subscribers' );
	
	}

	/**
	 * Render the bulk edit checkbox
	 *
	 * @param array $item
	 *
	 * @return string
	 */
	public function column_cb( $item ) {
		return sprintf(
			'<input type="checkbox" name="bulk_delete[]" value="%s" />', $item['id']
		);
	}


	/**
	 * Method for name column
	 *
	 * @param array $item an array of DB data
	 *
	 * @return string
	 */
	public function column_subject( $item ) {

		$es_nonce = wp_create_nonce( 'es_notification' );
		$page     = ig_es_get_request_data( 'page' );

		$title = '<strong>' . $item['subject'] . '</strong>';

		$actions = array(
			'view'          =>  sprintf( '<a href="?page=%s&action=%s&campaign_id=%d&_wpnonce=%s" class="text-indigo-600">%s</a>', esc_attr( $page ), 'view', $item['id'], $es_nonce, __( 'View', 'email-subscribers' ) ),
			'preview_email' => sprintf( '<a href="#" data-campaign-id="%s" data-campaign-type="%s" class="es-preview-report text-indigo-600">%s</a><img class="es-preview-loader inline-flex align-middle pl-2 h-5 w-7" src="%s" style="display:none;"/>', absint( $item['id'] ), $item['type'], __( 'Preview', 'email-subscribers' ), esc_url( ES_PLUGIN_URL ) . 'lite/admin/images/spinner-2x.gif' ),

		);

		return $title . $this->row_actions( $actions );
	}

	/**
	 * Method for start_at column
	 *
	 * @param array $item an array of DB data
	 *
	 * @return string
	 */
	public function column_start_at( $item ) {
		return ES_Common::convert_timestamp_to_date( $item['start_at'] );
	}

	/**
	 *  Associative array of columns
	 *
	 * @return array
	 */
	public function get_columns() {
		$columns = array(
			'cb'        => '<input type="checkbox" />',
			'subject'   => __( 'Subject', 'email-subscribers' ),
			'type'      => __( 'Type', 'email-subscribers' ),
			'status'    => __( 'Status', 'email-subscribers' ),
			'count'     => __( 'Total Contacts', 'email-subscribers' ),
			//'Opened' column is being added from manage_es_reports_columns filter.		
		);

		return $columns;
	}

	public function column_count( $item ) {
		$total_emails_sent       = $item['count'];
		$total_emails_to_be_sent = $item['count'];
		
		return $total_emails_to_be_sent;

	}


	/**
	 * Columns to make sortable.
	 *
	 * @return array
	 */
	public function get_sortable_columns() {
		$sortable_columns = array(
			'subject'   => array( 'subject', true ),
			'status'    => array( 'status', true ),
			'start_at'  => array( 'start_at', true ),
			'count'     => array( 'count', true )
		);

		return $sortable_columns;
	}


	/**
	 * Handles data query and filter, sorting, and pagination.
	 */
	public function prepare_items() {

		$this->_column_headers = $this->get_column_info();

		$per_page     = $this->get_items_per_page( 'reports_per_page', 20 );
		$current_page = $this->get_pagenum();
		$total_items  = $this->get_notifications( 0, 0, true );

		$this->set_pagination_args( array(
			'total_items' => $total_items, //WE have to calculate the total number of items
			'per_page'    => $per_page //WE have to determine how many items to show on a page
		) );
		
		$this->items = $this->get_notifications( $per_page, $current_page, false );

	}

	/**
	* Getting the list of sequence notification i.e sequence_message for a particular Sequence
	*
	* @since 4.5.2
	*
	*/
	public function get_notifications( $per_page = 5, $page_number = 1, $do_count_only = false ) {
		global $wpdb;

		$campaign_id = ig_es_get_request_data( 'campaign_id' );

		$query             = $wpdb->prepare( 'parent_id = %d', $campaign_id);
		$sequence_messages = ES()->campaigns_db->get_by_conditions( $query );

		if ( ! $do_count_only ) {
			$action_report		 = array();
			$campaign_table_data = array();

			foreach ($sequence_messages as $sequence_data) {
				if ( in_array( $sequence_data['type'], array( 'sequence_message', 'workflow_email' ), true ) ) {

					$action_report = $wpdb->get_row( $wpdb->prepare( "SELECT COUNT(DISTINCT(`contact_id`)) as count FROM {$wpdb->prefix}ig_actions WHERE `campaign_id`= %d AND `type` = %d ", $sequence_data['id'] , IG_MESSAGE_SENT ), ARRAY_A );
					if ( ! empty( $action_report ) ) {
						$campaign_table_data[] = array_merge(
							$sequence_data,
							array(
								'count' => $action_report['count'],
								'campaign_status' => $sequence_data['status'],
							)
						);
					}

				}
			}
			return $campaign_table_data;
		} else {
			return count( $sequence_messages ); 
		}
	}

	public static function get_instance() {
		if ( ! isset( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}
