<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Common class for wishlist objects of different wishlist plugin 
 * 
 * @since 4.6.10
 */
class IG_ES_Wishlist {

	/**
	 * Wishlist ID
	 * 
	 * @var int
	 */
	public $id = 0;

	/**
	 * Wishlist ower ID
	 * 
	 * @var int
	 */
	public $owner_id = 0;

	/**
	 * Products
	 * 
	 * @var array
	 */
	public $items;

	/**
	 * Get wishlist id
	 * 
	 * @return int
	 */
	public function get_id() {
		return absint( $this->id );
	}


	/**
	 * Get wishlist user id
	 * 
	 * @return int
	 */
	public function get_user_id() {
		return absint( $this->owner_id );
	}


	/**
	 * Get wishlist customer
	 * 
	 * @return IG_ES_Customer|bool
	 */
	public function get_customer() {
		return IG_ES_Customer_Factory::get_by_user_id( $this->get_user_id() );
	}


	/**
	 * Get wishlist plugin name
	 * 
	 * @return string
	 */
	public function get_integration() {
		return IG_ES_Wishlists::get_integration();
	}


	/**
	 * Get wishlist items
	 * 
	 * @return array
	 */
	public function get_items() {

		if ( isset( $this->items ) ) {
			return $this->items;
		}

		$this->items = array();

		if ( 'yith' === $this->get_integration() ) {

			$products = YITH_WCWL()->get_products(array(
				'wishlist_id' => $this->get_id(),
				'user_id' => $this->get_user_id()
			));

			if ( ! empty( $products ) ) {
				foreach ( $products as $product ) {
					$this->items[] = $product['prod_id'];
				}
			}
		}

		$this->items = array_unique( $this->items );

		return $this->items;
	}


	/**
	 * Get wishlist view link
	 * 
	 * @return string
	 */
	public function get_link() {

		if ( 'yith' === $this->get_integration() ) {
			return YITH_WCWL()->get_wishlist_url();
		}

		return '';
	}

}
