<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle Wishlist integration
 * 
 * @since 4.6.10
 */
class IG_ES_Wishlists {

	/**
	 * List of Wishlist integrations
	 * 
	 * @var array
	 */
	public static $integration_options = array(
		'yith' => 'YITH Wishlists',
	);

	/**
	 * Get wishlist plugin id
	 * 
	 * @return string|false
	 */
	public static function get_integration() {
		if ( class_exists( 'YITH_WCWL') ) {
			return 'yith';
		} else {
			return false;
		}
	}

	/**
	 * Get wishlist plugin title
	 * 
	 * @return string|false
	 */
	public static function get_integration_title() {
		$integration = self::get_integration();

		if ( ! $integration ) {
			return false;
		}

		return self::$integration_options[$integration];
	}


	/**
	 * Get wishlist by ID
	 *
	 * @param int $id
	 * @return bool|IG_ES_Wishlist
	 */
	public static function get_wishlist( $id ) {

		$integration = self::get_integration();

		if ( ! $id || ! $integration ) {
			return false;
		}

		if ( 'yith' === $integration ) {
			$wishlist = YITH_WCWL()->get_wishlist_detail( $id );
		} else {
			return false;
		}

		return self::get_normalized_wishlist( $wishlist );
	}



	/**
	 * Convert wishlist objects from both integrations into the same format
	 * Returns false if wishlist is empty
	 *
	 * @param $wishlist WP_Post|array
	 * @return IG_ES_Wishlist|false
	 */
	public static function get_normalized_wishlist( $wishlist ) {

		$integration = self::get_integration();

		if ( ! $wishlist || ! $integration ) {
			return false;
		}

		$normalized_wishlist = new IG_ES_Wishlist();
		if ( 'yith' === $integration ) {

			if ( ! is_object( $wishlist )  ) {
				return false;
			}

			$normalized_wishlist->id       = $wishlist->get_id();
			$normalized_wishlist->owner_id = $wishlist->get_user_id();
		}

		return $normalized_wishlist;
	}

	/**
	 * Get an array with the IDs of all wishlists.
	 *
	 * @return array
	 */
	public static function get_all_wishlist_ids() {
		return self::get_wishlist_ids();
	}

	/**
	 * Get wishlist IDs.
	 *
	 * @param int|bool $limit
	 * @param int      $offset
	 *
	 * @return array
	 */
	public static function get_wishlist_ids( $limit = false, $offset = 0 ) {

		$integration = self::get_integration();

		$wishlist_ids = array();
		if ( 'yith' === $integration ) {

			$wishlists = YITH_WCWL()->get_wishlists( array(
				'user_id'    => false,
				'show_empty' => false,
				'limit'      => false === $limit ? false : $limit,
				'offset'     => $offset,
			));

			foreach ( $wishlists as $wishlist ) {
				$wishlist_ids[] = $wishlist->get_id();
			}
		}

		if ( ! empty( $wishlist_ids ) ) {
			$wishlist_ids = array_map( 'absint', $wishlist_ids );
		}

		return $wishlist_ids;
	}

}
