<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Simple_Email_Service_Message' ) ) {
	/**
	 * Class ES_Simple_Email_Service_Message
	 *
	 * @since 4.6.14
	 */
	class ES_Simple_Email_Service_Message extends IG_ES_Simple_Email_Service_Message {

		public $template_variables = array();

		public $default_template_data = array();

		public $template_name = '';

		/**
		 * Add CC email
		 * 
		 * @return IG_ES_Simple_Email_Service_Message $this
		 * @see addTo()
		 */
		public function addTemplateVariables( $email, $template_variables ) {
			$this->template_variables[$email] = $template_variables;

			$this->is_clean = false;

			return $this;
		}

		/**
		 * Get the raw mail message
		 *
		 * @return string
		 */
		public function getRawMessage( $encode = true) {
			if ($this->is_clean && !is_null($this->raw_message) && $encode) {
				return $this->raw_message;
			}

			$this->is_clean = true;

			$boundary = uniqid(rand(), true);
			$raw_message = count($this->customHeaders) > 0 ? join("\n", $this->customHeaders) . "\n" : '';

			if (!empty($this->message_tags)) {
				$message_tags = array();
				foreach ($this->message_tags as $key => $value) {
					$message_tags[] = "{$key}={$value}";
				}

				$raw_message .= 'X-SES-MESSAGE-TAGS: ' . join(', ', $message_tags) . "\n";
			}

			if (!is_null($this->configuration_set)) {
				$raw_message .= 'X-SES-CONFIGURATION-SET: ' . $this->configuration_set . "\n";
			}

			$raw_message .= count($this->to) > 0 ? 'To: ' . $this->encodeRecipients($this->to) . "\n" : '';
			$raw_message .= 'From: ' . $this->encodeRecipients($this->from) . "\n";
			if (!empty($this->replyto)) {
				$raw_message .= 'Reply-To: ' . $this->encodeRecipients($this->replyto) . "\n";
			}

			if (!empty($this->cc)) {
				$raw_message .= 'CC: ' . $this->encodeRecipients($this->cc) . "\n";
			}
			if (!empty($this->bcc)) {
				$raw_message .= 'BCC: ' . $this->encodeRecipients($this->bcc) . "\n";
			}

			if ( null != $this->subject && strlen($this->subject) > 0 ) {
				$raw_message .= 'Subject: =?' . $this->subjectCharset . '?B?' . base64_encode($this->subject) . "?=\n";
			}

			if ( ! empty( $this->template_variables ) ) {
				$i = 1;
				foreach ( $this->template_variables as $index => $variables ) {
					$template_data = wp_json_encode( $variables );
					$raw_message .= 'Destinations.member.' . $i . '.ReplacementTemplateData: ' . $template_data . "\n";
					$i++;
				}
			}

			$raw_message .= 'MIME-Version: 1.0' . "\n";
			$raw_message .= 'Content-type: ' . ( $this->hasInlineAttachments() ? 'multipart/related' : 'Multipart/Mixed' ) . '; boundary="' . $boundary . '"' . "\n";
			$raw_message .= "\n--{$boundary}\n";
			$raw_message .= 'Content-type: Multipart/Alternative; boundary="alt-' . $boundary . '"' . "\n";

			if ( null  != $this->messagetext && strlen($this->messagetext) > 0 ) {
				$charset = empty($this->messageTextCharset) ? '' : "; charset=\"{$this->messageTextCharset}\"";
				$raw_message .= "\n--alt-{$boundary}\n";
				$raw_message .= 'Content-Type: text/plain' . $charset . "\n\n";
				$raw_message .= $this->messagetext . "\n";
			}

			if ( null != $this->messagehtml && strlen($this->messagehtml) > 0 ) {
				$charset = empty($this->messageHtmlCharset) ? '' : "; charset=\"{$this->messageHtmlCharset}\"";
				$raw_message .= "\n--alt-{$boundary}\n";
				$raw_message .= 'Content-Type: text/html' . $charset . "\n\n";
				$raw_message .= $this->messagehtml . "\n";
			}
			$raw_message .= "\n--alt-{$boundary}--\n";

			foreach ($this->attachments as $attachment) {
				$raw_message .= "\n--{$boundary}\n";
				$raw_message .= 'Content-Type: ' . $attachment['mimeType'] . '; name="' . $attachment['name'] . '"' . "\n";
				$raw_message .= 'Content-Disposition: ' . $attachment['attachmentType'] . "\n";
				if (!empty($attachment['contentId'])) {
					$raw_message .= 'Content-ID: ' . $attachment['contentId'] . '' . "\n";
				}
				$raw_message .= 'Content-Transfer-Encoding: base64' . "\n";
				$raw_message .= "\n" . chunk_split(base64_encode($attachment['data']), 76, "\n") . "\n";
			}

			$raw_message .= "\n--{$boundary}--\n";

			if (!$encode) {
				return $raw_message;
			}

			$this->raw_message = base64_encode($raw_message);

			return $this->raw_message;
		}
	}
}
