<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Email_Subscribers_Pro' ) ) {
	/**
	 * Class Email_Subscribers_Pro
	 *
	 * @since 3.5.0
	 */
	class Email_Subscribers_Pro {
		/**
		 * Pro assets URL
		 *
		 * @var string
		 */
		public $pro_plugin_url;

		/**
		 * Pro assets path
		 *
		 * @var string
		 */
		public $pro_plugin_path;

		/**
		 * Email_Subscribers_Pro constructor.
		 *
		 * @sinc 3.x
		 */
		public function __construct() {

			$this->pro_plugin_url  = untrailingslashit( plugins_url( '/', __FILE__ ) ) . '/';
			$this->pro_plugin_path = untrailingslashit( plugin_dir_path( __FILE__ ) );

			$this->load_dependencies();

			add_action( 'admin_enqueue_scripts', array( &$this, 'enqueue_admin_styles_and_scripts' ) );
			add_action( 'es_after_action_buttons', array( &$this, 'add_list_cleanup_btn' ) );

			// Add Submenu Pages.
			add_action( 'ig_es_add_submenu_page', array( &$this, 'add_pro_submenu_page' ), 10, 1 );

			if ( defined( 'DOING_AJAX' ) ) {
				add_action( 'wp_ajax_es_list_cleanup', array( &$this, 'es_list_cleanup' ) );
			}

			add_filter( 'ig_es_add_subscriber_data', array( &$this, 'es_add_extra_subscriber_data' ), 10, 1 );

			add_filter( 'manage_es_subscribers_columns', array( &$this, 'es_add_audience_col' ), 99, 1 );
			add_filter( 'es_subscribers_col_data', array( &$this, 'es_add_email_status_data' ), 10, 3 );
			add_filter( 'ig_es_before_save_settings', array( &$this, 'es_modify_settings' ), 10, 2 );
			add_filter( 'ig_es_track_clicks', array( &$this, 'can_track_clicks' ), 10, 4 );

			//optin optout page
			add_action( 'es_redirect_to_optin_page', array( &$this, 'es_optin_redirection' ), 10, 2 );
			add_filter( 'ig_es_registered_settings', array( &$this, 'es_add_settings_fields' ), 10, 2 );

			add_filter( 'ig_es_email_templates', array( &$this, 'es_get_pro_templates' ), 10, 1 );

			//contact report click kpi
			add_action( 'ig_es_after_contacts_kpis', array( &$this, 'es_contact_click_kpi' ) );

			//Add action to Duplicate broadcast
			add_filter( 'ig_es_campaign_actions', array( &$this, 'add_duplicate_campaign_action' ), 10, 2 );
			add_action( 'ig_es_campaign_action', array( &$this, 'duplicate_campaign' ), 10, 2 );

			// ES post digest include
			if ( is_file( $this->pro_plugin_path . '/pro-class-post-digest.php' ) ) {
				include_once $this->pro_plugin_path . '/pro-class-post-digest.php';
				new ES_Post_Digest();
			}

			// ES sequence include
			if ( is_file( $this->pro_plugin_path . '/pro-class-sequences.php' ) ) {
				include_once $this->pro_plugin_path . '/pro-class-sequences.php';
				new ES_Sequences();
			}

			// Include ES PRO Reports
			if ( is_file( $this->pro_plugin_path . '/classes/class-es-pro-reports-data.php' ) ) {
				include_once $this->pro_plugin_path . '/classes/class-es-pro-reports-data.php';
				new ES_Pro_Reports_Data();
			}

			// Filter to add Pro Workflow Triggers
			add_filter( 'ig_es_workflow_triggers', array( &$this, 'register_workflow_triggers' ) );

			// Filter to add Pro Workflow Required Data Types
			add_filter( 'ig_es_data_types_includes', array( &$this, 'register_workflow_data_types' ) );

			// Filter to add Pro Workflow Required Variables
			add_filter( 'ig_es_workflow_variables', array( &$this, 'register_workflow_variables' ) );

			// Filter to add Pro Workflow Actions
			add_filter( 'ig_es_workflow_actions', array( &$this, 'register_workflow_actions' ) );

			// Add tracking fields data
			add_filter( 'ig_es_broadcast_data', array( &$this, 'add_tracking_fields_data' ) );

			// Filter to register services offered in pro version.
			add_filter( 'ig_es_services', array( &$this, 'register_services' ) );

			// Filter to get plugin plan
			add_filter( 'ig_es_plan', array( &$this, 'get_plan' ), 12 );

			add_action( 'init', array( $this, 'init' ) );

			add_action( 'ig_es_workflow_updated', array( $this, 'maybe_enable_wc_cart_tracking' ), 10, 2 );
			add_action( 'ig_es_workflow_status_changed', array( $this, 'maybe_enable_wc_cart_tracking' ) );

			// Check if cart has to be restored
			add_action( 'wp_loaded', array( $this, 'check_for_cart_restore' ) );

			// Filters to show Pro version subscribers data
			add_filter( 'ig_es_subscribers_add_col_data', array( &$this, 'es_subscribers_add_col_data' ), 10, 2 );

			add_filter( 'ig_es_get_subscriber_ip', array( &$this, 'es_get_subscriber_ip' ), 10, 2 );

			// Filter to get country code based on ip for new subscribers
			add_filter( 'ig_es_get_country_based_on_ip', array( &$this, 'es_get_ip_related_country_data' ), 10, 1 );
			// Action scheduler action to add country code to contacts table in background. Called through Action Scheduler library.
			add_action( 'ig_es_add_country_code_to_audience', array( &$this, 'add_country_code_to_audience' ) );

			// Add attachment button to WP Editor.
			add_action( 'media_buttons', array( &$this, 'add_attachment_button' ) );

			// Add Mailers
			add_filter( 'ig_es_mailers', array( &$this, 'register_mailers' ) );

			// Add heading and content for importing existing subscribers.
			add_action( 'ig_es_subscriber_import_method_tab_heading', array(
				&$this,
				'add_import_existing_subscribers_heading'
			) );
			add_action( 'ig_es_subscriber_import_method_tab_content', array(
				&$this,
				'add_import_existing_subscribers_content'
			) );

			add_action( 'ig_es_subscriber_import_method_tab_heading', array( &$this, 'add_wc_importer_heading' ) );
			add_action( 'ig_es_subscriber_import_method_tab_content', array( &$this, 'add_wc_importer_content' ) );

			add_filter( 'manage_es_reports_columns', array( &$this, 'add_additional_reports_column' ), 99, 1 );
			add_filter( 'es_add_additional_report_column_data', array(
				&$this,
				'add_additional_report_column_data'
			), 10, 2 );

			add_action( 'wp_ajax_ig_es_prepare_import_data_from_wc_orders', array(
				&$this,
				'prepare_import_data_from_wc_orders'
			) );

			add_action( 'wp_ajax_ig_es_json_search_woocommerce_coupons', array(
				&$this,
				'handle_json_search_woocommerce_coupons'
			) );

			add_filter( 'bulk_actions-es_subscribers', array( &$this, 'add_bulk_actions' ) );
			add_action( 'ig_es_audience_screen_action_bulk_send_confirmation_email', array(
				&$this,
				'handle_bulk_send_confirmation_email_action'
			) );
		}

		/**
		 * Search for coupons and echo JSON.
		 */
		public function handle_json_search_woocommerce_coupons() {
			check_ajax_referer( 'search-products', 'security' );

			if ( ! current_user_can( 'manage_woocommerce' ) ) {
				die;
			}

			$term                   = ( isset( $_GET['term'] ) && ! empty( $_GET['term'] ) ) ? sanitize_text_field( wp_unslash( $_GET['term'] ) ) : '';
			$available_coupon_codes = IG_ES_Coupon::get_coupons_by_keyword( $term, true );

			wp_send_json( $available_coupon_codes );
			die;
		}

		/**
		 * Method to restore a abandoned cart
		 *
		 * @since 4.6.5
		 */
		public function check_for_cart_restore() {

			global $ig_es_tracker;

			if ( empty( $_GET['es'] ) || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) || is_admin() ) {
				return;
			}

			$is_woocommerce_active = $ig_es_tracker::is_plugin_activated( 'woocommerce/woocommerce.php' );

			// Return if WooCommerce is deactive or abandoned cart is disabled.
			if ( ! $is_woocommerce_active || ! IG_ES_Abandoned_Cart_Options::is_cart_tracking_enabled() ) {
				return;
			}

			$option = ig_es_get_request_data( 'es' );
			$hash   = ig_es_get_request_data( 'hash' );

			if ( ! empty( $hash ) ) {
				$data  = ig_es_decode_request_data( $hash );
				$token = ! empty( $data['token'] ) ? $data['token'] : '';
			} else {
				$token = ig_es_get_request_data( 'token' );
			}

			if ( 'restore-wc-cart' === $option ) {
				if ( ! empty( $token ) ) {
					$restored = IG_ES_WC_Carts::restore_cart( IG_ES_Cart_Factory::get_by_token( $token ) );
					$redirect = ES_Clean::string( ig_es_get_request_data( 'redirect' ) );

					$url_params       = array();
					$redirect_options = array( 'cart', 'checkout' );

					if ( ! in_array( $redirect, $redirect_options, true ) ) {
						$redirect = 'cart';
					}

					if ( $restored ) {
						wc_add_notice( __( 'Your cart has been restored.', 'email-subscribers' ) );
						$url_params['ig-es-cart-restored'] = 'success';
					} else {
						wc_add_notice( __( 'Your cart could not be restored, it may have expired.', 'email-subscribers' ), 'notice' );
						$url_params['ig-es-cart-restored'] = 'fail';
					}

					$redirect_url = add_query_arg(
						$url_params,
						wc_get_page_permalink( $redirect )
					);

					wp_safe_redirect( $redirect_url );
					exit();
				}
			}
		}

		/**
		 * Method to enable abandoned cart tracking
		 *
		 * @param array $workflow_ids IDs of updated workflows.
		 * @param array $workflow_data Workflow data.
		 *
		 * @since 4.6.5
		 */
		public function maybe_enable_wc_cart_tracking( $workflow_ids = array(), $workflow_data = array() ) {

			if ( empty( $workflow_ids ) ) {
				return;
			}

			// Conver to array of ids if not already array.
			if ( ! is_array( $workflow_ids ) ) {
				$workflow_ids = array( $workflow_ids );
			}

			$trigger_name    = isset( $workflow_data['trigger_name'] ) ? $workflow_data['trigger_name'] : '';
			$workflow_status = isset( $workflow_data['status'] ) ? (int) $workflow_data['status'] : IG_ES_WORKFLOW_STATUS_IN_ACTIVE;

			$abandoned_cart_workflow_trigger_name = array(
				'ig_es_wc_user_cart_abandoned',
				'ig_es_wc_guest_cart_abandoned',
				'ig_es_wc_cart_abandoned'
			);
			$is_abandoned_cart_workflow_active    = false;

			if ( in_array( $trigger_name, $abandoned_cart_workflow_trigger_name, true ) && IG_ES_WORKFLOW_STATUS_ACTIVE === $workflow_status ) {
				$is_abandoned_cart_workflow_active = true;
			} else {
				$workflow_query = new ES_Workflow_Query();
				$workflow_query->set_triggers( $abandoned_cart_workflow_trigger_name );
				$workflow_query->where_status( IG_ES_WORKFLOW_STATUS_ACTIVE );
				$workflow_query->set_return( 'ids' );
				$active_abandoned_cart_workflows = $workflow_query->get_results();
				if ( ! empty( $active_abandoned_cart_workflows ) ) {
					$is_abandoned_cart_workflow_active = true;
				}
			}

			if ( $is_abandoned_cart_workflow_active ) {
				update_option( 'ig_es_track_wc_cart', 'yes' );
			} else {
				update_option( 'ig_es_track_wc_cart', 'no' );
			}
		}

		/**
		 * Method to initialize required features like WooCommerce abandoned cart, wishlist.
		 *
		 * @since 4.6.5
		 */
		public function init() {

			global $ig_es_tracker;

			$is_woocommerce_active = $ig_es_tracker::is_plugin_activated( 'woocommerce/woocommerce.php' );

			if ( $is_woocommerce_active ) {

				if ( IG_ES_Abandoned_Cart_Options::is_cart_tracking_enabled() ) {
					IG_ES_WC_Carts::init();
				}

				IG_ES_WC_Products_On_Sale::init();
			}
		}

		/**
		 * Load the required dependencies for this plugin.
		 *
		 * @since 4.6.6
		 */
		public function load_dependencies() {
			$files_to_load = array(
				'mailers/aws-ses/php-aws-ses/SimpleEmailService.php',
				'mailers/aws-ses/php-aws-ses/SimpleEmailServiceMessage.php',
				'mailers/aws-ses/php-aws-ses/SimpleEmailServiceRequest.php',
				'mailers/aws-ses/class-es-simple-email-service.php',
				'mailers/aws-ses/class-es-simple-email-message.php',
				'mailers/aws-ses/class-es-amazon-ses-mailer.php',
				'mailers/class-es-mailgun-mailer.php',
				'mailers/class-es-sendgrid-mailer.php',
				'mailers/class-es-sparkpost-mailer.php',
				'mailers/class-es-postmark-mailer.php',
				'mailers/class-es-sendinblue-mailer.php',
				'mailers/class-es-mailjet-mailer.php',

				'classes/class-es-pro-sequence-report.php',
				'classes/class-ig-es-unsubscribe-feedback.php',
				'classes/class-es-pro-custom-fields.php',
				'classes/class-es-pro-custom-fields-table.php',

				'workflows/class-es-pro-workflow-gallery.php',
				
				// Workflow event helpers
				'workflows/event-helpers/class-ig-es-wc-products-on-sale.php',

				// Workflow admin
				'workflows/admin/class-es-pro-workflow-admin.php',
				'workflows/admin/class-es-pro-workflow-admin-ajax.php',

				// Triggers from Pro version
				'workflows/triggers/abstracts/ig-es-abstract-abandoned-cart.php',
				'workflows/triggers/class-es-trigger-wc-order-refunded.php',
				'workflows/triggers/class-es-trigger-wc-product-review-approved.php',
				'workflows/triggers/class-es-trigger-user-role-changed.php',
				'workflows/triggers/class-ig-es-trigger-wc-cart-abandoned.php',
				'workflows/triggers/class-ig-es-trigger-wc-user-cart-abandoned.php',
				'workflows/triggers/class-ig-es-trigger-wc-guest-cart-abandoned.php',
				'workflows/triggers/class-ig-es-trigger-wishlist-item-goes-on-sale.php',

				// Trigger Extra fields from Pro version
				'workflows/triggers/extras/class-es-pro-trigger-user-registered.php',

				// Action Extra fields from Pro version
				'workflows/actions/extras/class-es-pro-action-add-to-list.php',

				// Data Types from Pro version
				'workflows/data-types/class-es-data-type-review.php',
				'workflows/data-types/class-ig-es-data-type-cart.php',
				'workflows/data-types/class-ig-es-data-type-customer.php',
				'workflows/data-types/class-ig-es-data-type-guest.php',
				'workflows/data-types/class-ig-es-data-type-wishlist.php',

				// Actions from Pro version
				'workflows/actions/class-es-action-move-to-list.php',
				'workflows/actions/class-es-action-remove-from-list.php',

				// Abandoned cart
				'classes/abandoned-cart/customer-functions.php',
				'classes/abandoned-cart/class-ig-es-abandoned-cart-options.php',
				'classes/abandoned-cart/class-ig-es-factories.php',
				'classes/abandoned-cart/class-ig-es-factory.php',
				'classes/abandoned-cart/class-ig-es-customer-factory.php',
				'classes/abandoned-cart/class-ig-es-guest-factory.php',
				'classes/abandoned-cart/class-ig-es-cart-factory.php',
				'classes/abandoned-cart/class-ig-es-cart-item.php',
				'classes/abandoned-cart/class-ig-es-cart.php',
				'classes/abandoned-cart/class-ig-es-wc-cart-query.php',
				'classes/abandoned-cart/class-ig-es-customer.php',
				'classes/abandoned-cart/class-ig-es-guest.php',
				'classes/abandoned-cart/class-ig-es-wc-carts.php',

				// Coupon integration
				'classes/coupon/class-ig-es-coupon.php',

				// Wishlist Integration
				'classes/wishlist/class-ig-es-wishlist.php',
				'classes/wishlist/class-ig-es-wishlists.php',

				// Pro campaign rules
				'classes/class-es-pro-campaign-rules.php',

				// Pro bounce handler
				'classes/class-es-pro-bounce-handler.php',

				// Pro Sending mails in customer timezone
				'classes/class-es-pro-send-mail-in-customer-timezone.php',

				'classes/class-es-subscription-summary-automation.php',
				'classes/class-es-engagement-score.php',

				'classes/class-es-campaign-admin-pro.php',
			);

			foreach ( $files_to_load as $file ) {
				if ( is_file( $this->pro_plugin_path . '/' . $file ) ) {
					require_once $this->pro_plugin_path . '/' . $file;
				}
			}
		}

		/**
		 * Add Pro features Submenu Page
		 *
		 * @param $accessible_sub_menus
		 *
		 * @since 4.3.0
		 */
		public function add_pro_submenu_page( $accessible_sub_menus ) {
			if ( in_array( 'sequences', $accessible_sub_menus ) ) {
				add_submenu_page( 'es_dashboard', __( 'Sequence', 'email-subscribers' ), '<span id="ig-es-sequence">' . __( 'Sequence', 'email-subscribers' ) . '</span>', 'edit_posts', 'es_sequence', array(
					$this,
					'load_sequence'
				) );
			}

			if ( in_array( 'custom_fields', $accessible_sub_menus ) ) {
				add_submenu_page( 'es_dashboard', __( 'Custom Fields', 'email-subscribers' ), '<span id="ig-es-custom-fields">' . __( 'Custom Fields', 'email-subscribers' ) . '</span>', 'edit_posts', 'es_custom_fields', array(
					$this,
					'load_custom_fields'
				) );
			}

		}

		/**
		 * Load Sequences
		 *
		 * @since 4.3.0
		 */
		public function load_sequence() {
			$sequence = new ES_Sequences();
			$sequence->load_sequence();
		}

		/**
		 * Load Custom Fields
		 *
		 * @since 4.8.4
		 */
		public function load_custom_fields() {
			$sequence = new ES_Custom_Fields_Table();
			$sequence->load_custom_fields();
		}

		/**
		 * Check Whether can track link
		 *
		 * @param $can_track_link
		 * @param $contact_id
		 * @param $campaign_id
		 *
		 * @return bool
		 *
		 * @since 4.2.4
		 */
		public function can_track_clicks( $can_track_link, $contact_id = 0, $campaign_id = 0, $link_data = array() ) {
			$ig_es_track_link_clicks = get_option( 'ig_es_track_link_click', 'no' );

			if ( ! empty( $link_data ) ) {
				$campaign_id = ! empty( $link_data['campaign_id'] ) ? $link_data['campaign_id'] : 0;
				if ( ! empty( $campaign_id ) ) {
					$campaign = ES()->campaigns_db->get( $campaign_id );
					if ( ! empty( $campaign ) ) {
						$campaign_type = $campaign['type'];
						
						$supported_campaign_types = array( 
							IG_CAMPAIGN_TYPE_NEWSLETTER, 
							IG_CAMPAIGN_TYPE_POST_NOTIFICATION, 
							IG_CAMPAIGN_TYPE_POST_DIGEST, 
							IG_CAMPAIGN_TYPE_WORKFLOW_EMAIL 
						);

						$is_supported_type = in_array( $campaign_type, $supported_campaign_types, true );
						if ( $is_supported_type ) {
							$campaign_meta           = maybe_unserialize( $campaign['meta'] );
							$ig_es_track_link_clicks = ! empty( $campaign_meta['enable_link_tracking'] ) ? $campaign_meta['enable_link_tracking'] : $ig_es_track_link_clicks;
						}
					}
				}
			}

			if ( 'yes' === $ig_es_track_link_clicks ) {
				return true;
			}

			return $can_track_link;
		}

		/**
		 * Modify Settings
		 *
		 * @param $options
		 *
		 * @return mixed
		 *
		 * @since 4.2.4
		 */
		public function es_modify_settings( $options ) {

			$option_fields = array(
				'ig_es_track_link_click',
				'ig_es_track_utm',
				'ig_es_track_ip_address',
				'ig_es_enable_summary_automation'
			);

			foreach ( $option_fields as $option ) {

				if ( ! isset( $options[ $option ] ) ) {
					$options[ $option ] = 'no';
				}
			}

			return $options;
		}

		/**
		 * Get Email Templates
		 *
		 * @param $templates
		 *
		 * @return mixed
		 *
		 * @since 4.0.0
		 * @since 4.3.1
		 */
		public function es_get_pro_templates( $templates = array() ) {

			$template_files = glob( untrailingslashit( plugin_dir_path( __FILE__ ) ) . '/templates/*.php' );

			if ( is_array( $template_files ) && count( $template_files ) ) {

				foreach ( $template_files as $file ) {
					if ( is_file( $file ) && is_admin() ) {
						$file_name = basename( $file, '.php' );

						$templates[ $file_name ] = include $file;
					}
				}
			}

			return $templates;
		}


		// Register scripts
		public function enqueue_admin_styles_and_scripts() {

			if ( ! ES()->is_es_admin_screen() ) {
				return;
			}

			$attachment_max_file_size = apply_filters( 'ig_es_attachment_max_file_size', 2 * 1024 * 1024 );

			wp_enqueue_style( 'es_jquery-ui', $this->pro_plugin_url . 'assets/css/jquery-ui.min.css', array(), ES_PLUGIN_VERSION, 'all' );
			wp_register_script( 'es_admin_js', $this->pro_plugin_url . 'assets/js/admin.js', array(
				'jquery',
				'jquery-ui-core'
			), ES_PLUGIN_VERSION );
			wp_enqueue_script( 'es_admin_js' );

			if ( ES_Drag_And_Drop_Editor::is_dnd_editor_page() ) {
				//Include PRO D&D blocks
				//wp_register_script( 'es_pro_editor_js', 'http://localhost:4000/pro.js', array( 'es_editor_js' ), time() );
				wp_register_script( 'es_pro_editor_js', $this->pro_plugin_url . 'assets/js/editor.js', array( 'es_editor_js'), ES_PLUGIN_VERSION );
				wp_enqueue_script( 'es_pro_editor_js' );
			}
			

			wp_localize_script( 'es_admin_js', 'es_admin_data', array(
				'admin_ajax_url'                    => admin_url( 'admin-ajax.php' ),
				'total_subscribers'                 => ES()->contacts_db->count(),
				'current_date'                      => date_i18n( 'Y-m-d H:i:s' ),
				'i18n_data'                         => array(
					'empty_email_message'                   => __( 'Please enter an email address.', 'email-subscribers' ),
					'empty_template_message'                => __( 'Please add email body.', 'email-subscribers' ),
					'add_attachment_text'                   => __( 'Add Attachment', 'email-subscribers' ),
					/* translators: %s: Attachmen max file size. */
					'attachment_file_size_exceeded_message' => sprintf( __( 'Please attach a file having size lower than %s.', 'email-subscribers' ), esc_html( size_format( $attachment_max_file_size ) ) ),
					'delete_single_sequence_message' 		=> __( 'Are you sure you want to delete this?', 'email-subscribers' ),
					'checking_orders_message' 		        => __( 'Checking your orders...', 'email-subscribers' ),
					'scheduling_messages' => array(
						'send_immediately' => __( 'Send immediately', 'email-subscribers' ),
						'send_after'       => __( 'Send after', 'email-subscribers' ),
						'hour'             => __( 'hour(s)', 'email-subscribers' ),
						'day'              => __( 'day(s)', 'email-subscribers' ),
						'week'             => __( 'week(s)', 'email-subscribers' ),
					)
				),
				'attachment_max_file_size_in_bytes' => $attachment_max_file_size,
			) );
			//jquery ui css


			wp_enqueue_style( 'dashicons' );
			//datepicker
			if ( ! wp_script_is( 'jquery-ui-datepicker' ) ) {
				wp_enqueue_script( 'jquery-ui-datepicker' );
			}

			// Admin CSS
			wp_enqueue_style( 'es_admin_css', $this->pro_plugin_url . 'assets/css/admin.css', array(), ES_PLUGIN_VERSION, 'all' );
			// Select2 CSS
			if ( ! wp_style_is( 'select2', 'registered' ) ) {
				wp_register_style( 'select2', $this->pro_plugin_url . 'assets/css/select2.min.css', array(), '4.0.13' );
			}

			if ( ! wp_style_is( 'select2' ) ) {
				wp_enqueue_style( 'select2' );
			}

			// Select2 JS
			if ( ! wp_script_is( 'select2', 'registered' ) ) {
				wp_register_script( 'select2', $this->pro_plugin_url . 'assets/js/select2.min.js', array( 'jquery' ), '4.0.13', true );
			}

			if ( ! wp_script_is( 'select2' ) ) {
				wp_enqueue_script( 'select2' );
			}

			if ( ! wp_script_is( 'wc-enhanced-select' ) ) {
				wp_enqueue_script( 'wc-enhanced-select' );
			}

		}

		public function es_list_cleanup() {
			global $wpdb, $wpbd;
			$params = $_REQUEST;
			if ( ! empty( $params['done'] ) && true == $params['done'] && $params['current_count'] < 0 ) {
				update_option( 'es_cleanup_done', true );
				$timezone_format = _x( 'Y-m-d H:i:s', 'timezone date format' );
				$es_current_date = date_i18n( $timezone_format );
				update_option( 'es_cleanup_done_date', $es_current_date );

				return;
			}
			$limit         = 5;
			$subscribers   = $wpdb->get_col( $wpdb->prepare( "SELECT email FROM {$wpdb->prefix}ig_contacts LIMIT %d, %d", $params['start'], $limit ) );
			$es_clean_list = Email_Subscribers_Utils::es_list_cleanup( $subscribers );
			if ( 'SUCCESS' == $es_clean_list['status'] && is_array( $es_clean_list['data'] ) ) {

				foreach ( $es_clean_list['data'] as $email ) {
					$role_based  = ( ! empty( $email['attributes']['roleBased'] ) ) ? $email['attributes']['roleBased'] : 0;
					$webmail     = ( ! empty( $email['attributes']['webmail'] ) ) ? $email['attributes']['webmail'] : 0;
					$disposable  = ( ! empty( $email['attributes']['disposable'] ) ) ? $email['attributes']['disposable'] : 0;
					$deliverable = ( ! empty( $email['attributes']['deliverable'] ) ) ? $email['attributes']['deliverable'] : 0;
					$sendSafely  = ( ! empty( $email['attributes']['sendSafely'] ) ) ? $email['attributes']['sendSafely'] : 0;
					// sql

					$is_rolebased []   = ' WHEN email = "' . $email['_given'] . '" THEN ' . $role_based;
					$is_disposable []  = ' WHEN email = "' . $email['_given'] . '" THEN ' . $disposable;
					$is_deliverable [] = ' WHEN email = "' . $email['_given'] . '" THEN ' . $deliverable;
					$is_sendsafely []  = ' WHEN email = "' . $email['_given'] . '" THEN ' . $sendSafely;

					$status[]       = ' WHEN email = "' . $email['_given'] . '" AND ( ' . $disposable . ' == 1 OR ' . $deliverable . ' == 0 )  THEN "spam"';
					$is_webmail []  = ' WHEN email = "' . $email['_given'] . '" THEN ' . $webmail;
					$is_verified [] = ' WHEN email = "' . $email['_given'] . '" THEN 1';


				}

				$update_sql = 'UPDATE ' . $wpdb->prefix . 'ig_contacts SET
				is_rolebased = (CASE ' . implode( ' ', $is_rolebased ) . ' ELSE is_rolebased END),
				is_disposable = (CASE ' . implode( ' ', $is_disposable ) . ' ELSE is_disposable  END),
				is_deliverable = (CASE ' . implode( ' ', $is_deliverable ) . ' ELSE is_deliverable  END),
				is_sendsafely = (CASE ' . implode( ' ', $is_sendsafely ) . ' ELSE is_sendsafely  END),
				status = (CASE ' . implode( ' ', $status ) . ' ELSE status  END),
				is_webmail = (CASE ' . implode( ' ', $is_webmail ) . ' ELSE is_webmail  END),
				is_verified = (CASE ' . implode( ' ', $is_verified ) . ' ELSE is_verified  END)';


				$result = $wpbd->query( $update_sql );
			}
			// call es util function for cleanup
			$res['start'] = $params['start'] + $limit;

			echo json_encode( $res );
			exit();
		}

		public function es_add_extra_subscriber_data( $data ) {
			$email_arr = array( $data['email'] );
			$res       = Email_Subscribers_Utils::es_list_cleanup( $email_arr );
			if ( 'SUCCESS' == $res['status'] ) {
				$res                    = $res['data'][0];
				$data['is_disposable']  = ! empty( $res['attributes']['disposable'] ) ? $res['attributes']['disposable'] : 0;
				$data['is_deliverable'] = ! empty( $res['attributes']['deliverable'] ) ? $res['attributes']['deliverable'] : 0;
				$data['status']         = ( 1 == $res['attributes']['disposable'] || 0 == $res['attributes']['deliverable'] ) ? 'Spam' : $data['status'];
				$data['is_rolebased']   = ! empty( $res['attributes']['roleBased'] ) ? $res['attributes']['roleBased'] : 0;
				$data['is_sendsafely']  = ! empty( $res['attributes']['sendSafely'] ) ? $res['attributes']['sendSafely'] : 0;
				$data['is_webmail']     = ! empty( $res['attributes']['webmail'] ) ? $res['attributes']['webmail'] : 0;
				$data['is_verified']    = 1;
				if ( ( ! empty( $res['attributes']['disposable'] ) && '' == $res['attributes']['disposable'] ) ) {
					$data['status']  = 'ERROR';
					$data['message'] = 'es_invalid_email_notice';
				}
			}

			return $data;
		}

		public function add_list_cleanup_btn() {

			$es_cleanup_done      = get_option( 'es_cleanup_done' );
			$es_cleanup_done_date = get_option( 'es_cleanup_done_date' );
			$timezone_format      = _x( 'Y-m-d H:i:s', 'timezone date format' );
			$es_current_date      = date_i18n( $timezone_format );
			$date_diff            = floor( ( strtotime( $es_current_date ) - strtotime( $es_cleanup_done_date ) ) / ( 3600 * 24 ) );
			if ( true == $es_cleanup_done && $es_cleanup_done_date && $date_diff < 30 ) {
				return;
			}
			?>
			<span>
				<button value="" class="button" id="es-cleanup"
						titel="<?php echo esc_html__( 'Clean My List', 'email-subscribers' ); ?>"><span><?php echo esc_html__( 'Clean My List', 'email-subscribers' ); ?></span></button>
				<span class="es-notice es-notice-txt"><?php echo esc_html__( 'List cleanup is in progress...', 'email-subscribers' ); ?></span>
				<span class="es-notice-done es-notice-txt"><?php echo esc_html__( 'List cleanup completed successfully.', 'email-subscribers' ); ?></span>
				<img src="<?php echo esc_url( $this->pro_plugin_url ); ?>assets/images/spinner-2x.gif"
					 class="es-loader-img " style="display:none;"/>

			</span>
			<?php
		}

		public function es_add_audience_col( $es_subscribers_col ) {
			global $wpdb;

			$can_show = get_option( 'ig_es_track_ip_address', 'yes' );

			if ( 'yes' === $can_show ) {
				$audience['es_country_flag'] = '<svg class="h-4 w-4 mt-1 text-gray-600" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M3 21v-4m0 0V5a2 2 0 012-2h6.5l1 1H21l-3 6 3 6h-8.5l-1-1H5a2 2 0 00-2 2zm9-13.5V9" /><title>Country flag</title></svg>';
				$es_subscribers_col          = ig_es_array_insert_after( $es_subscribers_col, 'cb', $audience );
			}

			$audience_col['es_email_status'] = __( 'Email status', 'email-subscribers' );
			$audience_col['last_opened_at']  = __( 'Last opened at', 'email-subscribers' );

			$es_subscribers_col = ig_es_array_insert_after( $es_subscribers_col, 'lists', $audience_col );

			return $es_subscribers_col;
		}


		public function es_add_email_status_data( $data, $es_col_data, $contacts_list_class ) {
			switch ( $es_col_data ) {
				case 'es_email_status':
					if ( ( 0 == $data['is_disposable'] ) && ( 0 == $data['is_rolebased'] ) && ( 0 == $data['is_webmail'] ) && ( 1 == $data['is_verified'] ) ) {
						$email_status['status_data'] = '<span title="Verified" class="es_email_icons is_right"></span>';
					} else {
						$email_status['status_data'] = ( 1 == $data['is_rolebased'] ) ? '<span title="Role Based" class="es_email_icons is_rolebased"></span>' : '';
						$email_status['status_data'] .= ( 1 == $data['is_disposable'] ) ? '<span title="Disposable" class="es_email_icons is_disposable"></span>' : '';
						$email_status['status_data'] .= ( 1 == $data['is_webmail'] ) ? ' <span title="Webmail" class="es_email_icons is_webmail"></span>' : '';
						// $email_status['status_data'] .= ($data['is_deliverable'] == 0) ? ' <span title="Not Deliverable" class="es_email_icons is_deliverable"></span>' : '';
						$email_status['status_data'] .= ( 0 == $data['is_verified'] ) ? ' <span title="Not Deliverable" class="es_email_icons is_deliverable"></span>' : '';
					}
					$data['es_email_status'] = $email_status['status_data'];

					return $data;
				case 'es_country_flag':
					$data['es_country_flag'] = '';
					if ( empty( $data['ip_address'] ) || 'UNKNOWN' === $data['ip_address'] || empty( $data['country_code'] ) ) {
						return $data;
					}

					$country_name            = ES_Geolocation::get_countries_iso_code_name_map( $data['country_code'] );
					$data['es_country_flag'] = '<p  title="' . $country_name . '" id="es-flag-icon" class="inline-block flag-icon flag-icon-' . strtolower( $data['country_code'] ) . '"></p>';

					return $data;
				case 'last_opened_at':
					$data['last_opened_at'] = ! empty( $contacts_list_class->items_data['last_opened_at'][ $data['id'] ] ) ? $contacts_list_class->items_data['last_opened_at'][ $data['id'] ] : '-';

					return $data;

				default:
					return $data;
			}
		}

		/**
		 * Method to handle last opened at data
		 *
		 * @param array $data , $contact_ids
		 *
		 * @return array
		 *
		 * @since 4.6.5
		 */
		public function es_subscribers_add_col_data( $data = array(), $contact_ids = array() ) {
			$data['last_opened_at'] = ES()->actions_db->get_last_opened_of_contact_ids( $contact_ids, true );

			return $data;
		}

		/**
		 * Method to show scheduling options
		 *
		 * @param array $broadcast_data
		 *
		 * @return void
		 */
		public function show_scheduling_options( $broadcast_data = array() ) {
			//scheduler helper
			$timezone_format     = _x( 'Y-m-d h:i A', 'timezone date format' );
			$local_date          = '';
			$local_time          = '';
			$scheduled_date_time = ! empty( $broadcast_data['meta']['date'] ) ? $broadcast_data['meta']['date'] : '';
			if ( $scheduled_date_time ) {
				$schedule_date_timestamp = strtotime( $scheduled_date_time );
				$gmt_offset_option       = get_option( 'gmt_offset' );
				$gmt_offset              = ( ! empty( $gmt_offset_option ) ) ? $gmt_offset_option : 0;
				$locat_date_timestamp    = $schedule_date_timestamp + ( $gmt_offset * HOUR_IN_SECONDS );
				$local_date_time         = gmdate( 'Y-m-d H:i A', $locat_date_timestamp );
				$local_date_time_data    = explode( ' ', $local_date_time );
				$local_date              = $local_date_time_data[0];
				$local_time              = $local_date_time_data[1] . ' ' . $local_date_time_data[2];
			}
			$scheduling_option = ! empty( $broadcast_data['meta']['scheduling_option'] ) ? $broadcast_data['meta']['scheduling_option'] : 'schedule_later';
			?>
			<div class="block w-full px-4 pt-2 pb-8">
				<span class="border-t border-gray-200 pt-2 block text-sm font-medium leading-5 text-gray-700"><?php echo esc_html__( 'Send options', 'email-subscribers' ); ?></span>
				<div class="py-2">
					<input type="radio" name="broadcast_data[scheduling_option]" class="form-radio" id="schedule_later"
						   value="schedule_later" <?php checked( $scheduling_option, 'schedule_later' ); ?>>
					<label for="schedule_later"
						   class="text-sm font-medium text-gray-500"><?php echo esc_html__( 'Schedule for later', 'email-subscribers' ); ?>
					</label>
					<br>
					<?php
					if ( ! empty( $scheduling_option ) && 'schedule_now' === $scheduling_option ) {
						$schedule_wrapper_class = 'hidden';
					} else {
						$schedule_wrapper_class = '';
					}
					?>
					<div id="schedule_later"
						 class="display_schedule px-6 <?php echo esc_attr( $schedule_wrapper_class ); ?>">
						<div class="flex pt-4">
							<div class="flex w-full w-11/12">
								<label class="text-sm font-normal leading-5 text-gray-600 pt-1"><?php echo esc_html__( 'Date', 'email-subscribers' ); ?></label>
								<input class="cursor-pointer font-normal text-sm py-1 ml-2 form-input" type="text"
									   id="es_schedule_date" name="broadcast_data[es_schedule_date]"
									   value="<?php echo esc_attr( $local_date ); ?>">
							</div>
							<div>
								<svg fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round"
									 stroke-width="2" viewBox="0 0 24 24" class="text-gray-600 w-5 h-5 my-1 ml-2">
									<path d="M8 7V3m8 4V3m-9 8h10M5 21h14a2 2 0 002-2V7a2 2 0 00-2-2H5a2 2 0 00-2 2v12a2 2 0 002 2z"></path>
								</svg>
							</div>
						</div>
						<div class="flex pt-3">
							<div class="flex w-11/12">
								<label class="text-sm font-normal leading-5 text-gray-600 pt-1"><?php echo esc_html__( 'Time', 'email-subscribers' ); ?></label>
								<input class="es_schedule_time cursor-pointer font-normal text-sm px-3 py-1 ml-2"
									   type="text" id="es_schedule_time" name="broadcast_data[es_schedule_time]"
									   value="<?php echo esc_attr( $local_time ); ?>">
							</div>
							<div>
								<svg fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round"
									 stroke-width="2" viewBox="0 0 24 24"
									 class="text-gray-600 w-5 h-5 my-1 ml-2 float-right">
									<path d="M12 8v4l3 3m6-3a9 9 0 11-18 0 9 9 0 0118 0z"></path>
								</svg>
							</div>
						</div>
						<div class=" pb-3">
							<div class="block px-2 py-2 mt-3 bg-gray-300 rounded-md ">
								<h3 class="text-gray-600 text-sm font-normal"><?php echo esc_html__( 'Local Time: ', 'email-subscribers' ); ?>
									&nbsp;&nbsp;
									<?php echo esc_html__( date_i18n( $timezone_format ) ); ?>
								</h3>
							</div>
						</div>
					</div>
				</div>
				<input type="radio" name="broadcast_data[scheduling_option]" class="form-radio" id="send_now"
					   value="schedule_now" <?php checked( $scheduling_option, 'schedule_now' ); ?>>
				<label for="send_now"
					   class="text-sm font-medium text-gray-500"><?php echo esc_html__( 'Send Now', 'email-subscribers' ); ?></label>
			</div>


			<?php
		}

		public function es_add_settings_fields( $fields ) {
			//add unsubscribe page setting
			$page_ids = get_all_page_ids();
			$pages    = array();
			$pages[0] = __( 'Select page', 'email-subscribers' );
			foreach ( $page_ids as $page_id ) {
				if ( 'publish' === get_post_status( $page_id ) ) {
					$pages[ $page_id ] = get_the_title( $page_id );
				}

			}
			$field_unsubscribe['unsubscribe_page']                      = array(
				'id'          => 'ig_es_unsubscribe_page',
				'name'        => 'Unsubscribe Page',
				'type'        => 'select',
				'options'     => $pages,
				'placeholder' => '',
				'desc'        => __( 'Subscriber will be redirected to selected page (by default, homepage) once they click on unsubscribe link from the email.', 'email-subscribers' ),
				'default'     => ''
			);
			$fields['general']['unsubscription_messages']['sub_fields'] = array_merge( $field_unsubscribe, $fields['general']['unsubscription_messages']['sub_fields'] );
			unset( $fields['general']['unsubscription_messages']['sub_fields']['ig_es_subscription_success_message'] );

			//add optin page setting
			$field_optin['optin_page']                                = array(
				'id'          => 'ig_es_optin_page',
				'name'        => 'Double opt-in page',
				'type'        => 'select',
				'options'     => $pages,
				'placeholder' => '',
				'desc'        => __( 'Subscriber will be redirected to selected page (by default, homepage) once they click on email confirmation link from the double opt-in (confirmation) email.', 'email-subscribers' ),
				'default'     => ''
			);
			$fields['general']['subscription_messages']['sub_fields'] = array_merge( $field_optin, $fields['general']['subscription_messages']['sub_fields'] );
			unset( $fields['general']['subscription_messages']['sub_fields']['ig_es_subscribe_success_message'] );

			$track_link_click = array(
				'ig_es_track_link_click' => array(
					'id'      => 'ig_es_track_link_click',
					'name'    => __( 'Track clicks', 'email-subscribers' ),
					'info'    => __( 'Do you want to track when people click links in your emails? (We recommend keeping it enabled)', 'email-subscribers' ),
					'type'    => 'checkbox',
					'default' => 'no'
				)
			);

			$track_contact_ip_address['track_contact_ip_address'] = array(
				'id'      => 'ig_es_track_ip_address',
				'name'    => __( 'Track IP address', 'email-subscribers' ),
				'type'    => 'checkbox',
				'info'    => __( 'Record user\'s IP address on subscription' ),
				'default' => 'yes',
			);

			$fields['security_settings'] = array_merge( $fields['security_settings'], $track_contact_ip_address );

			$general_fields = $fields['general'];

			$general_fields = ig_es_array_insert_after( $general_fields, 'ig_es_track_email_opens', $track_link_click );

			$summary_automation = array(
				'summary_automation' => array(
						'id'         => 'summary_automation',
						'name'       => __( 'Weekly summary', 'email-subscribers' ),
						'sub_fields' => array(

								'ig_es_enable_summary_automation' => array(
										'id'      => 'ig_es_enable_summary_automation',
										'name'    => __( 'Enable?', 'email-subscribers' ),
										'type'    => 'checkbox',
										'default' => 'yes',
								),

								'ig_es_run_cron_on' => array(
										'id'      => 'ig_es_run_cron_on',
										'name'    => __( 'When our automated weekly email should be sent out?', 'email-subscribers' ),
										'desc'    => '',
										'type'    => 'select',
										'options' => ES_Common::run_summary_cron_on(),
										'default' => 'monday',
								),

								'ig_es_run_cron_time' => array(
										'id'      => 'ig_es_run_cron_time',
										'name'    => __( 'In which time we need to send our weekly summary?', 'email-subscribers' ),
										'desc'    => '',
										'type'    => 'select',
										'options' => ES_Common::get_railway_hrs_timings(),
										'default' => '4pm',
								),
						),
				)
		);

		$general_fields = ig_es_array_insert_after( $general_fields, 'unsubscription_messages', $summary_automation );

			$fields['general'] = $general_fields;

			$amazon_ses_access_key_id_defined     = ES()->is_const_defined( 'Amazon_SES', 'access_key_id' );
			$amazon_ses_secret_access_key_defined = ES()->is_const_defined( 'Amazon_SES', 'secret_access_key' );
			$amazon_ses_region_defined            = ES()->is_const_defined( 'Amazon_SES', 'region' );

			// Amazon SES settings
			$amazon_ses_fields                                              = array(
				'ig_es_amazon_ses_access_key_id' => array(
					'type'         => $amazon_ses_access_key_id_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[Amazon_SES][access_key_id]',
					'name'         => __( 'Access Key ID', 'email-subscribers' ),
					'desc'         => $amazon_ses_access_key_id_defined ? ES()->get_const_set_message( 'Amazon_SES', 'access_key_id' ) : '',
					'class'        => 'Amazon_SES',
					'disabled'     => $amazon_ses_access_key_id_defined ? 'disabled' : '',
					'value'        => $amazon_ses_access_key_id_defined ? '******************' : '',
				),

				'ig_es_amazon_ses_secret_access_key' => array(
					'type'         => $amazon_ses_secret_access_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[Amazon_SES][secret_access_key]',
					'name'         => __( 'Secret Access Key', 'email-subscribers' ),
					'desc'         => $amazon_ses_secret_access_key_defined ? ES()->get_const_set_message( 'Amazon_SES', 'secret_access_key' ) : '',
					'class'        => 'Amazon_SES',
					'disabled'     => $amazon_ses_secret_access_key_defined ? 'disabled' : '',
					'value'        => $amazon_ses_secret_access_key_defined ? '******************' : '',
				),

				'ig_es_amazon_ses_region' => array(
					'type'         => 'select',
					'options'      => ES_Amazon_SES_Mailer::get_regions_names(),
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[Amazon_SES][region]',
					'name'         => __( 'Closest Region', 'email-subscribers' ),
					'desc'         => $amazon_ses_region_defined ? ES()->get_const_set_message( 'Amazon_SES', 'region' ) : '',
					'tooltip_text' => __( 'To decrease network latency between your site and Amazon SES and speed up email sending, select the Amazon SES API region which is closest to where your website is hosted.', 'email-subscribers' ),
					'class'        => 'Amazon_SES',
					'disabled'     => $amazon_ses_region_defined ? 'disabled' : '',
					'value'        => $amazon_ses_region_defined ? ES()->get_const_value( 'Amazon_SES', 'region' ) : '',
				),

				'ig_es_amazon_ses_docblock' => array(
					'type' => 'html',
					'html' => $this->get_amazon_ses_doc_block(),
					'id'   => 'ig_es_amazon_ses_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $amazon_ses_fields );

			$mailgun_private_api_key_defined = ES()->is_const_defined( 'mailgun', 'private_api_key' );
			$mailgun_domain_name_defined     = ES()->is_const_defined( 'mailgun', 'domain_name' );
			$mailgun_region_defined          = ES()->is_const_defined( 'mailgun', 'region' );

			// Mailgun settings
			$mailgun_fields                                                 = array(
				'ig_es_mailgun_private_api_key' => array(
					'type'         => $mailgun_private_api_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[mailgun][private_api_key]',
					'name'         => __( 'Private API Key', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $mailgun_private_api_key_defined ? ES()->get_const_set_message( 'mailgun', 'private_api_key' ) : '',
					'class'        => 'mailgun',
					'disabled'     => $mailgun_private_api_key_defined ? 'disabled' : '',
					'value'        => $mailgun_private_api_key_defined ? '******************' : '',
				),

				'ig_es_mailgun_domain_name' => array(
					'type'         => 'text',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[mailgun][domain_name]',
					'name'         => __( 'Domain Name', 'email-subscribers' ),
					/* translators: %s - Domain Name link. */
					'desc'         => $mailgun_domain_name_defined ? ES()->get_const_set_message( 'mailgun', 'domain_name' ) : '',
					'class'        => 'mailgun',
					'disabled'     => $mailgun_domain_name_defined ? 'disabled' : '',
					'value'        => $mailgun_domain_name_defined ? '******************' : '',
				),

				'ig_es_mailgun_region' => array(
					'type'         => 'select',
					'options'      => array(
						'us' => __( 'United States', 'email-subscribers' ),
						'eu' => __( 'Europe', 'email-subscribers' ),
					),
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[mailgun][region]',
					'name'         => __( 'Region', 'email-subscribers' ),
					'desc'         => ( $mailgun_region_defined ? ES()->get_const_set_message( 'mailgun', 'region' ) : '' ),
					/* translators: %s - Mailgun Regions link. */
					'tooltip_text' => __( sprintf( 'Define which endpoint you want to use for sending messages. If you are operating under EU laws, you may be required to use EU region. More information on %s', '<a href="https://www.mailgun.com/regions" target="_blank" rel="noopener noreferrer">' .
																																																				   __( 'mailgun.com', 'email-subscribers' ) .
																																																				   '</a>' ), 'email-subscribers' ),
					'class'        => 'mailgun',
					'disabled'     => $mailgun_region_defined ? 'disabled' : '',
					'value'        => $mailgun_region_defined ? ES()->get_const_value( 'mailgun', 'region' ) : '',
				),

				'ig_es_mailgun_docblock' => array(
					'type' => 'html',
					'html' => $this->get_mailgun_doc_block(),
					'id'   => 'ig_es_mailgun_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $mailgun_fields );

			$sendgrid_api_key_defined = ES()->is_const_defined( 'sendgrid', 'api_key' );

			// Sendgrid settings
			$sendgrid_fields                                                = array(
				'ig_es_sendgrid_api_key' => array(
					'type'         => $sendgrid_api_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[sendgrid][api_key]',
					'name'         => __( 'API Key', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $sendgrid_api_key_defined ? ES()->get_const_set_message( 'sendgrid', 'api_key' ) : '',
					'class'        => 'sendgrid',
					'disabled'     => $sendgrid_api_key_defined ? 'disabled' : '',
					'value'        => $sendgrid_api_key_defined ? '******************' : '',
				),

				'ig_es_sendgrid_docblock' => array(
					'type' => 'html',
					'html' => $this->get_sendgrid_doc_block(),
					'id'   => 'ig_es_sendgrid_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $sendgrid_fields );

			$sparkpost_api_key_defined = ES()->is_const_defined( 'sparkpost', 'api_key' );
			$sparkpost_region_defined  = ES()->is_const_defined( 'sparkpost', 'region' );

			$sparkpost_fields                                               = array(
				'ig_es_sparkpost_api_key' => array(
					'type'         => $sparkpost_api_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[sparkpost][api_key]',
					'name'         => __( 'API Key', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $sparkpost_api_key_defined ? ES()->get_const_set_message( 'sparkpost', 'api_key' ) : '',
					'class'        => 'sparkpost',
					'disabled'     => $sparkpost_api_key_defined ? 'disabled' : '',
					'value'        => $sparkpost_api_key_defined ? '******************' : '',
				),

				'ig_es_sparkpost_region' => array(
					'type'         => 'select',
					'options'      => array(
						'us' => __( 'United States', 'email-subscribers' ),
						'eu' => __( 'Europe', 'email-subscribers' ),
					),
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[sparkpost][region]',
					'name'         => __( 'Region', 'email-subscribers' ),
					'desc'         => $sparkpost_region_defined ? ES()->get_const_set_message( 'sparkpost', 'region' ) : '',
					'tooltip_text' => __( 'Define which endpoint you want to use for sending messages. If you are operating under EU laws, you may be required to use EU region.', 'email-subscribers' ),
					'class'        => 'sparkpost',
					'disabled'     => $sparkpost_region_defined ? 'disabled' : '',
					'value'        => $sparkpost_region_defined ? ES()->get_const_value( 'sparkpost', 'region' ) : '',
				),

				'ig_es_sparkpost_docblock' => array(
					'type' => 'html',
					'html' => $this->get_sparkpost_doc_block(),
					'id'   => 'ig_es_sparkpost_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $sparkpost_fields );

			$postmark_api_token_defined = ES()->is_const_defined( 'postmark', 'api_token' );

			// Sendgrid settings
			$postmark_fields                                                = array(
				'ig_es_postmark_api_token' => array(
					'type'         => $postmark_api_token_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[postmark][api_token]',
					'name'         => __( 'API token', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $postmark_api_token_defined ? ES()->get_const_set_message( 'postmark', 'api_token' ) : '',
					'class'        => 'postmark',
					'disabled'     => $postmark_api_token_defined ? 'disabled' : '',
					'value'        => $postmark_api_token_defined ? '******************' : '',
				),

				'ig_es_postmark_docblock' => array(
					'type' => 'html',
					'html' => $this->get_postmark_doc_block(),
					'id'   => 'ig_es_postmark_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $postmark_fields );


			$sendinblue_api_key_defined = ES()->is_const_defined( 'sendinblue', 'api_token' );

			// Sendgrid settings
			$sendinblue_fields                                                = array(
				'ig_es_sendinblue_api_token' => array(
					'type'         => $sendinblue_api_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[sendinblue][api_token]',
					'name'         => __( 'API key', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $sendinblue_api_key_defined ? ES()->get_const_set_message( 'sendinblue', 'api_token' ) : '',
					'class'        => 'sendinblue',
					'disabled'     => $sendinblue_api_key_defined ? 'disabled' : '',
					'value'        => $sendinblue_api_key_defined ? '******************' : '',
				),

				'ig_es_sendinblue_docblock' => array(
					'type' => 'html',
					'html' => $this->get_sendinblue_doc_block(),
					'id'   => 'ig_es_sendinblue_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $sendinblue_fields );


			$mailjet_public_key_defined = ES()->is_const_defined( 'mailjet', 'public_key' );
			$mailjet_private_key_defined = ES()->is_const_defined( 'mailjet', 'private_key' );

			// Mailjet settings
			$mailjet_fields                                                = array(
				'ig_es_mailjet_public_key' => array(
					'type'         => $mailjet_public_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[mailjet][public_key]',
					'name'         => __( 'Public key', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $mailjet_public_key_defined ? ES()->get_const_set_message( 'mailjet', 'public_key' ) : '',
					'class'        => 'mailjet',
					'disabled'     => $mailjet_public_key_defined ? 'disabled' : '',
					'value'        => $mailjet_public_key_defined ? '******************' : '',
				),

				'ig_es_mailjet_private_key' => array(
					'type'         => $mailjet_private_key_defined ? 'text' : 'password',
					'placeholder'  => '',
					'supplemental' => '',
					'default'      => '',
					'id'           => 'ig_es_mailer_settings[mailjet][private_key]',
					'name'         => __( 'Private key', 'email-subscribers' ),
					/* translators: %s - API key link. */
					'desc'         => $mailjet_private_key_defined ? ES()->get_const_set_message( 'mailjet', 'private_key' ) : '',
					'class'        => 'mailjet',
					'disabled'     => $mailjet_private_key_defined ? 'disabled' : '',
					'value'        => $mailjet_private_key_defined ? '******************' : '',
				),

				'ig_es_mailjet_docblock' => array(
					'type' => 'html',
					'html' => $this->get_mailjet_doc_block(),
					'id'   => 'ig_es_mailjet_docblock',
					'name' => '',
				),
			);
			$fields['email_sending']['ig_es_mailer_settings']['sub_fields'] = array_merge( $fields['email_sending']['ig_es_mailer_settings']['sub_fields'], $mailjet_fields );

			return $fields;
		}

		public function es_optin_redirection( $es_page_request ) {
			if ( ! empty( $es_page_request ) ) {
				$es_page_option_name = ( 'unsubscribe' === $es_page_request ) ? 'ig_es_unsubscribe_page' : 'ig_es_optin_page';
				$es_redirection_page = (int) get_option( $es_page_option_name, 0 );
				if ( $es_redirection_page > 0 ) {
					$es_redirection_url = get_page_link( $es_redirection_page );
					wp_safe_redirect( $es_redirection_url );
					exit;
				}
			}
		}

		public function es_contact_click_kpi() {
			$es_total_contacts_clicks_links = ES_Reports_Data::get_total_contacts_clicks_links( 60 );
			?>
			<div class="mb-1">
				<span class="text-2xl font-bold leading-none text-gray-700"><?php echo number_format( $es_total_contacts_clicks_links ); ?></span>
			</div>
			<div class="text-sm text-gray-400 tracking-wide"><?php echo esc_html__( 'Clicked', 'email-subscribers' ); ?>
			</div>
			<?php
		}

		/**
		 * Register workflow triggers
		 *
		 * @param array $workflow_triggers
		 *
		 * @return array $workflow_triggers
		 *
		 * @since 4.5.3
		 */
		public function register_workflow_triggers( $workflow_triggers = array() ) {

			global $ig_es_tracker;

			if ( empty( $workflow_triggers ) || ! is_array( $workflow_triggers ) ) {
				$workflow_triggers = array();
			}

			$active_plugins = $ig_es_tracker::get_active_plugins();

			// Store required plugins and triggers dependent on it in key-value pairs.
			$required_plugins = array(
				'woocommerce/woocommerce.php' => array(
					'ig_es_wc_order_refunded'          => 'ES_Trigger_WC_Order_Refunded',
					'ig_es_wc_product_review_approved' => 'ES_Trigger_WC_Product_Review_Approved',
					'ig_es_wc_cart_abandoned'          => 'IG_ES_Trigger_WC_Cart_Abandoned',
					'ig_es_wc_user_cart_abandoned'     => 'IG_ES_Trigger_WC_User_Cart_Abandoned',
					'ig_es_wc_guest_cart_abandoned'    => 'IG_ES_Trigger_WC_Guest_Cart_Abandoned',
				),
			);

			foreach ( $required_plugins as $required_plugin => $trigger_data ) {
				// Check if required plugin is active.
				if ( in_array( $required_plugin, $active_plugins ) ) {
					foreach ( $trigger_data as $trigger_event => $trigger_handler_class ) {
						$workflow_triggers[ $trigger_event ] = $trigger_handler_class;
					}
				}
			}

			$wishlist_integration = IG_ES_Wishlists::get_integration();
			if ( $wishlist_integration ) {
				$workflow_triggers['ig_es_wc_wishlist_item_goes_on_sale'] = 'IG_ES_Trigger_Wishlist_Item_Goes_On_Sale';
			}

			$workflow_triggers['ig_es_user_role_changed'] = 'ES_Trigger_User_Role_Changed';

			return $workflow_triggers;
		}

		/**
		 * Register workflow data types
		 *
		 * @param array $data_types
		 *
		 * @return array $data_types
		 *
		 * @since 4.5.3
		 */
		public function register_workflow_data_types( $data_types = array() ) {

			global $ig_es_tracker;

			if ( empty( $data_types ) || ! is_array( $data_types ) ) {
				$data_types = array();
			}

			$active_plugins = $ig_es_tracker::get_active_plugins();

			// Store required plugins and data types dependent on it in key-value pairs.
			$required_plugins = array(
				'woocommerce/woocommerce.php' => array(
					'review'   => 'ES_Data_Type_Review',
					'customer' => 'IG_ES_Data_Type_Customer',
					'guest'    => 'IG_ES_Data_Type_Guest',
					'cart'     => 'IG_ES_Data_Type_Cart',
					'wishlist' => 'IG_ES_Data_Type_Wishlist',
				),
			);

			foreach ( $required_plugins as $required_plugin => $required_data_types ) {
				// Check if required plugin is active.
				if ( in_array( $required_plugin, $active_plugins ) ) {
					foreach ( $required_data_types as $data_type => $data_type_handler ) {
						$data_types[ $data_type ] = $data_type_handler;
					}
				}
			}

			return $data_types;
		}

		/**
		 * Register workflow variables
		 *
		 * @param array $variables
		 *
		 * @return array $variables
		 *
		 * @since 4.6.9
		 */
		public function register_workflow_variables( $variables = array() ) {

			global $ig_es_tracker;

			if ( empty( $variables ) || ! is_array( $variables ) ) {
				$variables = array();
			}

			$active_plugins = $ig_es_tracker::get_active_plugins();

			// Show only if WooCommerce is installed & activated
			$woocommerce_plugin = 'woocommerce/woocommerce.php';
			if ( in_array( $woocommerce_plugin, $active_plugins, true ) ) {

				$variables['cart']['link']       = $this->pro_plugin_path . '/workflows/variables/cart-link.php';
				$variables['cart']['item_count'] = $this->pro_plugin_path . '/workflows/variables/cart-item-count.php';
				$variables['cart']['items']      = $this->pro_plugin_path . '/workflows/variables/cart-items.php';
				$variables['cart']['total']      = $this->pro_plugin_path . '/workflows/variables/cart-total.php';

				$variables['guest']['id']             = $this->pro_plugin_path . '/workflows/variables/guest-id.php';
				$variables['guest']['email']          = $this->pro_plugin_path . '/workflows/variables/guest-email.php';
				$variables['guest']['first_name']     = $this->pro_plugin_path . '/workflows/variables/guest-first-name.php';
				$variables['guest']['last_name']      = $this->pro_plugin_path . '/workflows/variables/guest-last-name.php';
				$variables['guest']['full_name']      = $this->pro_plugin_path . '/workflows/variables/guest-full-name.php';
				$variables['guest']['country']        = $this->pro_plugin_path . '/workflows/variables/guest-country.php';
				$variables['guest']['state']          = $this->pro_plugin_path . '/workflows/variables/guest-state.php';
				$variables['guest']['phone']          = $this->pro_plugin_path . '/workflows/variables/guest-phone.php';
				$variables['guest']['company']        = $this->pro_plugin_path . '/workflows/variables/guest-company.php';
				$variables['guest']['address_line_1'] = $this->pro_plugin_path . '/workflows/variables/guest-address-line-1.php';
				$variables['guest']['address_line_2'] = $this->pro_plugin_path . '/workflows/variables/guest-address-line-2.php';
				$variables['guest']['postcode']       = $this->pro_plugin_path . '/workflows/variables/guest-postcode.php';
				$variables['guest']['city']           = $this->pro_plugin_path . '/workflows/variables/guest-city.php';

				$variables['customer']['user_id']        = $this->pro_plugin_path . '/workflows/variables/customer-user-id.php';
				$variables['customer']['username']       = $this->pro_plugin_path . '/workflows/variables/customer-username.php';
				$variables['customer']['email']          = $this->pro_plugin_path . '/workflows/variables/customer-email.php';
				$variables['customer']['first_name']     = $this->pro_plugin_path . '/workflows/variables/customer-first-name.php';
				$variables['customer']['last_name']      = $this->pro_plugin_path . '/workflows/variables/customer-last-name.php';
				$variables['customer']['full_name']      = $this->pro_plugin_path . '/workflows/variables/customer-full-name.php';
				$variables['customer']['country']        = $this->pro_plugin_path . '/workflows/variables/customer-country.php';
				$variables['customer']['state']          = $this->pro_plugin_path . '/workflows/variables/customer-state.php';
				$variables['customer']['phone']          = $this->pro_plugin_path . '/workflows/variables/customer-phone.php';
				$variables['customer']['company']        = $this->pro_plugin_path . '/workflows/variables/customer-company.php';
				$variables['customer']['address_line_1'] = $this->pro_plugin_path . '/workflows/variables/customer-address-line-1.php';
				$variables['customer']['address_line_2'] = $this->pro_plugin_path . '/workflows/variables/customer-address-line-2.php';
				$variables['customer']['postcode']       = $this->pro_plugin_path . '/workflows/variables/customer-postcode.php';
				$variables['customer']['city']           = $this->pro_plugin_path . '/workflows/variables/customer-city.php';

				$variables['coupon']['generate_unique_coupon'] = $this->pro_plugin_path . '/workflows/variables/coupon-generate-unique-coupon.php';

				$variables['wishlist']['items']       = $this->pro_plugin_path . '/workflows/variables/wishlist-items.php';
				$variables['wishlist']['items_count'] = $this->pro_plugin_path . '/workflows/variables/wishlist-items-count.php';
				$variables['wishlist']['view_link']   = $this->pro_plugin_path . '/workflows/variables/wishlist-view-link.php';
			}

			return $variables;
		}

		/**
		 * Register workflow actions
		 *
		 * @param array $workflow_actions
		 *
		 * @return array $workflow_actions
		 *
		 * @since 4.4.1
		 */
		public function register_workflow_actions( $workflow_actions = array() ) {

			if ( empty( $workflow_actions ) || ! is_array( $workflow_actions ) ) {
				$workflow_actions = array();
			}

			$pro_actions = array(
				'ig_es_move_to_list'     => 'ES_Action_Move_To_List',
				'ig_es_remove_from_list' => 'ES_Action_Remove_From_List',
				'ig_es_send_email' 		 => 'ES_Action_Send_Email',
			);

			$workflow_actions = array_merge( $workflow_actions, $pro_actions );

			return $workflow_actions;
		}

		/**
		 * Function to add values of checkbox fields incase they are not checked.
		 *
		 * @param array $broadcast_data
		 *
		 * @return array $broadcast_data
		 *
		 * @since 4.4.7
		 *
		 */
		public function add_tracking_fields_data( $broadcast_data = array() ) {

			$broadcast_meta = ! empty( $broadcast_data['meta'] ) ? maybe_unserialize( $broadcast_data['meta'] ) : array();

			if ( empty( $broadcast_meta['enable_link_tracking'] ) ) {
				$broadcast_meta['enable_link_tracking'] = 'no';
			}

			if ( empty( $broadcast_meta['enable_utm_tracking'] ) ) {
				$broadcast_meta['enable_utm_tracking'] = 'no';
			}

			$broadcast_data['meta'] = maybe_serialize( $broadcast_meta );

			return $broadcast_data;
		}

		/**
		 * Register services in the pro version
		 *
		 * @param array $services
		 *
		 * @return array $services
		 *
		 * @since 4.6.1
		 */
		public function register_services( $services = array() ) {

			$pro_services = array(
				'spam_score_check',
				'utm_tracking',
			);

			$services = array_merge( $services, $pro_services );

			return $services;
		}

		/**
		 * Method to get plugin plan
		 *
		 * @param string $plan
		 *
		 * @return string $plan
		 *
		 * @since 4.6.1
		 */
		public function get_plan( $plan = '' ) {

			$plan = 'pro';

			return $plan;
		}

		/**
		 * Method to add Duplicate broadcast option
		 *
		 * @param array $broadcast_data
		 *
		 * @return array
		 *
		 * @since 4.6.3
		 */
		public function add_duplicate_campaign_action( $actions = array(), $data = array() ) {

			$es_nonce = wp_create_nonce( 'es_notification' );
			$is_workflow_campaign = 'workflow' === $data['type'];
			// Don't provide duplicate option for workflow campaign
			if ( ! isset( $actions['duplicate'] ) && ! $is_workflow_campaign ) {
				/* translators: 1: Page 2: Duplicate action 3: Campaign id 4: Wp nonce */
				$actions['duplicate'] = '<a href="?page=' . esc_attr( 'es_campaigns' ) . '&action=duplicate&list=' . absint( $data['id'] ) . '&_wpnonce=' . $es_nonce . '">' . esc_html__( 'Duplicate', 'email-subscribers' ) . '</a>';
			}

			return $actions;

		}

		/**
		 * Method to Duplicate broadcast content
		 *
		 * @return void
		 *
		 * @since 4.6.3
		 */
		public function duplicate_campaign( $campaign_id = 0, $campaign_action = 'duplicate' ) {

			$campaign_duplicated = ig_es_get_request_data( 'duplicate', false );
			if ( 'duplicate' === $campaign_action ) {
				// In our file that handles the request, verify the nonce.
				$nonce = ig_es_get_request_data( '_wpnonce' );

				if ( ! wp_verify_nonce( $nonce, 'es_notification' ) ) {
					$message = __( 'You are not allowed to duplicate campaign.', 'email-subscribers' );
					$status  = 'error';
					ES_Common::show_message( $message, $status );
				} else {
					if ( ! $campaign_duplicated ) {
						ES()->campaigns_db->duplicate_campaign( $campaign_id );
						// Avoid duplicating it again on refresh
						$campaign_page_url = add_query_arg( 'duplicate', true, menu_page_url( 'es_campaigns' ) );
						wp_safe_redirect( $campaign_page_url );
						exit;
					}
				}

			}
			if ( $campaign_duplicated ) {
				$message = __( 'Campaign duplicated !', 'email-subscribers' );
				$status  = 'success';
				ES_Common::show_message( $message, $status );
			}
		}

		/**
		 * Method to get subscriber ip if tracking enabled
		 *
		 * @return string
		 *
		 * @since 4.6.5
		 * @since 4.6.10 Get subscriber ip if ip tracking option enabled
		 */
		public function es_get_subscriber_ip( $contact_data, $column_name ) {
			$can_track = get_option( 'ig_es_track_ip_address', 'yes' );

			if ( 'no' === $can_track ) {
				$contact_data[ $column_name ] = '';
			} else {
				if ( empty( $contact_data[ $column_name ] ) ) {
					$contact_data[ $column_name ] = ig_es_get_ip();
				}
			}

			return $contact_data;
		}

		/**
		 * Get country code based on ip for new subscribers
		 *
		 * @return string
		 *
		 * @since 4.6.7
		 */
		public function es_get_ip_related_country_data( $item = array() ) {

			$item['country_code'] = '';
			if ( empty( $item['ip_address'] ) || 'UNKNOWN' === $item['ip_address'] ) {
				return $item;
			}

			$device_country_code  = ES_Geolocation::geolocate_ip( $item['ip_address'] );
			$item['country_code'] = ! empty( $device_country_code['country_code'] ) ? $device_country_code['country_code'] : '';
			$item['timezone']     = ! empty( $device_country_code['timezone'] ) ? $device_country_code['timezone'] : 0;

			return $item;
		}

		/**
		 * Method to add country code and timezone to the contacts table in background. Gets called through the Action Scheduler library.
		 *
		 * @modified 5.0.2
		 * @since 4.6.7
		 */
		public function add_country_code_to_audience() {

			global $wpbd;

			$api_endpoint    = 'http://ip-api.com/batch';
			$subscriber_data = $wpbd->get_results( "SELECT id, ip_address FROM {$wpbd->prefix}ig_contacts WHERE (country_code IS NULL OR timezone IS NULL) AND ( ip_address <> ''  OR ip_address IS NOT NULL ) GROUP BY ip_address", ARRAY_A );

			if ( ! empty( $subscriber_data ) ) {
				// API can handle 100 IP address in one request and 15 requests per minute and so we are creating batches of same size
				$subscribers_batch_size = 100;

				// Create batches of subscribers each containing maximum subscribers equal to $subscribers_batch_size.
				$total_subscribers_batches = array_chunk( $subscriber_data, $subscribers_batch_size );

				// Calculate how many batches we need to send per hour
				$total_hours = ceil( count( $total_subscribers_batches ) / 10 );

				$batch_count = 0;
				foreach ( $total_subscribers_batches as $batch => $subscribers ) {

					//Since we can send max 15 requests each consisting of 100 ips per minute, we will run 10 batches of request per hour.
					if ( $batch_count < 10 && ! empty( $total_subscribers_batches ) ) {

						$ip_addresses = array_column( $subscribers, 'ip_address' );
						// Created array of IP address only
						$ip_address_list = json_encode( array_values( $ip_addresses ) );

						$response = wp_remote_post( $api_endpoint, array(
							'headers'     => array( 'Content-Type' => 'application/json; charset=utf-8' ),
							'body'        => $ip_address_list,
							'method'      => 'POST',
							'data_format' => 'body',
						) );

						$response_body             = json_decode( $response['body'], true );
						$ip_timezone_country_array = array();
						foreach ( $response_body as $key => $data ) {
							$country_code = '';
							$timezone     = '+0:00';
							if ( array_key_exists( 'countryCode', $data ) ) {
								$country_code = $data['countryCode'];
							}
							if ( array_key_exists( 'timezone', $data ) ) {
								$timezone = ES_Geolocation::convert_timezone_to_seconds( $data['timezone'] );
							}
							array_push( $ip_timezone_country_array, array( $country_code, $timezone ) );
						}

						$ip_country_array = array_combine( array_values( $ip_addresses ), $ip_timezone_country_array );
						foreach ( $ip_country_array as $ip => $details ) {
							list( $code, $timezone ) = $details;
							$wpbd->query( $wpbd->prepare( "UPDATE {$wpbd->prefix}ig_contacts SET country_code = %s, timezone = %s WHERE ip_address = %s", $code, $timezone, $ip ) );
						}

						unset( $total_subscribers_batches[ $batch ] );
						$batch_count ++;
					} else {
						$total_hours --;
						break;
					}
				}

				if ( $total_hours > 0 && ! empty( $total_subscribers_batches ) ) {
					IG_ES_Background_Process_Helper::add_action_scheduler_task( 'ig_es_add_country_code_to_audience', array(), false, true, time() + HOUR_IN_SECONDS );
				}

			}
		}

		/**
		 * Add an attachment button to WP Editor
		 *
		 * @param string $editor_id Editor id
		 *
		 * @since 4.6.7
		 */
		public function add_attachment_button( $editor_id ) {

			if ( ! ES()->is_es_admin_screen() ) {
				return;
			}

			$page_prefix           = ES()->get_admin_page_prefix();
			$attachments           = array();
			$attachment_field_name = '';
			$add_attachment_button = false;

			if ( ES()->is_es_admin_screen( $page_prefix . '_page_es_newsletters' ) ) {
				$add_attachment_button = true;
				$attachment_field_name = 'campaign_data[meta][attachments][]';
				$campaign_id           = ig_es_get_request_data( 'list' );
			} elseif ( false !== strpos( $editor_id, 'edit-sequence-message-' ) ) {
				$sub_string_length = strlen( 'edit-sequence-message-' );
				// Get campaign id from editor id.
				$campaign_id           = substr( $editor_id, strpos( $editor_id, 'edit-sequence-message-' ) + $sub_string_length );
				$attachment_field_name = 'seq_data[' . $campaign_id . '][attachments][]';
				$add_attachment_button = true;
			} elseif ( ES()->is_es_admin_screen( $page_prefix . '_page_es_sequence' ) ) {
				$add_attachment_button = true;
				$attachment_field_name = 'seq_data[{seq_message_id}][attachments][]';
			} elseif ( false !== strpos($editor_id, 'ig-es-workflow-email-content')) {
				$sub_string_length     = strlen( 'ig-es-workflow-email-content-' );
				$campaign_id           = substr( $editor_id, strpos( $editor_id, 'ig-es-workflow-email-content-' ) + $sub_string_length );
				$add_attachment_button = true;
				$attachment_field_name = 'ig_es_workflow_data[actions][{action_id}][attachments][]';
			} elseif ( ES()->is_es_admin_screen( $page_prefix . '_page_es_workflows' ) ) {
				$add_attachment_button = true;
				$attachment_field_name = 'ig_es_workflow_data[actions][{action_id}][attachments][]';
			}

			// Dont add the attachment button.
			if ( ! $add_attachment_button ) {
				return;
			}

			if ( ! empty( $campaign_id ) ) {
				$campaign_meta = ES()->campaigns_db->get_campaign_meta_by_id( $campaign_id );
				if ( ! empty( $campaign_meta['attachments'] ) ) {
					$attachments = $campaign_meta['attachments'];
				}
			}
			?>
			<div class="ig-es-attachments-wrapper bg-white inline-block">
				<button type="button" class="ig-es-add-attachment button">
					<svg class="flex-shrink-0 h-5 text-gray-400 inline-block" xmlns="http://www.w3.org/2000/svg"
						 viewBox="0 0 20 20" fill="currentColor" aria-hidden="true">
						<path fill-rule="evenodd"
							  d="M8 4a3 3 0 00-3 3v4a5 5 0 0010 0V7a1 1 0 112 0v4a7 7 0 11-14 0V7a5 5 0 0110 0v4a3 3 0 11-6 0V7a1 1 0 012 0v4a1 1 0 102 0V7a3 3 0 00-3-3z"
							  clip-rule="evenodd"></path>
					</svg>
					<?php echo esc_html__( 'Add Attachments', 'email-subscribers' ); ?>
				</button>
				<div class="my-1 shadow-inner text-sm text-gray-900 ig-es-attachment-container">
					<ul class="list-none ig-es-attachments border border-gray-100 rounded-md divide-y divide-gray-200">
						<li class="inline ig-es-attachment pl-3 pr-4 py-1 flex items-center justify-between text-sm"
							style="display:none;">
							<span class="w-0 flex-1 flex items-center">
								<img width="20" height="20" style="max-width: none"
									 class="ig-es-attachment-thumbnail size-thumbnail">
								<span class="ig-es-attachment-label ml-2 flex-1">
								</span>
								<input type="hidden" data-name="<?php echo esc_attr( $attachment_field_name ); ?>"/>
								<a href="#"
								   class="ig-es-delete-attachment font-medium text-gray-300 hover:text-gray-500">
									<svg class="w-6 h-6 text-gray-400 hover:text-gray-500 " fill="none"
										 stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path
												stroke-linecap="round" stroke-linejoin="round" stroke-width="2"
												d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path></svg>
								</a>
							</span>
						</li>
						<?php
						if ( ! empty( $attachments ) ) {
							foreach ( $attachments as $attachment_id ) {
								$attachment_file = get_attached_file( $attachment_id );
								$attachment_name = basename( $attachment_file );
								$attachment_icon = wp_get_attachment_image_src( $attachment_id, 'thumbnail', true );
								?>
								<li class="inline ig-es-attachment pl-3 pr-4 py-1 flex items-center justify-between text-sm">
								<span class="w-0 flex-1 flex items-center">
									<?php
									if ( ! empty( $attachment_icon[0] ) ) {
										$attachment_icon_url = $attachment_icon[0];
										?>
										<img width="20" height="20" src="<?php echo esc_url( $attachment_icon_url ); ?>"
											 style="max-width: none" class="ig-es-attachment-thumbnail size-thumbnail">
										<?php
									}
									?>
									<span class="w-auto ig-es-attachment-label ml-2 flex-1">
										<?php echo esc_html( $attachment_name ); ?>
									</span>
									<input type="hidden" name="<?php echo esc_attr( $attachment_field_name ); ?>"
										   value="<?php echo esc_attr( $attachment_id ); ?>"/>
									<a href="#"
									   class="ig-es-delete-attachment font-medium text-gray-300 hover:text-gray-500 vertical-align pt-1">
										<svg class="w-5 h-5 font-normal text-gray-400 hover:text-gray-500 " fill="none"
											 stroke="currentColor" viewBox="0 0 24 24"
											 xmlns="http://www.w3.org/2000/svg"><path stroke-linecap="round"
																					  stroke-linejoin="round"
																					  stroke-width="2"
																					  d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"></path></svg>
									</a>
								</span>
								</li>
								<?php
							}
						}
						?>
					</ul>
				</div>
			</div>
			<?php
		}

		/**
		 * Add Support For Amazon SES
		 *
		 * @param array $mailers
		 *
		 * @return array $mailers
		 *
		 * @since 4.6.7
		 */
		public function register_mailers( $mailers ) {

			$mailers['Amazon_SES'] = array(
				'name' => 'Amazon SES',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/aws.svg'
			);
			$mailers['mailgun']    = array(
				'name' => 'Mailgun',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/mailgun.svg'
			);
			$mailers['sendgrid']   = array(
				'name' => 'SendGrid',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/sendgrid.svg'
			);
			$mailers['sparkpost']  = array(
				'name' => 'SparkPost',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/sparkpost.png'
			);
			$mailers['postmark']   = array(
				'name' => 'Postmark',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/postmark.png'
			);
			$mailers['sendinblue']   = array(
				'name' => 'Sendinblue',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/sendinblue.png'
			);
			$mailers['mailjet']   = array(
				'name' => 'Mailjet',
				'logo' => ES_PLUGIN_URL . 'lite/admin/images/mailjet.png'
			);

			return $mailers;
		}

		/**
		 * Show import existing subscribers heading
		 *
		 * @since 4.6.7
		 */
		public function add_import_existing_subscribers_heading() {
			?>
			<label class="inline-flex items-center cursor-pointer w-56">
				<input type="radio" class="absolute w-0 h-0 opacity-0 es_mailer" name="es-import-subscribers"
					   value="es-sync-wordpress-users"/>
				<div class="mt-4 px-1 mx-4 border border-gray-200 rounded-lg shadow-md es-mailer-logo es-importer-logo bg-white">
					<div class="border-0 es-logo-wrapper">
						<svg class="w-6 h-6 text-gray-500" fill="none" stroke="currentColor" viewBox="0 0 24 24"
							 xmlns="http://www.w3.org/2000/svg">
							<path stroke-linecap="round" stroke-linejoin="round" stroke-width="2"
								  d="M12 4.354a4 4 0 110 5.292M15 21H3v-1a6 6 0 0112 0v1zm0 0h6v-1a6 6 0 00-9-5.197M13 7a4 4 0 11-8 0 4 4 0 018 0z"></path>
						</svg>
					</div>
					<p class="mb-2 text-sm inline-block font-medium text-gray-600">
						<?php echo esc_html__( 'Import existing WordPress users', 'email-subscribers' ); ?>
					</p>
				</div>
			</label>
			<?php
		}

		/**
		 * Show import existing subscribers content
		 *
		 * @since 4.6.7
		 */
		public function add_import_existing_subscribers_content() {
			?>
			<div class="w-5/6 flex flex-row es-sync-wordpress-users es-import" style="display: none">
				<div class="es-import-processing flex w-1/4">
					<div class="ml-6 pt-6">
						<label for="select_csv">
							<span class="block text-sm font-medium text-gray-600 pb-1 tracking-wide">
								<?php esc_html_e( 'Import WordPress users with following roles', 'email-subscribers' ); ?>
							</span>

						</label>
					</div>
				</div>
				<div class="w-3/4 ml-12 xl:ml-32 my-6 mr-4">
					<div id="wordpress-users">
						<?php

						// Get only those roles which current user allowed to edit.
						$editable_roles = get_editable_roles();

						if ( ! empty( $editable_roles ) ) {
							foreach ( $editable_roles as $role_id => $role ) {

								$role_name              = translate_user_role( $role['name'] );
								$user_roles[ $role_id ] = $role_name;
							}
						}

						if ( ! empty( $editable_roles ) ) {
							?>
							<div id="ig-es-wordpress-user-roles">
								<table>
									<tr>
										<?php
										$i = 0;
										foreach ( $editable_roles

										as $role_id => $role ) {
										$role_name = translate_user_role( $role['name'] );
											if ( 0 != $i && 0 === ( $i % 3 ) ) {
												?>
									</tr>
									<tr class='mt-3'>
											<?php } ?>
										<td class='pr-3 pt-2 text-sm leading-5 font-normal text-gray-500'>
											<label>
												<input type="checkbox" class="form-checkbox text-gray-600"
													   name="roles[]" value="<?php echo esc_attr( $role_id ); ?>"
													   checked> <?php echo esc_html( $role_name ); ?>
											</label>
										</td>
										<?php
										$i ++;
										}
										?>
									</tr>
								</table>
							</div>
							<?php
						}
						?>
						<div class="clearfix clear mt-10 -mb-4">
							<button
									type="button" id="ig-es-import-wordpress-users-btn"
									class="cursor-pointer ig-es-primary-button px-2 py-1">
								<span class="mr-1"><?php echo esc_html__( 'Proceed ', 'email-subscribers' ); ?></span>
								<span class="es-btn-arrow h-4 w-4"> → </span>
								<svg style="display:none" class="es-btn-loader h-4 w-4 text-white"
									 xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24">
									<circle class="opacity-25" cx="12" cy="12" r="10" stroke="currentColor"
											stroke-width="4"></circle>
									<path class="opacity-75" fill="currentColor"
										  d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path>
								</svg>
							</button>
						</div>
						<p class="import-status pt-4 pb-1 text-base font-medium text-gray-600 tracking-wide hidden">
							&nbsp;</p>
						<div id="progress" class="progress hidden w-10/12"><span class="bar"
																				 style="width:0%"><span></span></span>
						</div>
					</div>
				</div>
			</div>
			<?php
		}

		/**
		 * Show heading for import subscribers from WooCommerce orders section
		 *
		 * @since 4.7.5
		 */
		public function add_wc_importer_heading() {
			global $ig_es_tracker;
			$is_woocommerce_active = $ig_es_tracker::is_plugin_activated( 'woocommerce/woocommerce.php' );
			if ( ! $is_woocommerce_active ) {
				return;
			}
			?>
			<label class="wc-importer-heading inline-flex items-center cursor-pointer w-56">
				<input type="radio" class="absolute w-0 h-0 opacity-0 es_mailer" name="es-import-subscribers"
					   value="es-import-from-wc-orders"/>
				<div class="mt-4 px-1.5 mx-4 border border-gray-200 rounded-lg shadow-md es-mailer-logo es-importer-logo bg-white">
					<div class="border-0 es-logo-wrapper">
						<img src="<?php echo esc_url( ES_PLUGIN_URL . 'lite/admin/images/wc-logo.svg' ); ?>"/>
					</div>
					<p class="mb-2 text-sm inline-block font-medium text-gray-600">
						<?php echo esc_html__( 'Import from WooCommerce orders', 'email-subscribers' ); ?>
					</p>
				</div>
			</label>
			<?php
		}

		/**
		 * Show content for import subscribers from WooCommerce orders section
		 *
		 * @since 4.7.5
		 */
		public function add_wc_importer_content() {
			global $ig_es_tracker;
			$is_woocommerce_active = $ig_es_tracker::is_plugin_activated( 'woocommerce/woocommerce.php' );
			if ( ! $is_woocommerce_active ) {
				return;
			}
			?>
			<div class="w-5/6 es-import-from-wc-orders es-import" style="display: none">
				<div class="flex flex-row">
					<div class="flex w-1/4">
						<div class="ml-6 pt-6">
							<label for="select_csv">
								<span class="block text-sm font-medium text-gray-600 pb-1 tracking-wide">
									<?php echo esc_html__( 'Select order statuses', 'email-subscribers' ); ?>
								</span>

							</label>
						</div>
					</div>
					<div class="w-3/4 ml-12 xl:ml-32 my-6 mr-4">
						<?php

						// Get only those roles which current user allowed to edit.
						$order_statuses = array();
						if ( is_callable( 'wc_get_order_statuses' ) ) {
							$order_statuses = wc_get_order_statuses();
						}

						if ( ! empty( $order_statuses ) ) {
							?>
							<div id="ig-es-wc-order-statuses-wrapper">
								<table>
									<tr>
										<?php
										$i = 0;
										foreach ( $order_statuses

										as $status => $status_label ) {
											if ( 0 !== $i && 0 === ( $i % 3 ) ) {
												?>
									</tr>
									<tr class='mt-3'>
											<?php } ?>
										<td class='pr-3 pt-2 text-sm leading-5 font-normal text-gray-500'>
											<label>
												<input type="checkbox" class="form-checkbox text-gray-600"
													   name="wc-order-statuses[]"
													   value="<?php echo esc_attr( $status ); ?>"> <?php echo esc_html( $status_label ); ?>
											</label>
										</td>
										<?php
										$i ++;
										}
										?>
									</tr>
								</table>
							</div>
							<?php
						}
						?>
					</div>
				</div>
				<div class="flex flex-row border-b border-gray-100">
					<div class="flex w-1/4">
						<div class="ml-6 pt-6">
							<label for="import_contact_list_status"><span
										class="block pr-4 text-sm font-medium text-gray-600 pb-2">
								<?php echo esc_html__( 'Orders should contain these products', 'email-subscribers' ); ?> </span>
							</label>
						</div>
					</div>
					<div class="w-3/4 ml-12 xl:ml-32 my-6 mr-4">
						<select class="wc-product-search" multiple="multiple" id="wc-product-ids" name="wc-product-ids"
								data-placeholder="<?php esc_attr_e( 'Search products...', 'email-subscribers' ); ?>"
								data-action="woocommerce_json_search_products_and_variations"
								style="width: 60%;">
						</select>
						<div class="mt-5 ig-es-order-processing-notice text-sm font-medium text-green-500 tracking-wide hidden">
							<?php
							echo esc_html__( 'Checking your orders...', 'email-subscribers' );
							?>
						</div>
						<div class="clearfix clear mt-5 -mb-4">
							<button
									type="button" id="ig-es-import-from-wc-order-btn"
									class="cursor-pointer ig-es-primary-button px-2 py-1">
								<span class="mr-1"><?php echo esc_html__( 'Proceed ', 'email-subscribers' ); ?></span>
								<span class="es-btn-arrow h-4 w-4"> → </span>
								<svg style="display:none" class="es-btn-loader h-4 w-4 text-white"
									 xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24">
									<circle class="opacity-25" cx="12" cy="12" r="10" stroke="currentColor"
											stroke-width="4"></circle>
									<path class="opacity-75" fill="currentColor"
										  d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path>
								</svg>
							</button>
						</div>
					</div>
				</div>
			</div>
			<?php
			wp_enqueue_script( 'wc-enhanced-select' );
			?>
			<script type="text/javascript">
				jQuery('body').trigger('wc-enhanced-select-init');
			</script>
			<?php
		}

		/**
		 * Prepare import data from WC Orders
		 *
		 * @since 4.7.5
		 */
		public function prepare_import_data_from_wc_orders() {

			check_ajax_referer( 'ig-es-admin-ajax-nonce', 'security' );

			global $wpbd;

			$order_statuses               = ig_es_get_request_data( 'order_statuses', array() );
			$date_args                    = ig_es_get_request_data( 'date_args', array() );
			$product_ids                  = ig_es_get_request_data( 'product_ids', array() );
			$identifier                   = ig_es_get_request_data( 'identifier' );
			$product_list_enabled         = ig_es_get_request_data( 'product_list_enabled', 'no' );
			$variation_list_enabled       = ig_es_get_request_data( 'variation_list_enabled', 'no' );
			$list_ids                     = ig_es_get_request_data( 'list_ids', array() );
			$total_orders_count           = (int) ig_es_get_request_data( 'total_orders_count', 0 );
			$total_processed_orders_count = (int) ig_es_get_request_data( 'total_processed_orders_count', 0 );
			$total_matched_orders_count   = (int) ig_es_get_request_data( 'total_matched_orders_count', 0 );

			$product_list_enabled   = 'yes' === $product_list_enabled ? true : false;
			$variation_list_enabled = 'yes' === $variation_list_enabled ? true : false;

			$importing_from = 'wc-orders';

			$query_args = array(
				'type' => 'shop_order',
			);

			if ( ! empty( $order_statuses ) ) {
				$query_args['status'] = $order_statuses;
			}

			if ( ! empty( $date_args ) ) {
				$query_args[ $date_args['key'] ] = $date_args['value'];
			}

			if ( ! empty( $product_ids ) ) {

				$product_ids_count        = count( $product_ids );
				$product_ids_placeholders = array_fill( 0, $product_ids_count, '%d' );

				$order_ids = $wpbd->get_col(
					$wpbd->prepare(
						"SELECT order_id
						FROM {$wpbd->prefix}woocommerce_order_items
						WHERE order_item_id IN ( SELECT order_item_id FROM {$wpbd->prefix}woocommerce_order_itemmeta WHERE meta_key = '_product_id' AND meta_value IN( " . implode( ',', $product_ids_placeholders ) . " ) )
						AND order_item_type = 'line_item'",
						$product_ids
					)
				);

				// Force wc_get_orders return empty if we don't found any order.
				$order_ids = ! empty( $order_ids ) ? $order_ids : array( 0 );

				$query_args['post__in'] = $order_ids;
			}

			// Get total order count if it wasn't fetched previously.
			if ( empty( $total_orders_count ) ) {
				$query_args['limit']  = - 1;
				$query_args['offset'] = 0;
				$query_args['return'] = 'ids';
				$total_orders_count   = count( wc_get_orders( $query_args ) );
				unset( $query_args['return'] );
			}

			$response = array(
				'success' => false,
			);

			if ( $total_orders_count > 0 ) {
				$query_args['limit']  = 500;
				$query_args['offset'] = (int) $total_processed_orders_count; // We are setting offset to $total_processed_orders_count to allow fetching of remaining unprocessed orders.

				$orders = wc_get_orders( $query_args );


				if ( ! empty( $orders ) ) {

					$raw_data             = '';
					$seperator            = ';';
					$data_contain_headers = false;

					$headers = array(
						__( 'Email', 'email-subscribers' ),
						__( 'First Name', 'email-subscribers' ),
						__( 'Last Name', 'email-subscribers' ),
					);

					foreach ( $orders as $order ) {

						$order_status = $order->get_status();

						// Since WooCommerce store refunds for a given order as a new post type 'shop_order_refund'
						// To check if there any refunds after passed date, then we need to query the child posts created where post_date_gmt is after passed date.
						if ( 'refunded' === $order_status && ( ! empty( $date_args['key'] ) && 'date_refunded' === $date_args['key'] ) ) {
							$parent_order_id = $order->get_id();

							$refund_query_args = array(
								'type'       => 'shop_order_refund',
								'parent'     => $parent_order_id,
								'status'     => 'wc-completed',
								'date_query' => array(
									array(
										'column' => 'post_date_gmt',
										'after'  => $date_args['value'],
									),
								),
								'return'     => 'ids'
							);

							$refund_orders = wc_get_orders( $refund_query_args );
							if ( empty( $refund_orders ) ) {
								$total_processed_orders_count ++;
								continue;
							}
						}

						$email = $order->get_billing_email();
						if ( is_email( $email ) ) {
							$first_name = $order->get_billing_first_name();
							$last_name  = $order->get_billing_last_name();

							$subscriber_data = array(
								$email,
								$first_name,
								$last_name
							);

							if ( $product_list_enabled || $variation_list_enabled ) {
								// Get product items from the order.
								$line_items = $order->get_items();
								if ( ! empty( $line_items ) ) {
									$list_names = array();
									if ( ! empty( $list_ids ) ) {
										$list_names = ES()->lists_db->get_list_name_by_ids( $list_ids );
									}
									foreach ( $line_items as $line_item ) {
										$product = $line_item->get_product();

										if ( $product instanceof WC_Product ) {

											// If product is a variation then get its parent product.
											if ( $product->is_type( 'variation' ) ) {
												$parent_product_id = $product->get_parent_id();
												$parent_product    = wc_get_product( $parent_product_id );

												// Check if list also has to be created for variation product also.
												if ( $variation_list_enabled ) {
													$variation_list_id = ig_es_create_list_from_product( $product );
													if ( ! empty( $variation_list_id ) ) {
														$list_names[] = ES()->lists_db->get_list_name_by_id( $variation_list_id );
													}
												}
											} else {
												$parent_product = $product;
											}

											if ( $product_list_enabled ) {
												$parent_list_id = ig_es_create_list_from_product( $parent_product );
												if ( ! empty( $parent_list_id ) ) {
													$list_names[] = ES()->lists_db->get_list_name_by_id( $parent_list_id );
												}
											}
										}
									}

									if ( ! empty( $list_names ) ) {
										$list_names_str    = '"' . implode( ',', $list_names ) . '"';
										$subscriber_data[] = $list_names_str;
									}
								}

							}

							$raw_data .= implode( $seperator, $subscriber_data );
							$raw_data .= "\n";
							$total_matched_orders_count ++;
						}
						$total_processed_orders_count ++;
					}

					if ( ! empty( $raw_data ) ) {
						$result     = ES_Import_Subscribers::insert_into_temp_table( $raw_data, $seperator, $data_contain_headers, $headers, $identifier, $importing_from );
						$identifier = $result['identifier'];
					}
					$response['success']                      = true;
					$response['identifier']                   = $identifier;
					$response['total_orders_count']           = $total_orders_count;
					$response['total_processed_orders_count'] = $total_processed_orders_count;
					$response['total_matched_orders_count']   = $total_matched_orders_count;

					/* translators: 1. Processed orders count. 2. Total orders count. 3. Matched orders count. */
					$response['order_stats_html'] = sprintf( __( 'Currently %1$s of %2$s orders checked. Found %3$s matching orders.', 'email-subscribers' ),
						number_format_i18n( $total_processed_orders_count ),
						number_format_i18n( $total_orders_count ),
						number_format_i18n( $total_matched_orders_count )
					);
				}
			}

			// Check if we have processed all orders and still can't find any matching order then return the error message.
			if ( $total_processed_orders_count === $total_orders_count && 0 === $total_matched_orders_count ) {
				$response['success'] = false;
				$response['message'] = __( 'We can\'t find any matching orders in your store.', 'email-subscribers' );
			}

			wp_send_json_success( $response );
		}

		/**
		 * Method to add additional columns in Reports table
		 *
		 * @since 4.6.10
		 */
		public function add_additional_reports_column( $es_reports_col ) {

			$report_col['viewed'] = __( 'Total Opened', 'email-subscribers' );

			$es_reports_col = ig_es_array_insert_after( $es_reports_col, 'total_sent', $report_col );

			return $es_reports_col;
		}

		/**
		 * Add data for additional columns in Reports table
		 *
		 * @since 4.6.10
		 */
		public function add_additional_report_column_data( $data, $es_report_col ) {
			switch ( $es_report_col ) {
				case 'viewed':
					$message_id  = ! empty( $data['id'] ) ? $data['id'] : 0;
					$campaign_id = ! empty( $data['campaign_id'] ) ? $data['campaign_id'] : 0;

					if ( ! empty( $campaign_id ) && ! empty( $message_id ) ) {
						$data['viewed'] = ES()->actions_db->get_count_based_on_id_type( $campaign_id, $message_id, IG_MESSAGE_OPEN );
					} else {
						$data['viewed'] = 0;
					}
					$data['viewed'] = number_format_i18n( $data['viewed'] );

					return $data;

				default:
					return $data;
			}
		}

		/**
		 * Get HTML for Mailgun mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 4.7.0
		 */
		public function get_mailgun_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-mailgun-to-send-emails-in-the-email-subscribers-plugin/',
					'utm_medium' => 'mailgun_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_mailgun_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock mailgun">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure Mailgun to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_mailgun_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Get HTML for Sendgrid mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 4.7.0
		 */
		public function get_sendgrid_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-sendgrid-to-send-emails-in-the-email-subscribers-plugin/',
					'utm_medium' => 'sendgrid_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_sendgrid_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock sendgrid">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure SendGrid to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_sendgrid_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Get HTML for Sparkpost mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 4.7.0
		 */
		public function get_sparkpost_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-sparkpost-to-send-emails-in-the-email-subscribers-plugin/',
					'utm_medium' => 'sparkpost_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_sparkpost_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock sparkpost">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure Sparkpost to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_sparkpost_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Get HTML for Amazon SES mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 4.7.0
		 */
		public function get_amazon_ses_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-amazon-ses-to-send-emails-in-the-email-subscribers-plugin',
					'utm_medium' => 'amazon_ses_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_amazon_ses_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock Amazon_SES">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure Amazon SES to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_amazon_ses_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Get HTML for Postmark mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 4.7.1
		 */
		public function get_postmark_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-postmark-to-send-emails-in-the-email-subscribers-plugin/',
					'utm_medium' => 'postmark_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_postmark_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock postmark">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure Postmark to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_postmark_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Get HTML for Sendinblue mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 5.2.1
		 */
		public function get_sendinblue_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-sendinblue-to-send-emails-in-the-email-subscribers-plugin/',
					'utm_medium' => 'sendinblue_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_sendinblue_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock sendinblue">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure Sendinblue to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_sendinblue_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Get HTML for Mailjet mailer doc section
		 *
		 * @return string $html
		 *
		 * @since 5.2.1
		 */
		public function get_mailjet_doc_block() {
			$html = '';
			$url  = ES_Common::get_utm_tracking_url( array(
					'url'        => 'https://www.icegram.com/documentation/how-to-configure-mailjet-to-send-emails-in-the-email-subscribers-plugin/',
					'utm_medium' => 'mailjet_doc'
				)
			);

			ob_start();
			do_action( 'ig_es_before_get_mailjet_doc_block', $this );
			?>
			<div class="es_sub_headline ig_es_docblock mailjet">
				<ul>
					<li><a href="<?php echo esc_url( $url ); ?>"
						   target="_blank"><?php esc_html_e( 'How to configure Mailjet to send emails in the Email Subscribers plugin?', 'email-subscribers' ); ?></a>
					</li>
				</ul>
			</div>
			<?php
			do_action( 'ig_es_after_get_mailjet_doc_block', $this );
			$html = ob_get_clean();

			return $html;
		}

		/**
		 * Add bulk actions on the audience screen
		 *
		 * @param array $bulk_actions
		 *
		 * @return array $bulk_actions
		 *
		 * @since 4.7.6
		 */
		public function add_bulk_actions( $bulk_actions = array() ) {

			$bulk_actions['bulk_send_confirmation_email'] = __( 'Send confirmation email', 'email-subscribers' );

			return $bulk_actions;
		}

		/**
		 * Handle bulk send confirmation email action.
		 *
		 * @param array $subscriber_ids
		 *
		 * @since 4.7.6
		 */
		public function handle_bulk_send_confirmation_email_action( $subscriber_ids = array() ) {
			$queue_status = $this->queue_confirmation_emails( $subscriber_ids );
			if ( 'queued' === $queue_status ) {
				$message = __( 'Confirmation emails queued successfully and will be sent shortly.', 'email-subscribers' );
				ES_Common::show_message( $message, 'success' );
				$request_args = array(
					'action' => 'ig_es_process_queue',
				);
				// Send an asynchronous request to trigger sending of confirmation emails.
				IG_ES_Background_Process_Helper::send_async_ajax_request( $request_args, true );
			} elseif ( 'no_contacts' === $queue_status ) {
				$message = __( 'No contacts found. May be they are already queued or there isn\'t any unconfirmed contact in your selection.', 'email-subscribers' );
				ES_Common::show_message( $message, 'error' );
			} elseif ( 'failed' === $queue_status ) {
				$message = __( 'Failed to queue confirmation emails. Please try again later.', 'email-subscribers' );
				ES_Common::show_message( $message, 'error' );
			}
		}

		/**
		 * Queue confirmation emails
		 *
		 * @param array $subscriber_ids
		 *
		 * @return string $status
		 *
		 * @since 4.7.6
		 */
		public function queue_confirmation_emails( $subscriber_ids = array() ) {

			global $wpbd;

			$sql = "SELECT contacts.id, lists_contacts.list_id FROM {$wpbd->prefix}ig_contacts AS contacts";
			$sql .= " LEFT JOIN {$wpbd->prefix}ig_lists_contacts AS lists_contacts ON contacts.id = lists_contacts.contact_id";
			$sql .= " LEFT JOIN {$wpbd->prefix}ig_queue AS queue ON contacts.id = queue.contact_id AND queue.campaign_id = 0";
			$sql .= ' WHERE 1=1';

			// Subscriber status should be unconfirmed.
			$sql .= " AND lists_contacts.status = 'unconfirmed'";

			// Subscriber must not already been queued already.
			$sql .= ' AND queue.contact_id IS NULL';

			if ( ! is_null( $subscriber_ids ) ) {
				$subscriber_ids = ! is_array( $subscriber_ids ) ? array( (int) $subscriber_ids ) : array_filter( $subscriber_ids, 'is_numeric' );
				$subscriber_ids = array_filter( $subscriber_ids );
				if ( ! empty( $subscriber_ids ) ) {
					$subscriber_ids = array_map( 'esc_sql', $subscriber_ids );
					$sql            .= ' AND contacts.id IN (' . implode( ',', $subscriber_ids ) . ')';
				}
			}

			$sql .= ' GROUP BY contacts.id, lists_contacts.list_id';

			$entries = $wpbd->get_results( $sql );

			$subscriber_ids     = array();
			$subscriber_options = array();

			foreach ( $entries as $entry ) {
				$subscriber_id = $entry->id;
				if ( ! in_array( $subscriber_id, $subscriber_ids, true ) ) {
					$subscriber_ids[]                             = $subscriber_id;
					$subscriber_options[ $subscriber_id ]['type'] = 'optin_confirmation';
				}
			}

			$status = 'no_contacts';

			if ( ! empty( $subscriber_ids ) ) {
				$timestamp = time();

				$queued = ES()->queue->bulk_add(
					0,
					$subscriber_ids,
					$timestamp,
					20,
					false,
					1,
					false,
					$subscriber_options
				);

				$status = $queued ? 'queued' : 'failed';
			}

			return $status;
		}
	}

	if ( ES()->is_pro() ) {
		new Email_Subscribers_Pro();
	}
}
