<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Post_Digest' ) ) {

	class ES_Post_Digest {
		public function __construct() {
			//add template type as post digest
			add_filter( 'es_template_type', array( $this, 'add_template_type' ) );
			//add post notifications action
			add_action( 'ig_es_before_' . IG_CAMPAIGN_TYPE_POST_NOTIFICATION . '_content_settings', array( $this, 'add_post_digest_settings' ) );
			add_action( 'ig_es_before_' . IG_CAMPAIGN_TYPE_POST_DIGEST . '_content_settings', array( $this, 'add_post_digest_settings' ) );

			// Number of post field
			add_action( 'ig_es_show_' . IG_CAMPAIGN_TYPE_POST_NOTIFICATION . '_fields', array( $this, 'show_number_of_post_field' ) );
			add_action( 'ig_es_show_' . IG_CAMPAIGN_TYPE_POST_DIGEST . '_fields', array( $this, 'show_number_of_post_field' ) );

			add_action( 'ig_es_' . IG_CAMPAIGN_TYPE_POST_NOTIFICATION . '_scheduling_options_settings', array( $this, 'add_post_digest_scheduling_options' ) );
			add_action( 'ig_es_' . IG_CAMPAIGN_TYPE_POST_DIGEST . '_scheduling_options_settings', array( $this, 'add_post_digest_scheduling_options' ) );
			
			//save extra data
			add_filter( 'ig_es_' . IG_CAMPAIGN_TYPE_POST_DIGEST . '_data', array( $this, 'add_post_digest_data' ) );

			//refresh post digest content
			add_filter( 'ig_es_refresh_' . IG_CAMPAIGN_TYPE_POST_DIGEST . '_content', array( $this, 'refresh_post_digest_content' ), 10, 2 );
		}

		public function add_template_type( $template_type ) {
			
			$template_type['sequence'] = __( 'Sequence', 'email-subscribers' );

			// Start-IG-Code.
			$template_type['post_digest'] = __( 'Post Digest', 'email-subscribers' );
			// End-IG-Code.

			return $template_type;
		}

		public function add_post_digest_settings( $campaign_data ) {
			$campaign_type  = $campaign_data['type'];
			$meta           = ! empty( $campaign_data['meta'] ) ? $campaign_data['meta']: array();
			$is_post_digest = ( IG_CAMPAIGN_TYPE_POST_DIGEST === $campaign_type ) ? true : false;
			$editor_type    = ! empty( $campaign_data['meta']['editor_type'] ) ? $campaign_data['meta']['editor_type'] : IG_ES_DRAG_AND_DROP_EDITOR;

			$no_of_posts = 0;

			if ( ! empty( $meta ) ) {
				$no_of_posts = ( !empty($meta['rules']) && !empty($meta['rules']['no_of_posts']) ) ? $meta['rules']['no_of_posts'] : 0;
			}

			?>
			<div class="ig-es-campaign-is-post-digest-wrapper pt-4 pb-4 mx-4 border-b border-gray-200">
				<div class="flex w-full">
					<div class="w-11/12 text-sm font-normal text-gray-600"><?php echo esc_html__( 'Is a post digest?', 'email-subscribers' ); ?>
					</div>
					<div>
						<label for="is_post_digest" class="inline-flex items-center cursor-pointer ">
						<span class="relative">
							<input id="is_post_digest" type="checkbox" class="absolute es-check-toggle opacity-0 w-0 h-0" 
						name="campaign_data[is_post_digest]"  value="yes" <?php checked( $is_post_digest, true ); ?> />
						<span class="es-mail-toggle-line"></span>
						<span class="es-mail-toggle-dot"></span>	
						</span>
						</label>
					</div>
				</div>
			</div>
			<?php
			if ( IG_ES_CLASSIC_EDITOR === $editor_type ) {
				$this->show_number_of_post_field( $campaign_data );
			}
		}

		public function show_number_of_post_field( $campaign_data ) {
			$campaign_type  = ! empty( $campaign_data['type'] ) ? $campaign_data['type'] : '';
			$meta           = ! empty( $campaign_data['meta'] ) ? $campaign_data['meta']: array();
			$no_of_posts    = 0;
			$is_post_digest = IG_CAMPAIGN_TYPE_POST_DIGEST === $campaign_type ? true : false;
			$editor_type    = ! empty( $campaign_data['meta']['editor_type'] ) ? $campaign_data['meta']['editor_type'] : IG_ES_DRAG_AND_DROP_EDITOR;
			if ( ! empty( $meta ) ) {
				$no_of_posts = ( ! empty( $meta['rules'] ) && ! empty( $meta['rules']['no_of_posts'] ) ) ? $meta['rules']['no_of_posts'] : 0;
			}
			?>
			<div class="ig-es-campaign-number-of-post-wrapper pt-4 pb-4 mx-4 <?php echo IG_ES_DRAG_AND_DROP_EDITOR === $editor_type  ? esc_attr( 'border-t' ) : esc_attr( 'border-b' ); ?> border-gray-200 <?php echo ! $is_post_digest ? esc_attr( 'hidden' ) : ''; ?>">
				<div class="flex w-full">
					<div class="w-10/12 text-sm font-normal text-gray-600 field-heading">
						<?php echo esc_html__( 'Number of posts', 'email-subscribers' ); ?>
					</div>
					<div class="w-2/12 field-value">
						<input value="<?php echo esc_attr( $no_of_posts ); ?>" class="w-full" name="campaign_data[no_of_posts]" type="number" max="" min=""/>
					</div>
				</div>
			</div>
			<?php
		}

		public function add_post_digest_scheduling_options( $campaign_data ) {
			$allowedtags    = ig_es_allowed_html_tags_in_esc();
			$campaign_type  = $campaign_data['type'];
			$is_post_digest = IG_CAMPAIGN_TYPE_POST_DIGEST === $campaign_type ? true : false;
			?>
			<div class="ig-es-campaign-post-digest-scheduler-wrapper pt-4 pb-4 mx-4 border-t border-gray-200 <?php echo ! $is_post_digest ? 'hidden' : ''; ?>">
				<div class="w-full">
					<div class="text-sm font-normal text-gray-600 pb-2">
						<?php echo esc_html__( 'When to send?', 'email-subscribers' ); ?>
					</div>
					<div class="w-full">
					<?php 
						echo wp_kses( $this->es_scheduler_html( $campaign_data ), $allowedtags ); 
					?>
					</div>
				</div>
			</div>
			<?php
		}

		public function es_scheduler_html( $campaign_data ) {
			$meta  = ! empty( $campaign_data['meta'] ) ? $campaign_data['meta'] : array();
			$rules = array();
			$rules = !empty($meta['rules']) ? $meta['rules'] : $rules;
			if ( empty($rules)) {
				$rules['time_frame']  = 'daily';
				$rules['month_day']   = 1;
				$rules['week_day']    = 0 ;
				$rules['time_of_day'] = 0;
			}

			$time_frame  = array(
				'daily'       => __( 'Once a day at...', 'email-subscribers' ),
				'weekly'      => __( 'Weekly on...', 'email-subscribers' ),
				'monthly'     => __( 'Monthly on the...', 'email-subscribers' ),
				'immediately' => __( 'Immediately', 'email-subscribers' )
			);
			$week_day    = array(
				'0' => __( 'Sunday', 'email-subscribers' ),
				'1' => __( 'Monday', 'email-subscribers' ),
				'2' => __( 'Tuesday', 'email-subscribers' ),
				'3' => __( 'Wednesday', 'email-subscribers' ),
				'4' => __( 'Thursday', 'email-subscribers' ),
				'5' => __( 'Friday', 'email-subscribers' ),
				'6' => __( 'Saturday', 'email-subscribers' ),
			);
			$time_of_day = array(
				'0'     => '12:00 am',
				'3600'  => '1:00 am',
				'7200'  => '2:00 am',
				'10800' => '3:00 am',
				'14400' => '4:00 am',
				'18000' => '5:00 am',
				'21600' => '6:00 am',
				'25200' => '7:00 am',
				'28800' => '8:00 am',
				'32400' => '9:00 am',
				'36000' => '10:00 am',
				'39600' => '11:00 am',
				'43200' => '12:00 pm',
				'46800' => '1:00 pm',
				'50400' => '2:00 pm',
				'54000' => '3:00 pm',
				'57600' => '4:00 pm',
				'61200' => '5:00 pm',
				'64800' => '6:00 pm',
				'68400' => '7:00 pm',
				'72000' => '8:00 pm',
				'75600' => '9:00 pm',
				'79200' => '10:00 pm',
				'82800' => '11:00 pm'
			);
			$month_day   = array(
				'1'  => '1st',
				'2'  => '2nd',
				'3'  => '3rd',
				'4'  => '4th',
				'5'  => '5th',
				'6'  => '6th',
				'7'  => '7th',
				'8'  => '8th',
				'9'  => '9th',
				'10' => '10th',
				'11' => '11th',
				'12' => '12th',
				'13' => '13th',
				'14' => '14th',
				'15' => '15th',
				'16' => '16th',
				'17' => '17th',
				'18' => '18th',
				'19' => '19th',
				'20' => '20th',
				'21' => '21th',
				'22' => '22th',
				'23' => '23th',
				'24' => '24th',
				'25' => '25th',
				'26' => '26th',
				'27' => '27th',
				'28' => '28th',
				'30' => '30th'
			);

			$time_frame_html = '';

			$time_frame_html = '<span id="field_interval_type">
				<select class="form-select w-full h-9 text-sm rounded-md cursor-pointer" name="campaign_data[time_frame]">';
			foreach ( $time_frame as $key => $value ) {
				$time_frame_html .= '<option value="' . $key . '" ' . selected( $rules['time_frame'], $key, false ) . '/>' . $value . '</option>';
			}
			$time_frame_html .= '</select></span>';

			$month_day_html = '<span id="field_month_day" class="es_intervals mr-1 w-1/2">
			<select class="form-select w-full h-9 text-sm rounded-md cursor-pointer" name="campaign_data[month_day]"  >';
			foreach ( $month_day as $key => $value ) {
				$month_day_html .= '<option value="' . $key . '" ' . selected( $rules['month_day'], $key, false ) . '/>' . $value . '</option>';
			}
			$month_day_html .= '</select></span>';

			$week_day_html = '<span id="field_week_day" class="es_intervals mr-1 w-1/2">
			<select class="form-select w-full h-9 text-sm rounded-md cursor-pointer" name="campaign_data[week_day]"  >';
			foreach ( $week_day as $key => $value ) {
				$week_day_html .= '<option value="' . $key . '" ' . selected( $rules['week_day'], $key, false ) . '/>' . $value . '</option>';
			}
			$week_day_html .= '</select></span>';


			$time_of_day_html = '<span id="field_time_of_day" class="es_intervals">
			<select class="form-select w-full h-9 text-sm rounded-md cursor-pointer" name="campaign_data[time_of_day]">';
			foreach ( $time_of_day as $key => $value ) {
				$time_of_day_html .= '<option value="' . $key . '" ' . selected( $rules['time_of_day'], $key, false ) . '/>' . $value . '</option>';
			}
			$time_of_day_html .= '</select></span>';

			$es_scheduler_html = '
				<div class="w-full">
					' . $time_frame_html . '
				</div>
				<div class="w-full flex pt-1">
					' . $month_day_html . $week_day_html . $time_of_day_html .
				'</div>';

			return $es_scheduler_html;
		}

		public function add_post_digest_data( $campaign_data ) {
			
			// Get nonce field value.
			$campaign_nonce = ig_es_get_request_data( 'ig_es_campaign_nonce' );
				// Verify nonce.
			if ( wp_verify_nonce( $campaign_nonce, 'ig-es-campaign-nonce' ) ) {
				if ( IG_CAMPAIGN_TYPE_POST_DIGEST === $campaign_data['type'] ) {
					
					$meta                         = maybe_unserialize( $campaign_data['meta'] );
					$meta['rules']['time_frame']  = ig_es_get_data( $campaign_data, 'time_frame', 'daily', true );
					$meta['rules']['month_day']   = ig_es_get_data( $campaign_data, 'month_day', 1, true );
					$meta['rules']['week_day']    = ig_es_get_data( $campaign_data, 'week_day', 0, true );
					$meta['rules']['interval']    = ( 'weekly' === $meta['rules']['time_frame'] ) ?  $meta['rules']['week_day']: $meta['rules']['month_day'] ;
					$meta['rules']['time_of_day'] = ig_es_get_data( $campaign_data, 'time_of_day', 0, true );
					$meta['rules']['action']      = 'ig_es_time_based_campaign';
					$meta['rules']['weekdays']    = array( 1, 2, 3, 4, 5, 6, 0 );
					$no_of_posts                  = ig_es_get_data( $campaign_data, 'no_of_posts', 5, true );
					$meta['rules']['no_of_posts'] = (int) $no_of_posts;
					$d['time_frame']              = $meta['rules']['time_frame'];
					$d['interval']                = $meta['rules']['interval'];
					$d['weekdays']                = $meta['rules']['weekdays'];
					$d['time_of_day']             = $meta['rules']['time_of_day'];

					$next_run = ig_es_get_next_future_schedule_date( $d );

					$meta['next_run'] = $next_run;

					$campaign_data['meta'] = maybe_serialize($meta);
				}
			}
			
			return $campaign_data;
		}

		public static function get_post_id_for_post_digest( $campaign_id, $ignore_stored_post_ids = false, $ignore_last_run = false ) {
			global $wpdb;
			$include_all_post = false;
			$include_no_post  = false;
			//get recent no of posts
			$meta = ES()->campaigns_db->get_campaign_meta_by_id( $campaign_id );
			// Check if we have post ids stored in the campaign.
			if ( ! $ignore_stored_post_ids && ! empty( $meta['post_ids'] ) && is_array( $meta['post_ids'] ) ) {
				// If post ids are set in the campaign, then we don't need to fetch them again as they are already set in the last run of the report.
				return $meta['post_ids'];
			}

			$no_of_posts      = ( !empty( $meta['rules']) && !empty( $meta['rules']['no_of_posts'] ) ) ? $meta['rules']['no_of_posts'] : 5;
			$categories_str   = ES()->campaigns_db->get_campaign_categories_str_by_id( $campaign_id );
			$categories       = ES_Common::convert_categories_string_to_array( $categories_str, true );
			$custom_post_type = array();
			//decide period to fetch post ids
			$last_run = !empty($meta['last_run']) ? $meta['last_run'] : '';

			foreach ($categories as $key => $category) {
				if ( false !== strpos( $category, '{T}' ) ) {
					$custom_post_type[] = str_replace('{T}', '', $category);
					if ( isset( $categories ) && isset( $categories[ $key ] ) ) {
						unset( $categories[ $key ] );
					}
				}
				if ( 0 === $key ) {
					if ( 'All' == $category) {
						$include_all_post = true;
						unset( $categories );
					} elseif ( 'None' === $category ) {
						$include_no_post = true;
						unset( $categories );
					}
				}
			}

			$post_type = array();
			$post_ids  = array();
			if ( ( ! empty( $categories ) || $include_all_post ) && ! $include_no_post ) {
				$post_args =  array(
					'post_type' => 'post',
					'posts_per_page' => $no_of_posts,
					'orderby' => 'date',
					'order' => 'DESC',
					'cat' => ( !$include_all_post ) ? implode(',', $categories) : '',
				// 'date_query'    => array(
				//  'column'  => 'post_date',
				//  'after'   => date("Y-m-d H:i:s", $last_run)
				//  )
				);
				
				if ( !empty($last_run) && ! $ignore_last_run ) {
					$post_args['date_query'] = array(
						'column'  => 'post_date',
						'after'   => gmdate('Y-m-d H:i:s', $last_run)
					);
				}
				$post_ids = get_posts( $post_args );
			}
			$custom_post_ids = array();
			if (!empty($custom_post_type)) {
				$custom_post_args = array(
					'post_type' => $custom_post_type,
					'posts_per_page' => $no_of_posts,
					'orderby' => 'date',
					'order' => 'DESC',
				);
				$custom_post_ids  = get_posts( $custom_post_args );
			}
			$posts = array_merge($post_ids, $custom_post_ids);
			usort( $posts, array('ES_Post_Digest', 'sort_by_date'));
			$posts    =  array_slice($posts, 0, $no_of_posts, true);
			$post_ids = array_map( array('ES_Post_Digest', 'get_post_ids'), $posts );  
			//update last run
			$meta['last_run'] = strtotime(ig_get_current_date_time());
					
			// Allow third party developers to modify posts to be sent for this post digest campaign.
			$post_ids = apply_filters( 'ig_es_post_ids_for_post_digest', $post_ids, $campaign_id );
					
			$meta['post_ids'] = $post_ids;
			ES()->campaigns_db->update_campaign_meta( $campaign_id, $meta );
			return $post_ids; 

		}

		public static function sort_by_date( $a, $b) {
			return strtotime($b->post_date) - strtotime($a->post_date);
		}

		public static function get_post_ids( $post ) {
			return $post->ID;
		}

		public static function process_post_digest_template( $template_content = '', $post_ids = array() ) {

			//find post digest block
			//loop over post ids
			//replace with container in template
			//TODO:: handle empty post ids condition - send email to admin as well as stop client email
			if ( strpos( $template_content, '{{POSTDIGEST}}' ) >= 0 && !empty($post_ids) ) {
				$post_digest_block  = ES_Common::get_in_between_content( $template_content, '{{POSTDIGEST}}', '{{/POSTDIGEST}}' );
				$post_block_content = '';
				foreach ( $post_ids as $post_id ) {
					$post_block_content .= '<div class="es_post_digest_block">' . ES_Handle_Post_Notification::prepare_body( $post_digest_block, $post_id, 0 ) . '</div>';
				}
				$find             = '{{POSTDIGEST}}' . $post_digest_block . '{{/POSTDIGEST}}';
				$replace          = '<div class="es_post_digest_wrapper">' . $post_block_content . '</div>';
				$template_content = str_replace( $find, $replace, $template_content );
			}
			return $template_content;
		}

		public function refresh_post_digest_content( $content, $args) {
			$campaign_id = $args['campaign_id'];
			$campaign    = ES()->campaigns_db->get( $campaign_id );
			$post_ids    = self::get_post_id_for_post_digest( $campaign_id );
			
			$email_body = '';
			if ( ! empty( $campaign['body'] ) ) {
				$email_body = $campaign['body'];
			} else {
				$template_id = ES()->campaigns_db->get_template_id_by_campaign( $campaign_id );
				$template    = get_post( $template_id );
				if ( ! empty( $template ) ) {
					$email_body = $template->post_content;
				}
			}

			$content['body'] = self::process_post_digest_template( $email_body, $post_ids );
			return $content;
		}

	}
}
