<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'ES_Sequences' ) ) {

	class ES_Sequences {

		public function __construct() {
			add_action( 'ig_es_after_campaign_type_buttons', array( $this, 'es_add_campaign_type' ) );
			add_action( 'wp_ajax_get_seq_action_row', array( &$this, 'get_seq_action_row' ) );
			add_action( 'wp_ajax_save_seq_data', array( &$this, 'save_seq_data' ) );
			add_filter( 'ig_es_campaign_list_where_caluse', array( &$this, 'remove_seq_messages_from_list' ) );

		}

		public function es_add_campaign_type() {
			?>
			<a href="admin.php?page=es_sequence&action=new" class="ig-es-title-button px-3 py-1 ml-2 align-middle"><?php echo esc_html__( 'Create Sequence', 'email-subscribers' ); ?></a>
			<?php
		}

		public function load_sequence() {
			$action = ig_es_get_request_data( 'action' );
			?>
			<div class="wrap pt-4 font-sans">
				<?php

				if ( 'edit' === $action ) {
					$list = ig_es_get_request_data( 'list' );
					$this->edit_list( absint( $list ) );
				} else {
					$this->es_newseq_callback();
				}
				?>
			</div>
			<?php
		}


		public function prepare_seq_form( $seq_id = '' ) {

			$is_new = empty( $seq_id ) ? 1 : 0;

			$action  = 'new';
			$heading = __( 'New Sequence', 'email-subscribers' );
			if ( ! $is_new ) {
				$action  = 'edit';
				$heading = __( 'Edit Sequence', 'email-subscribers' );
			}
			$nonce = wp_create_nonce( 'es_sequence' );
			//fetch seq data
			$parent_seq_data = ES()->campaigns_db->get( $seq_id );
			$parent_seq_data    = ! empty( $parent_seq_data ) ? $parent_seq_data : array();
			$seq_messages_data  = ES()->campaigns_db->get_campaign_by_parent_id( $seq_id );
			$seq_data           = array();
			$seq_data['parent'] = $parent_seq_data;
			usort( $seq_messages_data, array( 'ES_Sequences', 'sort_by_days' ) );
			$seq_data['messages'] = $seq_messages_data;
			$allowedtags          = ig_es_allowed_html_tags_in_esc();

			if ( ! function_exists( 'ig_es_wp_js_editor_admin_scripts' ) ) {
				/**
				 * Include WP JS Editor library's main file. This file contains required functions to enqueue required js file which being used to create WordPress editor dynamcially.
				 */
				require_once ES_PLUGIN_DIR . 'lite/includes/libraries/wp-js-editor/wp-js-editor.php';
			}

			// Load required html/js for dynamic WordPress editor.
			ig_es_wp_js_editor_admin_scripts();
			?>
			<div class="max-w-full -mt-3 font-sans">
				<header class="wp-heading-inline">
					<nav class="text-gray-400 my-0" aria-label="Breadcrumb">
						<ol class="list-none p-0 inline-flex">
							<li class="flex items-center text-sm tracking-wide">
								<a class="hover:underline" href="admin.php?page=es_campaigns"><?php esc_html_e( 'Campaigns ', 'email-subscribers' ); ?></a>
								<svg class="fill-current w-2.5 h-2.5 mx-2 mt-mx" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M285.476 272.971L91.132 467.314c-9.373 9.373-24.569 9.373-33.941 0l-22.667-22.667c-9.357-9.357-9.375-24.522-.04-33.901L188.505 256 34.484 101.255c-9.335-9.379-9.317-24.544.04-33.901l22.667-22.667c9.373-9.373 24.569-9.373 33.941 0L285.475 239.03c9.373 9.372 9.373 24.568.001 33.941z"></path></svg>
							</li>
						</ol>
					</nav>
					<div class="flex">
						<div class="w-1/2">
							<h2 class="-mt-1 text-2xl font-medium text-gray-700 sm:leading-7 sm:truncate">
								<?php echo esc_html( $heading ); ?> 
							</h2>
						</div>
						<div class="w-1/2 -mt-2.5 inline text-right">
							<a class="px-1.5 py-2 mt-2 es-documentation" href="https://www.icegram.com/documentation/email-sequence/?utm_source=in_app&utm_medium=sequence&utm_campaign=es_doc" target="_blank">
								<svg class="w-6 h-6 -mt-1 inline text-gray-500" fill="none" stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
									<title><?php esc_html_e('Documentation ', 'email-subscribers'); ?></title>
									<path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 6.253v13m0-13C10.832 5.477 9.246 5 7.5 5S4.168 5.477 3 6.253v13C4.168 18.477 5.754 18 7.5 18s3.332.477 4.5 1.253m0-13C13.168 5.477 14.754 5 16.5 5c1.747 0 3.332.477 4.5 1.253v13C19.832 18.477 18.247 18 16.5 18c-1.746 0-3.332.477-4.5 1.253"></path>
								</svg>
							</a>
						</div>
					</div>
				</header>
				<div class="">
					<hr class="wp-header-end">
				</div>
				<form id="es_seq_form" action="admin.php?page=es_sequence&action=<?php echo esc_attr( $action ); ?>&list=<?php echo esc_attr( $seq_id ); ?>&_wpnonce=<?php echo esc_attr( $nonce ); ?>" method="post">
					<div class="es_seq_main_wrapper mt-5">
						<input type="hidden" name="es_parent_seq_id" value="<?php echo ! empty( $parent_seq_data['id'] ) ? esc_attr( $parent_seq_data['id'] ) : ''; ?>"/>
						<div class="es_sequence_name_wrapper">
							<input placeholder="<?php echo esc_html__( 'Add Sequence name', 'email-subscribers' ); ?>" type="text" class="es_sequence_name" name="es_sequence_name" value="<?php echo ! empty( $parent_seq_data['name'] ) ? esc_attr( $parent_seq_data['name'] ) : ''; ?>">
							<input class="es_seq_btn es_seq_btn_save align-middle px-4 py-2 ig-es-primary-button hover:shadow-md" type="submit" name="submit" value="<?php echo esc_html__( 'Save All', 'email-subscribers' ); ?>"/>
							<br>

							<?php
							add_filter( 'safe_style_css', 'ig_es_allowed_css_style' );
							?>
						</div>
						<div class="es_sequences_wrapper">
							<div class="sidebar sidebar--left sidebar--tabs border-t border-gray-200">
								<ul id="sequence_email_templates" class="side_tabs ui-sortable">
									<?php
									if ( ! empty( $seq_messages_data ) ) {
										foreach ( $seq_messages_data as $seq_message ) {
											echo wp_kses( $this->get_left_action_row( $seq_message ), $allowedtags );
										}
									} else {
										echo wp_kses( $this->get_left_action_row(), $allowedtags );
									}
									?>
								</ul>
								<div class="es_add_email_btn">
									<div class="es_seq_btn" id="es_add_next_seq_email"><?php echo esc_html__( '+ Add Email', 'email-subscribers' ); ?></div>
								</div>
							</div>
							<div id="editing_area" class="content content--tabs content--sequence course_form shadow-md border border-gray-200">
								<?php
								if ( ! empty( $seq_messages_data ) ) {
									foreach ( $seq_messages_data as $seq_message ) {
										echo wp_kses( $this->get_content_action_row( $seq_message ), $allowedtags );
									}
								} else {
									echo wp_kses( $this->get_content_action_row(), $allowedtags );
								}
								?>
							</div>
						</div>
					</div>
					<input type="hidden" name="submitted" value="submitted">
				</form>
			</div>
			<?php

		}

		public function sort_by_days( $a, $b ) {
			$meta_a = maybe_unserialize( $a['meta'] );
			$meta_b = maybe_unserialize( $b['meta'] );

			return ( $meta_a['rules']['amount'] < $meta_b['rules']['amount'] ) ? - 1 : 1;
		}

		public function es_newseq_callback() {
			$submitted = ig_es_get_request_data( 'submitted' );
			if ( 'submitted' === $submitted ) {
				// Get nonce field value.
				$nonce = ig_es_get_request_data( '_wpnonce' );
				// Verify nonce.
				if ( wp_verify_nonce( $nonce, 'es_sequence' ) ) {
					$seq_data = ig_es_get_request_data( 'seq_data', '', false );
					$name     = ig_es_get_request_data( 'es_sequence_name', '' );
					if ( empty( $seq_data ) ) {
						$message = __( 'Sequence should have atleast one email', 'email-subscribers' );
						ES_Common::show_message( $message, 'error' );
						$this->prepare_seq_form();

						return;
					}

					//add parent campaign
					$seq_parent_data           = array();
					$ajax_message_ids_map      = array();
					$seq_parent_data['type']   = 'sequence';
					$seq_parent_data['name']   = $name;
					$seq_parent_data['status'] = 1;
					$seq_parent_data['slug']   = sanitize_title( $name );
					$parent_id                 = ES()->campaigns_db->save_campaign( $seq_parent_data );

					foreach ( $seq_data as $seq_index => $seq ) {
						//create new campaign for each seq
						$campaign_data['type']        = 'sequence_message';
						$campaign_data['body']        = wp_kses_post( wpautop( $seq['body'] ) );
						$campaign_data['subject']     = sanitize_text_field( $seq['subject'] );
						$campaign_data['name']        = sanitize_text_field( $seq['subject'] );
						$campaign_data['slug']        = sanitize_title( $seq['subject'] );
						$campaign_data['parent_id']   = $parent_id;
						$campaign_data['parent_type'] = 'sequence';
						// $meta['utm_campaign_name']  = $seq['utm_campaign_name'];
						$meta = array();
						$meta['rules']['action'] = 'ig_es_contact_insert';
						$meta['rules']['unit']   = $seq['delay_unit'];
						$meta['rules']['amount'] = $seq['delay_amount'];
						if ( ! empty( $seq['attachments'] ) ) {
							$meta['attachments'] = $seq['attachments'];
						}
						if ( ! empty( $seq['list_conditions'] ) ) {
							$meta['list_conditions'] = IG_ES_Campaign_Rules::remove_empty_conditions( $seq['list_conditions'] );
							$meta['list_conditions'] = $this->replace_ajax_message_ids_with_db_ids( $meta['list_conditions'], $ajax_message_ids_map );
						}
						$campaign_data['status'] = 1;
						$campaign_data['meta']   = maybe_serialize( $meta );
						$campaign_id = ES()->campaigns_db->save_campaign( $campaign_data );
						if ( ! empty( $campaign_id ) ) {
							$ajax_message_ids_map[$seq_index] = $campaign_id;
						}
					}
					$message = __( 'Sequence added successfully!', 'email-subscribers' );
					ES_Common::show_message( $message, 'success' );
					$this->prepare_seq_form( $parent_id );
				} else {
					$message = __( 'Sorry, you are not allowed to add sequence.', 'email-subscribers' );
					ES_Common::show_message( $message, 'error' );
				}
			} else {
				$this->prepare_seq_form();
			}
		}

		public function edit_list( $id ) {

			global $wpdb;

			$seq = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}ig_campaigns WHERE id = %d LIMIT 0, 1", $id ), ARRAY_A );

			$submitted = ig_es_get_request_data( 'submitted' );
			if ( 'submitted' === $submitted ) {
				// Get nonce field value.
				$nonce = ig_es_get_request_data( '_wpnonce' );
				// Verify nonce.
				if ( wp_verify_nonce( $nonce, 'es_sequence' ) ) {	
					// Save edited sequence.
					$seq_data = ig_es_get_request_data( 'seq_data', '', false );
					$name     = ig_es_get_request_data( 'es_sequence_name', '' );
					if ( empty( $seq_data ) ) {
						$message = __( 'Sequence should have atleast one email', 'email-subscribers' );
						ES_Common::show_message( $message, 'error' );
						$this->prepare_seq_form();

						return;
					}
					if ( ! empty( $seq ) ) {
						//edit existing camapign
						//save parent seq
						$parent_seq_data['name']     = $name;
						ES()->campaigns_db->save_campaign( $parent_seq_data, $id );
						$seq_messages_data = ES()->campaigns_db->get_campaign_by_parent_id( $id );
						$seq_id_data_map   = array();
						foreach ( $seq_messages_data as $key => $value ) {
							$seq_id_data_map[ $value['id'] ] = $value;
						}
						$seq_ids_to_delete = array_keys( array_diff_key( $seq_id_data_map, $seq_data ) );

						ES()->campaigns_db->delete_campaigns( $seq_ids_to_delete );
						$ajax_message_ids_map = array();
						foreach ( $seq_data as $key => $seq ) {

							if ( ! empty( $seq_id_data_map[ $key ] ) ) {
								$campaign_data['subject']  = sanitize_text_field( $seq['subject'] );
								$campaign_data['name']     = sanitize_text_field( $seq['subject'] );
								$campaign_data['slug']     = sanitize_title( $seq['subject'] );
								$campaign_data['body']     = wp_kses_post( wpautop( $seq['body'] ) );
								$meta                      = maybe_unserialize( $seq_id_data_map[ $key ]['meta'] );
								$meta['rules']['unit']     = $seq['delay_unit'];
								$meta['rules']['amount']   = $seq['delay_amount'];
								if ( ! empty( $seq['attachments'] ) ) {
									$meta['attachments'] = $seq['attachments'];
								} else {
									$meta['attachments'] = array();
								}
								if ( ! empty( $seq['list_conditions'] ) ) {
									$meta['list_conditions'] = IG_ES_Campaign_Rules::remove_empty_conditions( $seq['list_conditions'] );
								} else {
									$meta['list_conditions'] = array();
								}
								// $meta['utm_campaign_name'] = $seq['utm_campaign_name'];
								$campaign_data['meta'] = maybe_serialize( $meta );
								ES()->campaigns_db->save_campaign( $campaign_data, $key );
							} else {
								$campaign_data['type']        = 'sequence_message';
								$campaign_data['body']        = wp_kses_post( wpautop( $seq['body'] ) );
								$campaign_data['subject']     = sanitize_text_field( $seq['subject'] );
								$campaign_data['name']        = sanitize_text_field( $seq['subject'] );
								$campaign_data['slug']        = sanitize_title( $seq['subject'] );
								$campaign_data['parent_id']   = $id;
								$campaign_data['parent_type'] = 'sequence';
								// $meta['utm_campaign_name']  = $seq['utm_campaign_name'];
								$meta                    = array();
								$meta['rules']['action'] = 'ig_es_contact_insert';
								$meta['rules']['unit']   = $seq['delay_unit'];
								$meta['rules']['amount'] = $seq['delay_amount'];
								if ( ! empty( $seq['attachments'] ) ) {
									$meta['attachments'] = $seq['attachments'];
								} else {
									$meta['attachments'] = array();
								}
								if ( ! empty( $seq['list_conditions'] ) ) {
									$meta['list_conditions'] = IG_ES_Campaign_Rules::remove_empty_conditions( $seq['list_conditions'] );
									$meta['list_conditions'] = $this->replace_ajax_message_ids_with_db_ids( $meta['list_conditions'], $ajax_message_ids_map );
								} else {
									$meta['list_conditions'] = array();
								}
								$campaign_data['status'] = 1;
								$campaign_data['meta']   = maybe_serialize( $meta );

								$campaign_id = ES()->campaigns_db->save_campaign( $campaign_data );
								if ( ! empty( $campaign_id ) ) {
									$ajax_message_ids_map[$key] = $campaign_id;
								}
							}
						}
						$message = __( 'Sequence updated successfully!', 'email-subscribers' );
						ES_Common::show_message( $message, 'success' );
					}
				} else {
					$message = __( 'Sorry, you are not allowed to edit sequence.', 'email-subscribers' );
					ES_Common::show_message( $message, 'error' );
				}
			} else {
				$seq = array_shift( $seq );
				$id  = $seq['id']; //check this
			}
			$this->prepare_seq_form( $id );
		}

		public function get_sequence_headers() {
			ob_start();

			?>
			<input class="es_seq_btn es_seq_btn_save cursor-pointer align-middle px-4 py-2 ig-es-primary-button hover:shadow-md" type="submit" name="submit" value="<?php echo esc_attr__( 'Save All', 'email-subscribers' ); ?>"/>
			<?php
			$html = ob_get_clean();

			return $html;
		}


		public function get_seq_action_row( $seq_data ) {
			$pre_seq_no         = ig_es_get_request_data( 'pre_seq_id', 0 );
			$seq_no             = $pre_seq_no + 1;
			$left_action_row    = $this->get_left_action_row( $seq_data, $seq_no );
			$content_action_row = $this->get_content_action_row( $seq_data, $seq_no );
			// $settings_action_row = $this->get_settings_action_row( $seq_id );

			echo json_encode( array( 'id' => $seq_no, 'left_action_row' => $left_action_row, 'content_action_row' => $content_action_row ) );
			die();

		}

		public function get_left_action_row( $seq_data = array(), $seq_no = 1 ) {
			$seq_message_id = ! empty( $seq_data['id'] ) ? $seq_data['id'] : $seq_no;
			$meta           = ! empty( $seq_data ) ? maybe_unserialize( $seq_data['meta'] ) : '';
			$delay_amount   = ! empty( $meta['rules']['amount'] ) ? $meta['rules']['amount'] : 1;
			$delay_unit     = ! empty( $meta['rules']['unit'] ) ? $meta['rules']['unit'] : 'immediately';
			$delay_text	    = $this->get_delay_text( $delay_unit, $delay_amount );
			ob_start();
			?>
			<li class="side-tab side-tab--sortable es_seq_list" data-seq-id="<?php echo esc_attr( $seq_message_id ); ?>" style="">

				<div class="es-seq-subject-wrapper"><strong class="js_subject"><?php echo ! empty( $seq_data['name'] ) ? esc_html( $seq_data['name'] ) : esc_html__( 'Welcome', 'email-subscribers' ); ?></strong></div>
				<span class="es-seq-side-options">
					<span class="es_time">
						<span class="dashicons dashicons-clock" title="<?php echo esc_attr__( 'Delete this email', 'email-subscribers' ); ?>"></span>
						<span class="js_delay"><?php echo esc_html( $delay_text ); ?></span>
					</span>
					<span class="delete_email dashicons dashicons-trash" title="<?php echo esc_attr__( 'Delete this email', 'email-subscribers' ); ?>"></span>
				</span>

			</li>
			<?php
			$html = ob_get_clean();

			return $html;
		}

		public function get_content_action_row( $seq_data = array(), $seq_no = 1 ) {
			$seq_message_id = ! empty( $seq_data['id'] ) ? $seq_data['id'] : $seq_no;
			$meta           = ! empty( $seq_data ) ? maybe_unserialize( $seq_data['meta'] ) : '';
			$allowedtags    = ig_es_allowed_html_tags_in_esc();
			ob_start();
			?>
			<div class="es_seq_right_wrapper" data-seq-id="<?php echo esc_attr( $seq_message_id ); ?>">
				<div class="es_wrapper">
					<div class="sequence-controls pl-1">
						<?php
						add_filter( 'safe_style_css', 'ig_es_allowed_css_style' );
						echo wp_kses( $this->get_day_control_row( $seq_message_id, $seq_data ), $allowedtags );
						?>
					</div>
					<div class="sequence-controls pl-1 pb-3">
						<?php
						do_action( 'ig_es_show_campaign_rules', $seq_message_id, $seq_data );
						?>
					</div>
					<div class="sequence-subject">
						<label>
							<span class="text-gray-700 font-medium text-opacity-75 text-sm tracking-wide leading-6"><?php echo esc_html__( 'Email Subject', 'email-subscribers' ); ?></span>
						</label>
						<input class="course_email_subject_field " placeholder="<?php echo esc_html__( 'Welcome', 'email-subscribers' ); ?>" value="<?php echo ! empty( $seq_data['subject'] ) ? esc_attr( $seq_data['subject'] ) : ''; ?>" name="seq_data[<?php echo esc_attr( $seq_message_id ); ?>][subject]" required="" autocomplete="course_subject_field" id="email_template_subject" type="text" maxlength="255"
						size="50" style="">
					</div>
					<div class="sequence-content">
						<label><span class="text-gray-700 font-medium text-opacity-75 text-sm tracking-wide leading-6"><?php echo esc_html__( 'Content', 'email-subscribers' ); ?></span></label>
						<?php
						$body = ! empty( $seq_data['body'] ) ? $seq_data['body'] : '';
						// Load text editor through 'wp_editor' if it isn't an ajax request otherwise load through JS.
						if ( ! ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
							$editor_args = array(
								'textarea_name' => 'seq_data[' . $seq_message_id . '][body]',
								'textarea_rows' => 40,
								'editor_class'  => 'wp-editor-content',
								'media_buttons' => true,
								'tinymce'       => true,
								'quicktags'     => true,
							);
							wp_editor( $body, 'edit-sequence-message-' . $seq_message_id, $editor_args );
						} else {
							?>
							<br>
							<textarea class="w-full" rows="70" name="<?php echo esc_attr( 'seq_data[' . $seq_message_id . '][body]' ); ?>" id="<?php echo esc_attr( 'edit' . $seq_message_id ); ?>">
								<?php
								echo wp_kses( $body, $allowedtags );
								?>
							</textarea>
							<?php
						}
						?>
					</div>
				</div>

			</div>
			<?php
			$html = ob_get_clean();

			return $html;
		}

		public function get_day_control_row( $seq_id, $seq_data = array() ) {
			$meta              = ! empty( $seq_data ) ? maybe_unserialize( $seq_data['meta'] )  : '';
			$delay_amount      = ! empty( $meta['rules']['amount'] ) ? $meta['rules']['amount'] : 1;
			$delay_unit        = ! empty( $meta['rules']['unit'] ) ? $meta['rules']['unit'] : 'immediately';
			$hide_amount_field = 'immediately' === $delay_unit ? true : false;
			$amount_field_css  = '';
			if ( $hide_amount_field ) {
				$amount_field_css = 'display: none;';
			}
			ob_start();
			?>
			<div class="sequence-controls__col sequence-controls__col--delay pb-2">
				<div class="js_delay">
					<label for="send_offset_input" class="sequence-controls__col__label"><span class="text-gray-700 font-medium text-opacity-75 text-sm tracking-wide leading-6"><?php echo esc_html__( 'Send ', 'email-subscribers' ); ?></span></label>
					<input value="<?php echo esc_attr( $delay_amount ); ?>" id="send_offset_input" class="delay-amount sequence-controls__offset-input -min" min="1" type="number" required="" name="seq_data[<?php echo esc_attr( $seq_id ); ?>][delay_amount]" style="<?php echo esc_attr( $amount_field_css ); ?>">
					<select name="seq_data[<?php echo esc_attr( $seq_id ); ?>][delay_unit]" id="delay_unit_<?php echo esc_attr( $seq_id ); ?>" class="delay-unit" required>
						<option value="immediately" <?php selected( $delay_unit, 'immediately' ); ?>><?php echo esc_html__( 'immediately', 'email-subscribers' ); ?></option>
						<option value="hour" <?php selected( $delay_unit, 'hour' ); ?>><?php echo esc_html__( 'hour(s)', 'email-subscribers' ); ?></option>
						<option value="day" <?php selected( $delay_unit, 'day' ); ?>><?php echo esc_html__( 'day(s)', 'email-subscribers' ); ?></option>
						<option value="week" <?php selected( $delay_unit, 'week' ); ?>><?php echo esc_html__( 'week(s)', 'email-subscribers' ); ?></option>
					</select>
					<span>
						<?php
						echo esc_html__( ' after subscription', 'email-subscribers' );
						?>
					</span>
				</div>
			</div>

			<?php
			$html = ob_get_clean();

			return $html;
		}

		public function remove_seq_messages_from_list( $query ) {
			$query[] = " type != 'sequence_message'";

			return $query;

		}

		/**
		 * Replace temporary message ids of sequence messages with their actual db ids added through ajax
		 * 
		 * @param array $conditions_data
		 * @param array $message_ids_map
		 * 
		 * @return array $conditions_data
		 * 
		 * @since 4.6.13
		 */
		public function replace_ajax_message_ids_with_db_ids( $conditions_data = array(), $message_ids_map = array() ) {
			if ( ! empty( $conditions_data ) && ! empty( $message_ids_map ) ) {
				$list_conditions = $conditions_data;
				$campaign_rules_data = IG_ES_Campaign_Rules::get_campaign_rules();
				$campaign_fields = array_keys( $campaign_rules_data['Campaign'] );
				foreach ( $list_conditions as $i => $and_cond ) {
					foreach ( $and_cond as $j => $cond ) {
						$condition_field = $list_conditions[ $i ][ $j ]['field'];
						if ( in_array( $condition_field, $campaign_fields ) ) {
							$condition_value = $list_conditions[ $i ][ $j ]['value'];
							if ( is_array( $condition_value ) ) {
								foreach ( $condition_value as $index => $value ) {
									if ( isset( $message_ids_map[ $value ] ) ) {
										$sequence_id = $message_ids_map[ $value ];
										$condition_value[$index] = $sequence_id;
									}
								}
							} else {
								if ( isset( $message_ids_map[ $condition_value ] ) ) {
									$sequence_id = $message_ids_map[ $condition_value ];
									$condition_value[$index] = $sequence_id;
								}
							}
							$list_conditions[ $i ][ $j ]['value'] = $condition_value;
						}
					}
				}
				$conditions_data = $list_conditions;
			}
			return $conditions_data;
		}

		public function get_delay_text( $delay_unit, $delay_amount = 0 ) {
			$delay_text = '';
			if ( ! empty( $delay_unit ) ) {
				switch ( $delay_unit ) {
					case 'hour':
						/* translators: 1. Delay amount 2. delay unit */
						$delay_text = sprintf( __( 'Send after %1$d %2$s', 'email-subscribers' ), $delay_amount, _n( 'hour', 'hours', $delay_amount, 'email-subscribers' ) );
						break;
					case 'day':
						/* translators: 1. Delay amount 2. delay unit */
						$delay_text = sprintf( __( 'Send after %1$d %2$s', 'email-subscribers' ), $delay_amount, _n( 'day', 'days', $delay_amount, 'email-subscribers' ) );
						break;
					case 'week':
						/* translators: 1. Delay amount 2. delay unit */
						$delay_text = sprintf( __( 'Send after %1$d %2$s', 'email-subscribers' ), $delay_amount, _n( 'week', 'weeks', $delay_amount, 'email-subscribers' ) );
						break;
					default:
						$delay_text = __( 'Send immediately', 'email-subscribers' );
						break;
				}
			}
			return $delay_text;
		}
	}
}
