<?php
/**
 * Template for weekly summary email
 * 
 * @var $contacts_growth string Growth of subscribers that compared to previous subscribers count
 * @var $total_subscribed numeric-string Total subscribers count in this interval
 * @var $total_unsubscribed numeric-string Total un-subscribers count in this interval
 * @var $total_sent_mails numeric-string Total sent mails
 * @var $total_opened_mails numeric-string Total mail opened count
 * @var $total_clicked_mails numeric-string Total link clicked count
 * @var $site_name string Site name
 * @var $admin_name string admin name
 * @var $logo_url string Logo image URL
 * @var $dashboard_url string dashboard URL
 * @var $reports_url string reports URL
 * @var $start_date string Start date
 * @var $end_date string end date
 */

$es_wp_styles = wp_styles();
?>
<html>
<?php
$es_wp_styles->do_item( 'google-fonts' );
?>
<style>
	html {
		-moz-tab-size: 4;
		-o-tab-size: 4;
		tab-size: 4;
		line-height: 1.15;
		-webkit-text-size-adjust: 100%;
	}
	body {
		background: #efeeea;
		margin: 0;
		font-family: system-ui, -apple-system, Segoe UI, Roboto, Helvetica, Arial, sans-serif, Apple Color Emoji, Segoe UI Emoji;
	}

	.container {
		background: #FFFFFF;
		border: 1px solid #efeeea;
		max-width: 600px;
		margin: 20px auto;
		padding: 10px;
		border-radius: 5px;
	}

	.center {
		text-align: center;
	}

	.logo-container {
		margin: 20px 10px;
	}

	.heading {
		font-size: 24px;
		margin: 20px auto;
		color: #2a231c;
	}

	.sub-heading {
		color: #433c36;
		font-size: 16px;
	}

	.my-50 {
		margin: 50px auto;
		font-size: 15px;
	}

	.mt-50 {
		margin-top: 50px;
	}

	table {
		width: 100%;
	}

	td {
		padding: 10px;
		text-align: center;
	}

	td .title {
		color: #433c36;
		margin: 10px 0;
		font-size: 15px;
	}

	td .value {
		color: #5850ec;
		font-size: 24px;
		font-weight: 500;
	}

	.button {
		background: #5850ec;
		margin: unset;
		padding: 20px;
		display: inline-block;
		text-decoration: none;
		color: #FFFFFF;
	}

	a {
		color: #5850ec;
		text-decoration: none;
	}

	.m-auto {
		margin: auto;
	}

	.w-95p {
		width: 90%;
	}

	.desc {
		line-height: 25px;
	}
</style>
<body>
	<div class="container">
		<div class="logo-container center">
			<img src="<?php esc_attr_e( $logo_url ); ?>" width="64" alt="Email subscribers logo"/>
		</div>
		<div>
			<p class="center heading"><?php esc_html_e( 'Weekly Account Summary', 'email-subscribers' ); ?></p>
			<p class="center sub-heading"><?php esc_html_e( $start_date ); ?> - <?php esc_html_e( $end_date ); ?></p>
			<p class="center my-50"><?php esc_html_e( 'Here are your weekly stats from', 'email-subscribers' ); ?>
				<b><?php esc_html_e( 'Email Subscribers and Newsletters.', 'email-subscribers' ); ?></b></p>
		</div>
		<div>
			<table>
				<tr>
					<td>
						<div class="value"><?php esc_html_e( $contacts_growth ); ?>%</div>
						<div class="title"><?php esc_html_e( 'Contacts growth', 'email-subscribers' ); ?></div>
					</td>
					<td>
						<div class="value"><?php esc_html_e( $total_subscribed ); ?></div>
						<div class="title"><?php esc_html_e( 'Subscribers', 'email-subscribers' ); ?></div>
					</td>
					<td>
						<div class="value"><?php esc_html_e( $total_unsubscribed ); ?></div>
						<div class="title"><?php esc_html_e( 'Unsubscribes', 'email-subscribers' ); ?></div>
					</td>
				</tr>
				<tr>
					<td>
						<div class="value"><?php esc_html_e( $total_sent_mails ); ?></div>
						<div class="title"><?php esc_html_e( 'Sent Emails', 'email-subscribers' ); ?></div>
					</td>
					<td>
						<div class="value"><?php esc_html_e( $total_opened_mails ); ?></div>
						<div class="title"><?php esc_html_e( 'Opens', 'email-subscribers' ); ?></div>
					</td>
					<td>
						<div class="value"><?php esc_html_e( $total_clicked_mails ); ?></div>
						<div class="title"><?php esc_html_e( 'Clicks', 'email-subscribers' ); ?></div>
					</td>
				</tr>
			</table>
		</div>
		<div class="center mt-50">
			<a href="<?php esc_attr_e( $reports_url ); ?>"
			   class="button"><?php esc_html_e( 'View More Stats', 'email-subscribers' ); ?></a>
		</div>
		<div class="w-95p m-auto">
			<p class="heading center"><?php esc_html_e( 'Boost Your Stats', 'email-subscribers' ); ?></p>
			<p class="center mt-50 desc"><?php esc_html_e( 'Visit our Guides & Tutorials to learn ', 'email-subscribers' ); ?>
				<a href="https://www.icegram.com/unsubscribes/"><?php esc_html_e( 'Why are Your Email Unsubscribes Increasing and How to Fix Them?', 'email-subscribers' ); ?></a>
			</p>
		</div>
	</div>
</body>
</html>
