<?php
/**
 * Action to move contact to the selected list
 *
 * @since       4.4.1
 * @version     1.0
 * @package     Email Subscribers
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to add contact to the selected list
 *
 * @class ES_Action_Move_To_List
 *
 * @since 4.4.1
 */
class ES_Action_Move_To_List extends ES_Workflow_Action {

	/**
	 * Load action admin details.
	 *
	 * @since 4.4.1
	 */
	public function load_admin_details() {
		$this->title = __( 'Move to list', 'email-subscribers' );
		$this->group = __( 'List', 'email-subscribers' );
	}

	/**
	 * Load action fields
	 *
	 * @since 4.4.1
	 */
	public function load_fields() {

		$lists = ES()->lists_db->get_list_id_name_map();

		$list_field = new ES_Select();
		$list_field->set_name( 'ig-es-list' );
		$list_field->set_title( __( 'Select List', 'email-subscribers' ) );
		$list_field->set_options( $lists );
		$list_field->set_required();
		$this->add_field( $list_field );
	}

	/**
	 * Called when an action should be run
	 *
	 * @since 4.4.1
	 */
	public function run() {

		$list_id = $this->get_option( 'ig-es-list' );

		if ( ! $list_id ) {
			return;
		}

		$raw_data = $this->workflow->data_layer()->get_raw_data();
		if ( ! empty( $raw_data ) ) {
			foreach ( $raw_data as $data_type_id => $data_item ) {
				$data_type = ES_Workflow_Data_Types::get( $data_type_id );
				if ( ! $data_type || ! $data_type->validate( $data_item ) ) {
					continue;
				}
				$data = array();
				if ( is_callable( array( $data_type, 'get_data' ) ) ) {
					$data = $data_type->get_data( $data_item );
				}
				if ( ! empty( $data['email'] ) ) {
					$email      = $data['email'];
					$contact_id = ES()->contacts_db->get_contact_id_by_email( $email );
					if ( $contact_id ) {
						$contact_ids = array( $contact_id );
						ES()->lists_contacts_db->move_contacts_to_list( $contact_ids, $list_id );
					}
				}
			}
		}
	}

}
