<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle workflow admin ajax functionality in Pro version
 *
 * @class ES_Pro_Workflow_Admin_Ajax
 *
 * @since 4.7.6
 */
class ES_Pro_Workflow_Admin_Ajax {

	/**
	 * Hook in methods
	 *
	 * @since 4.7.6
	 */
	public static function init() {
		$ajax_events = array(
			'run_workflow',
			'update_workflow_last_ran_at',
		);

		foreach ( $ajax_events as $ajax_event ) {
			add_action( 'wp_ajax_ig_es_' . $ajax_event, array( __CLASS__, $ajax_event ) );
		}
	}


	/**
	 * Run workflow
	 */
	public static function run_workflow() {

		check_ajax_referer( 'ig-es-admin-ajax-nonce', 'security' );

		$response = array();

		$workflow_id = ig_es_get_request_data( 'workflow_id' );

		if ( ! empty( $workflow_id ) ) {

			$workflow = new ES_Workflow( $workflow_id );
			if ( $workflow->exists && $workflow->is_runnable() ) {

				$last_ran_at           = $workflow->get_last_ran_at();
				$last_ran_at_timestamp = strtotime( $last_ran_at );
				
				$trigger_name   = $workflow->get_trigger_name();
				$order_statuses = array();
				$date_args      = array();

				switch ( $trigger_name ) {
					case 'ig_es_wc_order_created':
						$order_statuses = array_keys( wc_get_order_statuses() );
						if ( ! empty( $last_ran_at ) ) {
							$date_args['key']   = 'date_created';
							$date_args['value'] = '>' . $last_ran_at_timestamp;
						}
						break;
					case 'ig_es_wc_order_completed':
						$order_statuses = array( 'wc-completed' );
						if ( ! empty( $last_ran_at ) ) {
							$date_args['key']   = 'date_completed';
							$date_args['value'] = '>' . $last_ran_at_timestamp;
						}
						break;
					case 'ig_es_wc_order_refunded':
						$order_statuses = array( 'wc-refunded' );
						if ( ! empty( $last_ran_at ) ) {
							$date_args['key']   = 'date_refunded';
							$date_args['value'] = $last_ran_at;
						}
						break;
				}

				$list_ids               = array();
				$product_list_enabled   = false;
				$variation_list_enabled = false;
				
				$actions = $workflow->get_actions();
				if ( ! empty( $actions ) ) {
					foreach ( $actions as $action ) {
						$action_name = $action->get_name();
						if ( 'ig_es_add_to_list' === $action_name ) {
							$list_id    = $action->get_option( 'ig-es-list' );
							$list_ids[] = $list_id;
							
							if ( ! $product_list_enabled ) {
								$product_list_enabled = $action->get_option( 'ig-es-product-list-enabled' );
							}
							
							if ( ! $variation_list_enabled ) {
								$variation_list_enabled = $action->get_option( 'ig-es-variation-list-enabled' );
							}
						}
					}
				}

				if ( ! empty( $order_statuses ) && $list_ids ) {
					$response = array(
						'order_statuses'         => $order_statuses,
						'list_ids'               => $list_ids,
						'product_list_enabled'   => $product_list_enabled ? 'yes': 'no',
						'variation_list_enabled' => $variation_list_enabled ? 'yes' : 'no',
						'date_args'				 => $date_args,
					);
				}
			}
		}

		wp_send_json_success( $response );
	}

	/**
	 * Update Workflow last ran at
	 */
	public static function update_workflow_last_ran_at() {

		check_ajax_referer( 'ig-es-admin-ajax-nonce', 'security' );

		$response = array(
			'success' => false,
		);

		$workflow_id = ig_es_get_request_data( 'workflow_id' );

		if ( ! empty( $workflow_id ) ) {

			$workflow = new ES_Workflow( $workflow_id );
			if ( $workflow->exists && $workflow->is_runnable() ) {
				$last_ran_at = $workflow->update_last_ran_at();
				if ( ! empty( $last_ran_at ) ) {
					$last_ran_at = ig_es_format_date_time( $last_ran_at );
				}
				$response['success']     = true;
				$response['last_ran_at'] = $last_ran_at;
			}
		}

		wp_send_json_success( $response );
	}
}
