<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for Product Review Data
 * 
 * @class Data_Type_Review
 * 
 * @since 4.5.3
 */
class ES_Data_Type_Review extends ES_Workflow_Data_Type {

	/**
	 * Validate data
	 * 
	 * @param $item
	 * @return bool
	 * 
	 * @since 4.5.3
	 */
	public function validate( $item ) {
		return $item instanceof WP_Comment;
	}


	/**
	 * Return id from given data item object
	 * 
	 * @param $item
	 * @return mixed
	 * 
	 * @since 4.5.3
	 */
	public function compress( $item ) {
		return $item->comment_ID;
	}


	/**
	 * Return data item object from given id
	 * 
	 * @param $compressed_item
	 * @param $compressed_data_layer
	 * @return \WP_Comment|false
	 * 
	 * @since 4.5.3
	 */
	public function decompress( $compressed_item, $compressed_data_layer ) {
		if ( ! $compressed_item ) {
			return false;
		}

		return get_comment( $compressed_item );
	}

	/**
	 * Return data item object from given data.
	 * 
	 * @param WP_Comment $comment
	 * @return array
	 * 
	 * @since 4.5.3
	 */
	public function get_data( $comment ) {
		
		$data = array();

		if ( $comment instanceof WP_Comment ) {
			$email = ! empty( $comment->comment_author_email ) ? $comment->comment_author_email : '';
			// Found email?
			if ( ! empty( $email ) ) {
				$comment_author = ! empty( $comment->comment_author ) ? $comment->comment_author : '';
				$data           = array(
					'name'   => $comment_author,
					'email'  => $email,
					'source' => 'wc_product_review'
				);
			}
		}

		return $data;
	}
}
