<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle IG_ES_Cart data item
 * 
 * @class IG_ES_Data_Type_Cart
 */
class IG_ES_Data_Type_Cart extends ES_Workflow_Data_Type {

	/**
	 * Validate given data item
	 * 
	 * @param $item
	 * @return bool
	 */
	public function validate( $item ) {
		return is_a( $item, 'IG_ES_Cart');
	}


	/**
	 * Returns id of given data item object. Only validated $items should be passed to this method
	 * 
	 * @param IG_ES_Cart $item
	 * @return mixed
	 */
	public function compress( $item ) {
		return $item->get_id();
	}


	/**
	 * Return data item object from given id.
	 * 
	 * @param $compressed_item
	 * @param $compressed_data_layer
	 * @return mixed
	 */
	public function decompress( $compressed_item, $compressed_data_layer ) {
		if ( ! $compressed_item ) {
			return false;
		}

		$cart = IG_ES_Cart_Factory::get( ES_Clean::id( $compressed_item ) );

		return $cart;
	}

	/**
	 * Abstract required data from data item object
	 *
	 * @since 4.6.5
	 *
	 * @param IG_ES_Customer $item Data item object.
	 * @return array
	 */
	public function get_data( $item ) {

		$data = array();

		if ( $item instanceof IG_ES_Cart ) {

			$cart_link = $item->get_cart_url();

			// prepare data.
			$cart = array(
				'link' => $cart_link,
			);

			$data['cart'] = $cart;
		}

		return $data;
	}

}
