<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle IG_ES_Guest data item
 * 
 * @class IG_ES_Data_Type_Guest
 */
class IG_ES_Data_Type_Guest extends ES_Workflow_Data_Type {

	/**
	 * Validate given data item
	 * 
	 * @param $item
	 * @return bool
	 */
	public function validate( $item ) {
		if ( ! ( $item instanceof IG_ES_Guest ) ) {
			return false;
		}

		if ( ! $item->get_email() ) {
			return false; // social login users may not have an email address defined
		}

		return true;
	}


	/**
	 * Returns id of given data item object. Only validated $items should be passed to this method
	 * 
	 * @param IG_ES_Guest $item
	 * @return mixed
	 */
	public function compress( $item ) {
		return $item->get_email();
	}


	/**
	 * Return data item object from given id.
	 * 
	 * @param $compressed_item
	 * @param $compressed_data_layer
	 * @return mixed
	 */
	public function decompress( $compressed_item, $compressed_data_layer ) {

		// Guests are compressed by their email not their ID
		if ( is_string( $compressed_item ) && is_email( $compressed_item ) ) {
			$guest_email = $compressed_item;
		} else {
			return false;
		}

		$guest_email = ES_Clean::email( $guest_email );
		$guest = IG_ES_Guest_Factory::get_by_email( $guest_email );

		if ( ! $guest ) {
			// still pass the guest object even if it doesn't exist in the database
			// In most cases it should have been stored but there is no harm since all we need is an email
			$guest = new IG_ES_Guest();
			$guest->set_email( $guest_email );
		}

		return $guest;
	}

	/**
	 * Abstract required data from data item object
	 *
	 * @since 4.6.5
	 *
	 * @param IG_ES_Guest $item Data item object.
	 * @return array
	 */
	public function get_data( $item ) {

		$data = array();

		if ( $item instanceof IG_ES_Guest ) {

			$email            = $item->get_email();
			$guest_first_name = $item->get_first_name();
			$guest_last_name  = $item->get_last_name();

			// prepare data.
			$data = array(
				'first_name' => $guest_first_name,
				'last_name'  => $guest_last_name,
				'email'      => $email,
				'source'     => 'woocommerce-abandoned-cart',
				'status'     => 'verified',
				'hash'       => ES_Common::generate_guid(),
				'created_at' => ig_get_current_date_time(),
			);
		}

		return $data;
	}

}
