<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Abstract class Abandon cart
 * 
 * @class IG_ES_Trigger_Abstract_Abandoned_Cart
 */
abstract class IG_ES_Trigger_Abstract_Abandoned_Cart extends ES_Workflow_Trigger {

	/**
	 * Load action admin details.
	 *
	 */
	public function load_admin_details() {
		$this->description .= ' ' . sprintf(
			/* translators: %s: Cart abandoned timeout */
			__( 'Carts are considered abandoned if they are inactive for %d minutes.', 'email-subscribers' ),
			IG_ES_WC_Carts::get_cart_abandoned_timeout()
		);

		$this->group = __( 'Carts', 'email-subscribers' );
	}


	/**
	 * Register trigger hooks.
	 */
	public function register_hooks() {
		add_action( 'ig_es_wc_cart_status_changed', array( $this, 'status_changed' ), 10, 3 );
	}


	/**
	 * Handle cart status change event
	 * 
	 * @param IG_ES_Cart $cart
	 * @param string $old_status
	 * @param string $new_status
	 */
	public function status_changed( $cart, $old_status, $new_status ) {
		if ( 'abandoned' === $new_status ) {
			$this->cart_abandoned( $cart );
		}
	}


	/**
	 * Handle cart abandoned event
	 * 
	 * @param IG_ES_Cart $cart
	 */
	public function cart_abandoned( $cart ) {

		if ( ! $cart->has_items() ) {
			return;
		}

		$cart_data = array(
			'cart' => $cart,
		);

		if ( $cart->get_user_id() ) {
			$customer              = $cart->get_customer();
			$cart_data['customer'] = $customer;
		} else {
			$guest                 = $cart->get_guest();
			$cart_data['guest']    = $guest;
		}

		$this->maybe_run(
			$cart_data
		);
	}

	/**
	 * Validate workflow
	 * 
	 * @param $workflow ES_Workflow
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {

		$cart = $workflow->data_layer()->get_cart();

		if ( ! $cart ) {
			return false;
		}

		return true;
	}

}
