<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/***
 * ES_Trigger_WC_Order_Refunded class.
 *
 * @since 4.5.3
 */
class ES_Trigger_WC_Order_Refunded extends ES_Workflow_Trigger {

	/**
	 * Declares data items available in trigger.
	 *
	 * @var array
	 * 
	 * @since 4.5.3 
	 */
	public $supplied_data_items = array( 'wc_order' );

	/**
	 * Load trigger admin props.
	 * 
	 * @since 4.5.3
	 */
	public function load_admin_details() {
		$this->title       = __( 'WooCommerce Order Refunded', 'email-subscribers' );
		$this->description = __( 'Fires whenever WooCommerce order gets refunded.', 'email-subscribers' );
		$this->group 	   = __( 'Order', 'email-subscribers' );
	}

	/**
	 * Register trigger hooks.
	 * 
	 * @since 4.5.3
	 */
	public function register_hooks() {
		add_action( 'woocommerce_order_status_changed', array( $this, 'handle_order_status_changed' ), 10, 4 );
	}

	/**
	 * Catch WooCommerce order status changed hook
	 *
	 * @param $order_id
	 * @param $old_status
	 * @param $new_status
	 * @param WC_Order $wc_order WooCommerce order object
	 * 
	 * @since 4.5.3
	 */
	public function handle_order_status_changed( $order_id, $old_status, $new_status, $wc_order = null ) {

		if ( 'refunded' !== $old_status && 'refunded' === $new_status ) {

			if ( $wc_order instanceof WC_Order ) {

				$data = array(
					'wc_order' => $wc_order,
				);

				$this->maybe_run( $data );
			}
		}
	}

	/**
	 * Validate a workflow.
	 *
	 * @param ES_Workflow $workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {

		$wc_order = $workflow->data_layer()->get_item( 'wc_order' );

		if ( ! $wc_order || ! $wc_order instanceof WC_Order ) {
			return false;
		}

		return true;
	}

}
