<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle cart abandonment for guest customers
 * 
 * @class IG_ES_Trigger_WC_Guest_Cart_Abandoned
 */
class IG_ES_Trigger_WC_Guest_Cart_Abandoned extends IG_ES_Trigger_Abstract_Abandoned_Cart {

	public $supplied_data_items = array( 'cart', 'guest', 'coupon', 'shop' );

	/**
	 * Load action admin details.
	 *
	 */
	public function load_admin_details() {
		$this->title = __( 'Cart Abandoned - Guests Only', 'email-subscribers' );
		$this->description = __( 'This trigger fires when a cart belonging to a guest customer is abandoned.', 'email-subscribers' );
		parent::load_admin_details();
	}


	/**
	 * Handle cart abandoned event
	 * 
	 * @param IG_ES_Cart $cart
	 */
	public function cart_abandoned( $cart ) {
		$guest = $cart->get_guest();
		if ( ! $cart->get_user_id() && $cart->has_items() ) {
			$this->maybe_run(array(
				'guest' => $guest,
				'cart' => $cart
			));
		}
	}

	/**
	 * Validate workflow
	 * 
	 * @param $workflow Workflow
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {
		$cart = $workflow->data_layer()->get_cart();

		if ( $cart->get_user_id() ) {
			return false;
		}

		return parent::validate_workflow( $workflow );
	}

}
