<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle cart abandonment for registered customers
 * 
 * @class IG_ES_Trigger_WC_User_Cart_Abandoned
 */
class IG_ES_Trigger_WC_User_Cart_Abandoned extends IG_ES_Trigger_Abstract_Abandoned_Cart {

	public $supplied_data_items = array( 'cart', 'customer', 'coupon', 'shop' );

	/**
	 * Load action admin details.
	 */
	public function load_admin_details() {
		$this->title = __( 'Cart Abandoned - Registered Users Only', 'email-subscribers' );
		$this->description = __( 'This trigger fires when a cart belonging to a registered customer is abandoned.', 'email-subscribers' );
		parent::load_admin_details();
	}


	/**
	 * Handle cart abandoned event
	 * 
	 * @param IG_ES_Cart $cart
	 */
	public function cart_abandoned( $cart ) {
		if ( $cart->get_user_id() ) {
			parent::cart_abandoned( $cart );
		}
	}

	/**
	 * Validate workflow
	 * 
	 * @param $workflow Workflow
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {
		$customer = $workflow->data_layer()->get_customer();
		$cart = $workflow->data_layer()->get_cart();

		if ( ! $customer || ! $cart ) {
			return false;
		}

		if ( ! $cart->get_user_id() ) {
			return false;
		}

		return parent::validate_workflow( $workflow );
	}


}
