<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to handle 'ig_es_wc_wishlist_products_gone_on_sale' event
 * 
 */
class IG_ES_Trigger_Wishlist_Item_Goes_On_Sale extends ES_Workflow_Trigger {

	public $supplied_data_items = array( 'customer', 'wishlist', 'coupon', 'shop' );

	public function load_admin_details() {
		/* translators: %s Title of wishlist plugin integrated */
		$this->title       = sprintf( __( 'Wishlist Item On Sale (%s)', 'email-subscribers' ), IG_ES_Wishlists::get_integration_title() );
		$this->group       = __( 'Wishlists', 'email-subscribers' );
		$this->description = __(
			'This trigger doesn\'t fire instantly when a product goes on sale. Instead, it performs a check for new on-sale products in every 15 minutes. 
			 Please note this doesn\'t work for guests because their wishlist data only exists in their session data.',
			'email-subscribers' );
	}

	public function register_hooks() {
		$integration = IG_ES_Wishlists::get_integration();

		if ( ! $integration ) {
			return;
		}

		add_action( 'ig_es_wc_wishlist_products_gone_on_sale', array( $this, 'handle_wishlist_products_gone_on_sale' ), 10, 2 );
	}


	/**
	 * Handler wishlist products goes on sale events
	 * 
	 * @param array $products_ids
	 */
	public function handle_wishlist_products_gone_on_sale( $wishlist, $customer ) {
		
		$data = array(
			'wishlist' => $wishlist,
			'customer' => $customer,
		);

		$this->maybe_run( $data );
	}

	/**
	 * Validate a workflow.
	 *
	 * @param ES_Workflow $workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow ) {

		$wishlist = $workflow->data_layer()->get_item( 'wishlist' );
		$customer = $workflow->data_layer()->get_customer();

		if ( ! $wishlist || ! $wishlist instanceof IG_ES_Wishlist || ! $customer ) {
			return false;
		}


		return true;
	}
}

