<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Variable class to get value for {{cart.items}} placeholder
 */
class IG_ES_Variable_Cart_Items extends IG_ES_Variable_Abstract_Product_Display {

	public $supports_cart_table = true;

	public function load_admin_details() {
		parent::load_admin_details();
		$this->description = __( 'Display a product listing of the items in the cart.', 'email-subscribers' );
	}


	/**
	 * Get cart items HTML
	 * 
	 * @param $cart Cart
	 * @param $parameters array
	 * @param $workflow Workflow
	 * @return mixed
	 */
	public function get_value( $cart, $parameters, $workflow ) {

		$cart_items = $cart->get_items();
		$template = isset( $parameters['template'] ) ? $parameters['template'] : false;

		$products    = array();
		$product_ids = array();

		if ( empty( $cart_items ) ) {
			return false;
		}

		foreach ( $cart_items as $item ) {
			$variation_id = $item->get_variation_id();
			$product_id   = $item->get_product_id();
			if ( $variation_id ) {
				$product_ids[] = $variation_id;
			} elseif ( $product_id ) {
				$product_ids[] = $product_id;
			}
		}

		$product_ids = array_unique( $product_ids );

		foreach ( $product_ids as $product_id ) {
			$products[] = wc_get_product( $product_id );
		}

		$args = array_merge( $this->get_default_product_template_args( $workflow, $parameters ), [
			'products'   => $products,
			'cart_items' => $cart->get_items_raw(), // legacy
			'cart'       => $cart,
		]);

		return $this->get_product_display_html( $template, $args );
	}
}

return new IG_ES_Variable_Cart_Items();
