<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Variable class to get value for {{coupon.generate_unique_coupon}} placeholder
 */
class IG_ES_Variable_Coupon_Generate_Unique_Coupon extends IG_ES_Workflow_Variable {

	/**
	 * Method to set description and other admin props
	 *
	 */
	public function load_admin_details() {
		$this->description = __( 'Generates a unique WooCommerce coupon based on a template coupon', 'email-subscribers' );
		$field             = new ES_Select();
		$field->set_name( 'template' );
		$field->set_required( true );
		$field->set_placeholder( false );
		$field->set_description( __( 'Name of the coupon that will cloned. NOTE: Don\'t delete the coupon code in WooCommerce dashboard', 'email-subscribers' ) );
		$field->add_classes( 'wc-product-search' );
		$field->set_id( 'wc-coupon-template-ids' );
		$field->add_extra_attr( 'data-placeholder', __( 'Search coupon template', 'email-subscribers' ) );
		$field->add_extra_attr( 'data-action', 'ig_es_json_search_woocommerce_coupons' );
		$field->add_extra_attr( 'data-required', 'yes' );
		$field->add_extra_attr( 'style', 'width:100%;' );
		$this->add_parameter_field( $field );
		//Coupon expiry days field
		$this->add_parameter_number_field( 'expires', __( 'The number of days the coupon will be valid for. Leave this field blank or put \'0\' to generate a coupon without an expiry date.', 'email-subscribers' ), false, '' );
		//Coupon prefix field
		$this->add_parameter_text_field( 'prefix', __( 'The prefix for the coupon code. To remove the prefix, leave this field blank.', 'email-subscribers' ), false, 'es-' );
		//Coupon usage limit field
		$this->add_parameter_number_field( 'limit', __( 'The number of times the generated coupon can be used. Leave this field blank or put \'0\' for an unlimited number of coupon usage.', 'email-subscribers' ), false, '1' );
		$this->use_fallback = false;
		$this->render_wc_coupon_input_field();
	}

	public function render_wc_coupon_input_field() {
		?>
		<script type="text/javascript">
			jQuery('body').trigger('wc-enhanced-select-init');
		</script>
		<?php
	}

	/**
	 * Check that the coupon code is generated previously for keyword
	 *
	 * @param $parameters
	 * @param $previously_generated_coupons
	 *
	 * @return int|string|null
	 */
	public function get_existing_coupon_for_keyword( $parameters, $previously_generated_coupons ) {
		if ( is_array( $previously_generated_coupons ) && ! empty( $previously_generated_coupons ) ) {
			foreach ( $previously_generated_coupons as $coupon_code => $generated_coupon_parameters ) {
				if ( ig_es_is_arrays_are_equal( $generated_coupon_parameters, $parameters ) ) {
					return $coupon_code;
				}
			}
		}

		return null;
	}

	/**
	 * Generate new coupon code and return the code
	 *
	 * @param $parameters
	 * @param ES_Workflow $workflow
	 *
	 * @return string
	 */
	public function get_value( $parameters, $workflow ) {
		if ( is_array( $parameters ) && isset( $parameters['template'] ) && ! empty( $parameters['template'] ) ) {
			$parameters = wp_parse_args( $parameters, array(
				'limit'   => '',
				'expires' => '0',
				'prefix'  => ''
			) );

			$es_coupon  = new IG_ES_Coupon();
			if ( $workflow->preview_mode ) {
				return $es_coupon->get_coupon_code_for_preview( $parameters );
			}

			$previously_generated_coupons = isset( $workflow::$extra['previously_generated_coupons'] ) ? $workflow::$extra['previously_generated_coupons'] : array();
			$existing_coupon_code         = $this->get_existing_coupon_for_keyword( $parameters, $previously_generated_coupons );
			if ( is_null( $existing_coupon_code ) ) {
				$template_coupon = $parameters['template'];
				$new_coupon      = $es_coupon->create_coupon_from_template( $template_coupon, $parameters );
				if ( $new_coupon instanceof WC_Coupon ) {
					$new_coupon_code                                  = $new_coupon->get_code();
					$previously_generated_coupons[ $new_coupon_code ] = $parameters;
					$workflow::$extra['previously_generated_coupons'] = $previously_generated_coupons;

					return $new_coupon_code;
				}
			} else {
				return $existing_coupon_code;
			}
		}

		return '';
	}

}

return new IG_ES_Variable_Coupon_Generate_Unique_Coupon();
